# Copyright 2013 Sim Ayers
# version 1.5
=begin
Fixed the dihedral angle drawing

Fixed the Trapezoidal Timber Hip Rafter plugin to draw a hip rafter with a 90 eave angle
Add Tetrahedron Slice
Added Draw Down Technique
=end
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------

=begin
$	 global variable
@	 instance variable
[a-z] or _	 local variable
[A-Z]	 constant
=end


require 'sketchup.rb'


# Show the Ruby Console at startup so we can
# see any programming errors we may make.
#Sketchup.send_action "showRubyPanel:"



module ROOFFRAMING

class TetrahedronTool

#global variables in the class

$pitch_angle = 33.6907;
$plan_angle = 45.0;
$eave_length = 12;
$show_angles = "no";
$folded = "yes";
$dihedral = "yes";
$unit_circle= "no";
=begin
Set $DEBUG_VERBOSE = true 
when you want to debug the script, the puts statement will output debug info to the Ruby Console Window
=end

$DEBUG_VERBOSE = false
  
def initialize
  
    
    $pitch_angle = 33.6907 if not $pitch_angle;
    $plan_angle = 45.0 if not $plan_angle;
    $eave_length = 12 if not $eave_length;
    $show_angles = "no" if not $show_angles;
    $folded = "yes" if not $folded;
    $dihedral = "yes" if not $dihedral;
   $unit_circle = "no" if not $unit_circle;
    # First prompt for the dimensions.  This is done using the inputbox method
        
    # The first step is to create some arrays which contain the prompts
    # and default values.
    prompts = ["Rafter Pitch Angle","Plan Angle","Eave Length",
    "Dim Angles","Folded","Show Dihedral Angle","Unit Circle"]
    #default values
    values = [$pitch_angle,$plan_angle,$eave_length,$show_angles,$folded,$dihedral,$unit_circle]
    
     popups = [nil, nil, nil, "yes|no","yes|no|Backing Angle|Both","yes|no","no|yes"]
    
    # Now display the inputbox
    results = inputbox prompts, values,popups, "Teterahedron Angles";

    return if not results # This means that the user cancelled the operation
    $pitch_angle,$plan_angle,$eave_length,$show_angles,$folded,$dihedral,$unit_circle = results
    
    #$eave_length.to_f.to_l;
    
   # $eave_length2 =  $eave_length.to_s; 
    
    if ($pitch_angle <= 0.0 )
     	UI.messagebox("Rafter Pitch Angle can not be 0.")
     	return
    end
    
    if ($eave_length <= 0.0 )
     	UI.messagebox("Eave Length can not be 0.")
     	return
    end
    if ($plan_angle >= 180.0 )
     	UI.messagebox("Plan Angle can not be greater 180.")
     	return
    end
    if ($plan_angle <= 0.0 )
     	UI.messagebox("Plan Angle can not be 0.")
     	return
    end
    show = true
    show = false if($show_angles == "no")
    backing = false
    backing = true if($folded == "Backing Angle")
    backing = false if($folded == "Both")
    both = false
    both =  true if($folded == "Both");
    show_dihedral = false
    show_dihedral = true if($dihedral =="yes");
    slice = false;
    show_unit_circle = false
    show_unit_circle = true if($unit_circle =="yes");
    
    
    
    p1 = [0,0,0]
    if($folded == "no" || $folded == "Backing Angle" || $folded == "Both")
    	draw_tetrahedron_unfolded(p1,$plan_angle,$pitch_angle,$eave_length,show,backing,show_unit_circle); 
    	draw_tetrahedron(p1,$plan_angle,$pitch_angle,$eave_length,show,both,show_dihedral,slice,show_unit_circle) if $folded == "Both";
    else
    	draw_tetrahedron(p1,$plan_angle,$pitch_angle,$eave_length,show,both,show_dihedral,slice,show_unit_circle) 
    end
    
     
    
end


# draw a tetrahedron folded
def draw_tetrahedron(p1,d,s,eave_length,show_angles,both,show_dihedral,slice,show_unit_circle)

    
    model = Sketchup.active_model   
    model.start_operation "Create Tetrahedron";
    
    model_ents = model.active_entities
    group = model_ents.add_group
    ents = group.entities
    group.name="Create Tetrahedron"
    
       
	wplayer=model.layers.add("Tetrahedron")
	wplayer.visible=true
	group.layer=wplayer
	group.casts_shadows=false
	group.receives_shadows=false

 		
	dd = d.degrees # plan angle
	ss = s.degrees # roof pitch angle
	r1 = Math.atan(Math.tan(ss) * Math.sin(dd)); # hip rafter angle
	ee = Math.atan(Math.cos(ss) / Math.tan(dd)); # jack rafter angle
	bb = Math.atan(Math.sin(r1) / Math.tan(dd)); # hip rafter backing angle
  
  if show_unit_circle    
      eave_length = Math.sin(ee);      
      
    end
    
  run = (eave_length * Math.tan(dd)); # common rafter run
	rise = run * Math.tan(ss); # common rafter rise
	
	radius = eave_length/3; #radius used for angluar text position
	
	puts "dd " + dd.radians.to_s if $DEBUG_VERBOSE; 
	puts "ss " + ss.radians.to_s if $DEBUG_VERBOSE; 
	puts "r1 " + r1.radians.to_s if $DEBUG_VERBOSE; 
	puts "show angles " + show_angles.to_s if $DEBUG_VERBOSE; 
	
	# draw plan angle
	pts = []
	pts[0] = [p1.x,p1.y,0]
	pts[1] = [ p1.x + eave_length,p1.y ,0 ]
	pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
	pts[3] = pts[0]
	new_rafter = ents.add_face pts
	dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles && !both ;
	dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles && !both ;

	# draw common rafter pitch
	pts = []
	pts[0] = [ p1.x + eave_length,p1.y ,0 ]
	pts[1] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
	pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , rise]
	pts[3] = pts[0]
	new_rafter = ents.add_face pts
	dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
	dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;

	# draw hip rafter
	pts = []
	pts[0] = [p1.x,p1.y,0]
	pts[1] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
	pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , rise]
	pts[3] =  pts[0]
  hip_run = pts[0].distance pts[1]
	new_rafter = ents.add_face pts
	dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
	dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;

	# draw roof sheathing
	pts = []
	pts[0] = [p1.x,p1.y,0]
	pts[1] = [ p1.x + eave_length ,p1.y , 0]
	pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , rise]
	pts[3] = pts[0]
	new_rafter = ents.add_face pts
	dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
	dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;


  pts = draw_dihedral_angle_folded(dd,ss,r1,bb,hip_run,eave_length,p1.x,p1.y,ents,show_angles,radius,slice) if show_dihedral;

    Sketchup.active_model.rendering_options["ModelTransparency"]=true 
    view = Sketchup.active_model.active_view 
    view.field_of_view = 30
    #result = Sketchup.send_action 10501; # set view to Top
    view.zoom_extents 

    group.to_component 
    model.commit_operation
    
end	
# draw a dihedral angle folded
def draw_dihedral_angle_folded(dd,ss,r1,ba,hip_length,eave_length,originx,originy,ents,show_angles,radius,slice)
    angle = 90.degrees - dd
    leg_hypot = eave_length * Math.cos(angle);
    leg_rise = leg_hypot * Math.sin(angle);
    leg_run = leg_hypot * Math.cos(angle);
    backing_run = eave_length - leg_run
    hip_altitude_run = backing_run / Math.cos(dd);
    hip_altitude = hip_altitude_run * Math.sin(r1);
    hip_altitude_leg = hip_altitude / Math.tan(r1);
    
    hip_rise = hip_altitude_leg * Math.sin(r1);
    hip_run = hip_rise / Math.tan(r1);
    point_x = hip_run * Math.cos(dd);
    point_y = hip_run * Math.sin(dd);
    
    pts_org = [originx,originy,0]
      
    pts1 = [originx+eave_length,originy,0]
    pts3 = [originx+backing_run,originy+leg_rise,0]
    new_rafter = ents.add_line pts1, pts3
    
    dimension_angle(pts_org ,pts3,pts1,radius/2, ents ) if show_angles;
    
    pts1a = [originx+backing_run,originy+leg_rise,0]
    pts2 = [originx+point_x,originy+point_y,hip_rise]
    new_rafter = ents.add_line pts1a, pts2
   dimension_angle(pts1 ,pts3,pts2,radius/2, ents ) if show_angles;
 
    pts1 = [originx+point_x,originy+point_y,hip_rise]
    pts2 = [originx+eave_length,originy,0]
    new_rafter = ents.add_line pts1, pts2

    dimension_angle(pts1,pts2,pts3,radius, ents ) if show_angles;
    
    if (slice)
         dihedral_angle = dimension_angle_test(pts1,pts2,pts3);
        dihedral_angle = dihedral_angle.degrees;
         puts "dihedral angle  " + dihedral_angle.radians.to_s if $DEBUG_VERBOSE;    
        angle_text = "Dihedral Angle = " + dihedral_angle.radians.to_s ;
        angle_text += "\nSaw Blade Bevel Angle";
        vector = Geom::Vector3d.new(0,0,1);
        vector2 = vector.normalize!
        t = ents.add_text(angle_text, pts2,vector2);
        t.leader_type = 1
    end 

    puts "backing_run " + backing_run.to_s if $DEBUG_VERBOSE; 
    puts "leg_run " + leg_run.to_s if $DEBUG_VERBOSE; 
    puts "leg_rise " + leg_rise.to_s if $DEBUG_VERBOSE; 
    puts "hip_altitude_run  " + hip_altitude_run.to_s if $DEBUG_VERBOSE; 
    puts "hip_altitude  " + hip_altitude.to_s if $DEBUG_VERBOSE; 
    puts "hip_rise  " + hip_rise.to_s if $DEBUG_VERBOSE; 
    puts "hip_run  " + hip_run.to_s if $DEBUG_VERBOSE; 
    pts =[];
    pts[0] = pts1;
    pts[1] = pts2;
    pts[2] = pts3;
    return pts;
   
end

# draw tetrahedron unfolded
def draw_tetrahedron_unfolded(p1,d,s,eave_length,show_angles,backing,show_unit_circle)

    
    model = Sketchup.active_model   
    model.start_operation "Create Tetrahedron";

    model_ents = model.active_entities
    group = model_ents.add_group
    ents = group.entities
    group.name="Create Tetrahedron"
    wplayer=model.layers.add("Tetrahedron")
    wplayer.visible=true
    group.layer=wplayer
    group.casts_shadows=false
    group.receives_shadows=false

 		
    dd = d.degrees
    ss = s.degrees
    r1 = Math.atan(Math.tan(ss) * Math.sin(dd));
    ee = Math.atan(Math.cos(ss) / Math.tan(dd));
    sheathing_angle = 90.degrees - ee;
    bb = Math.atan(Math.sin(r1) / Math.tan(dd));
    if show_unit_circle
        eave_length = Math.sin(ee);  
    end
    
    run = (eave_length * Math.tan(dd))
    rise = run * Math.tan(ss)
    hip_run = (eave_length / Math.cos(dd))
    
    hip_rafter = hip_run / Math.cos(r1)
    common_rafter = run / Math.cos(ss)
    alitude = hip_run * Math.sin(r1)
    r2 = r1 + dd
    d2 = 90.degrees - dd
    backing_leg = alitude + hip_run
    backing_run = run / Math.tan(d2);

    radius = eave_length/3

    puts "dd " + dd.radians.to_s if $DEBUG_VERBOSE; 
    puts "ss " + ss.radians.to_s if $DEBUG_VERBOSE; 
    puts "r1 " + r1.radians.to_s if $DEBUG_VERBOSE; 
    puts "show angles " + show_angles.to_s if $DEBUG_VERBOSE; 

    # plan angle
    pts = []
    pts[0] = [p1.x,p1.y,0]
    pts[1] = [ p1.x + eave_length,p1.y ,0 ]
    pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;

    #rafter pitch
    pts = []
    pts[0] = [ p1.x + eave_length,p1.y ,0 ]
    pts[1] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[2] = [ p1.x + eave_length + rise,p1.y + (eave_length * Math.tan(dd)),0]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
    pt1 = 
    # hip rafter
    pts = []
    pts[0] = [p1.x,p1.y,0]
    pts[1] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[2] = [ p1.x + (hip_rafter * Math.cos(r2)) ,p1.y + (hip_rafter * Math.sin(r2)) , 0]
    pts[3] =  pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;


    # roof sheathing
    pts = []
    pts[0] = [p1.x,p1.y,0]
    pts[1] = [ p1.x + eave_length ,p1.y , 0]
    pts[2] = [ p1.x + eave_length ,p1.y - common_rafter,0]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
	
	if backing
      # backing triangle
      pts = []
      pts[0] = [p1.x + eave_length + backing_run,p1.y,0]
      pts[1] = [ p1.x + eave_length ,p1.y + run, 0]
      pts[2] = [ p1.x + (backing_leg * Math.cos(dd)) ,p1.y + (backing_leg * Math.sin(dd)),0]
      pts[3] = pts[0]
      new_rafter = ents.add_face pts
      dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
      dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
      Sketchup.active_model.rendering_options["ModelTransparency"]=true  
	end
	
    view = Sketchup.active_model.active_view 
    fov = Sketchup.active_model.active_view.field_of_view
    view.zoom_extents 
    puts "view angle " + fov.to_s if $DEBUG_VERBOSE; 
    #view.field_of_view = 30
    group.to_component 
    model.commit_operation
    
  end	
  
  def draw_slice(p1,miter_angle,bevel_angle,rafter_width,show_angles,radius,ents)
    
      edge_AC = rafter_width / Math.tan(bevel_angle);
      edge_AB = rafter_width / Math.sin(bevel_angle);
      edge_CE = edge_AC / Math.tan(miter_angle);
      angle_CEB = Math.atan(rafter_width/edge_CE);
      edge_EB = rafter_width / Math.sin(angle_CEB);
      edge_AE = edge_CE / Math.cos(miter_angle);
      angles = [0,0,0];
      #angles = [angle_b,angle_a,angle_c];
      angles = get_triangle_SSS(edge_AB,edge_EB,edge_AE)
      angle_b = angles[0];
      angle_a = angles[1];
      angle_c = angles[2];

      puts "angle_b  " + angle_b.radians.to_s if $DEBUG_VERBOSE; 
      puts "angle_a  " + angle_a.radians.to_s if $DEBUG_VERBOSE; 
      puts "angle_c  " + angle_c.radians.to_s if $DEBUG_VERBOSE; 
      puts "angle_CEB  " + angle_CEB.radians.to_s if $DEBUG_VERBOSE;    

      point_E =  [p1.x,p1.y,0];
      point_B = [p1.x+rafter_width,p1.y,edge_CE];
      point_C = [p1.x,p1.y,edge_CE];
      point_A = [p1.x,p1.y+edge_AC,edge_CE];
  
      # draw face A
      pts = []
      pts[0] = point_E;
      pts[1] = point_B
      pts[2] = point_C;
      pts[3] = pts[0]
      new_rafter = ents.add_face pts
       status = new_rafter.back_material=[153,107,91]       
      new_rafter.material= Sketchup::Color.new(153,107,91)
      dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
      #dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
      #dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;

      # draw face C
      pts = []
      pts[0] = point_E;
      pts[1] = point_C;
      pts[2] = point_A
      pts[3] = pts[0]
      new_rafter = ents.add_face pts
       status = new_rafter.back_material=[153,107,91]       
      new_rafter.material= Sketchup::Color.new(153,107,91)
      dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
      #dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
      #dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;


      # draw face D
      pts = []
      pts[0] = point_A;
      pts[1] = point_C;
      pts[2] = point_B
      pts[3] = pts[0]
      new_rafter = ents.add_face pts
       status = new_rafter.back_material=[153,107,91]       
      new_rafter.material= Sketchup::Color.new(153,107,91)
      dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
      #dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
      #dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;
 
      # draw face B
      pts = []
      pts[0] = point_A;
      pts[1] = point_B;
      pts[2] = point_E
      pts[3] = pts[0]
      new_rafter = ents.add_face pts
       status = new_rafter.back_material=[153,107,91]       
      new_rafter.material= Sketchup::Color.new(153,107,91)
      #dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
      #dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
      #dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;
 
  end
  
  def draw_slice_unfolded(p1,miter_angle,bevel_angle,rafter_width,show_angles,radius,ents)
    
      edge_AC = rafter_width / Math.tan(bevel_angle);
      edge_AB = rafter_width / Math.sin(bevel_angle);
      edge_CE = edge_AC / Math.tan(miter_angle);
      angle_CEB = Math.atan(rafter_width/edge_CE);
      edge_EB = rafter_width / Math.sin(angle_CEB);
      edge_AE = edge_CE / Math.cos(miter_angle);
      angles = [0,0,0];
      #angles = [angle_b,angle_a,angle_c];
      angles = get_triangle_SSS(edge_AB,edge_EB,edge_AE)
      angle_b = angles[0];
      angle_a = angles[1];
      angle_c = angles[2];

      puts "angle_b  " + angle_b.radians.to_s if $DEBUG_VERBOSE; 
      puts "angle_a  " + angle_a.radians.to_s if $DEBUG_VERBOSE; 
      puts "angle_c  " + angle_c.radians.to_s if $DEBUG_VERBOSE; 
      puts "angle_CEB  " + angle_CEB.radians.to_s if $DEBUG_VERBOSE;    


      point_C = [p1.x,p1.y,0];
      point_EC = [p1.x,p1.y-edge_CE,0];
      point_B = [p1.x+rafter_width,p1.y,0];
      point_A = [p1.x,p1.y+edge_AC,0];
      point_EA = [p1.x-edge_CE,p1.y,0];

      point_EB = [p1.x-edge_CE,p1.y,0];

      angle_ABC = dimension_angle_test(point_A,point_B,point_C);
      angle_ABC = angle_ABC.degrees;
      puts "angle_ABC  " + angle_ABC.radians.to_s if $DEBUG_VERBOSE;    
      rotate = 180.degrees -  angle_ABC - angle_c;
      rotate_x = edge_EB * Math.cos(rotate);
      rotate_y = edge_EB * Math.sin(rotate);
      point_BE = [p1.x+rafter_width+rotate_x,p1.y+rotate_y,0];

      # draw face A
      pts = []
      pts[0] = point_C;
      pts[1] = point_EC
      pts[2] = point_B;
      pts[3] = pts[0]
      new_rafter = ents.add_face pts
      status = new_rafter.back_material=[153,107,91]       
      new_rafter.material= Sketchup::Color.new(153,107,91)
 
      #dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
      #dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
      dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;

      # draw face C
      pts = []
      pts[0] = point_C;
      pts[1] = point_EA;
      pts[2] = point_A
      pts[3] = pts[0]
      new_rafter = ents.add_face pts
      status = new_rafter.back_material=[153,107,91]       
      new_rafter.material= Sketchup::Color.new(153,107,91)
 
      #dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
      #dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
      dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;


      # draw face D
      pts = []
      pts[0] = point_A;
      pts[1] = point_C;
      pts[2] = point_B
      pts[3] = pts[0]
      new_rafter = ents.add_face pts
      status = new_rafter.back_material=[153,107,91]       
      new_rafter.material= Sketchup::Color.new(153,107,91)
 
      dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
      #dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
      #dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;
 
      # draw face B
      pts = []
      pts[0] = point_A;
      pts[1] = point_B;
      pts[2] = point_BE
      pts[3] = pts[0]
      new_rafter = ents.add_face pts
      status = new_rafter.back_material=[153,107,91]       
      new_rafter.material= Sketchup::Color.new(153,107,91)
 
      #dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
      #dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
      #dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;
      
          
      new_rafter = ents.add_line point_BE,point_C

      angle_ECB = dimension_angle_test(point_BE,point_C,point_B);
      angle_ECB = angle_ECB.degrees;
      puts "angle_ECB  " + angle_ECB.radians.to_s if $DEBUG_VERBOSE;   

      rotate = angle_ECB + 90.degrees;
      rotate_x = edge_CE * Math.cos(rotate);
      rotate_y = edge_CE * Math.sin(rotate);
      point_BBE = [p1.x+rotate_x,p1.y+rotate_y,0];

      new_rafter = ents.add_line point_BBE,point_C

      rafter_run = rafter_width * Math.cos(bevel_angle);
      rafter_run_x = rafter_run * Math.cos(bevel_angle);
      rafter_run_y = rafter_run * Math.sin(bevel_angle);
      point_D = [p1.x+rafter_run_x,p1.y+rafter_run_y,0];

      new_rafter = ents.add_line point_BBE,point_D
      pts = []
      pts[0] = point_C;
      pts[1] = point_BBE;
      pts[2] = point_D
      pts[3] = pts[0]
      new_rafter = ents.add_face pts
      new_rafter.material= Sketchup::Color.new(204,00,0)
      dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;
     
     dimension_angle(point_D,point_BE,point_B,radius, ents ) if show_angles;
    dimension_angle(point_D,point_BE,point_A,radius, ents ) if show_angles;
    
      angle = dimension_angle_test(point_BBE,point_D,point_C);
      angle = angle.degrees;
      puts "return angle  " + angle.radians.to_s if $DEBUG_VERBOSE;   

      return angle.radians;
 
  end
  
  
  
    
# draw a dihedral angle unfolded -- not finished
def draw_dihedral_angle_unfolded(dd,ss,r1,ba,hip_run,eave_length,originx,originy,ents)
    altitude = hip_run * Math.sin(r1);
    altitude_leg = altitude / Math.tan(r1);
    pt1 = []
    pt = [altitude_leg,0,0];
    pt1.x = originX + ( (pt.x * cosa) - (pt.y * sina));
    pt1.y = originY + ( (pt.x * sina) + (pt.y * cosa));
	
  
end
# This was modified and enhanced from dim_angle.rb,
# Copyright 2005, Didier Bur, which, in turn, was based on the demo
# rectangle.rb by @Last Software
# Copyright  2011 Stephen Baumgartner <steve@slbaumgartner.com>

def dimension_angle(pts0,pts1,pts2,radius,ents)
    
    # make vectors from the three points.  These vectors point
    # out from the angle vertex along the two edges.  They are needed by
    # some subsequent methods that require directions for offsets, not
    # points
    vec1 = pts1.vector_to pts0
    vec2 = pts1.vector_to pts2
    
    # trap degenerate cases
    if(vec1.length == 0 or vec2.length == 0)
      return
    end
    if(vec1.parallel? vec2)
     return
    end
    
    # the angle bisector vector - used for placing the text
    bisector = (vec1+vec2)
    
    # scale the vectors to extend 1.5 the radius.  This will affect
    # the drawn edges.  The value 1.5 is arbitrary - change it if another
    # look is desired
    vec1.length = radius * 1.5
    vec2.length = radius * 1.5
    
    # bisector length controls text placement.  0.75 is arbitrary - 
    # change it if another look is desired
    bisector.length = radius * 0.5
    
    # find the angle between the vectors and the normal to their plane
    # this calculation should not explode, since we trapped the case of
    # parallel above (angle = 0 or 180), but the calculation of the normal
    # will be numerically unstable when angle is very small or very close to
    # 180.  I have tested at 0.1 degrees without problems.
    angle = vec1.angle_between vec2
    text = angle_sprintf(angle)
    normal = (vec1 * vec2).normalize
 
      # interior angle mode
      # draw the arc across the angle at the selected radius
      arc = ents.add_arc pts1, vec1, normal, radius, 0, angle, 30
      
      # draw the text and leader line
      # the leader connects to the center of the arc (it was drawn with 30 segments)
      leader_point = arc[15].start.position
      
      # note: the vector offset of the leader is 3D, which will cause the text to
      # "float" and remain legible as the view is orbited.  I haven't found a way to
      # lock the leader and text into the plane of the angle.
      t = ents.add_text text,leader_point,bisector
      t.leader_type = 1
      
    
  end
  
  def dimension_angle_test(pts0,pts1,pts2)
    
    # make vectors from the three points.  These vectors point
    # out from the angle vertex along the two edges.  They are needed by
    # some subsequent methods that require directions for offsets, not
    # points
    vec1 = pts1.vector_to pts0
    vec2 = pts1.vector_to pts2    
    # trap degenerate cases
    if(vec1.length == 0 or vec2.length == 0)
      return 0
    end
    if(vec1.parallel? vec2)
      return 0
    end
       
    angle = vec1.angle_between vec2
    return angle_degrees(angle)
    
end
  

def angle_degrees(angle)
	return "0.00" if not angle
	return sprintf("%0.5f",angle.radians).to_f
  
end

def angle_sprintf(angle)
	return "0.00" if not angle
	return sprintf("%0.5f",angle.radians)
end

def length_sprintf(length)
	return "0.00" if not length
	return sprintf("%0.5f",length)
end

def point_sprintf(point)
	return sprintf("(%0.5f,%0.5f,%0.5f)", point.x,point.y,point.z)
end

def origin_angle(point)

	quad_angle = 0.00;
	quad = 0;
	angle = Math.atan(point.y / point.x)
	angle = angle.abs
	
	if( point.y > 0.0 &&  point.x > 0.0)
		quad = 0; quad_angle = angle;
	end
	if( point.y == 0.0 &&  point.x < 0.0)
		quad = 0; quad_angle =180.0.degrees;
	end
	if( point.y < 0.0 &&  point.x == 0.0)
		quad = 0; quad_angle =270.0.degrees;
	end
	if( point.y > 0.0 &&  point.x < 0.0)
		quad = 1; quad_angle = 180.0.degrees - angle;
	end
	if( point.y < 0.0 &&  point.x < 0.0)
		quad = 2; quad_angle = 180.0.degrees + angle;
	end
	if( point.y < 0.0 &&  point.x > 0.0)
		quad = 3; quad_angle = 360.0.degrees - angle;
	end
    puts "origin angle " + quad_angle.radians.to_s if $DEBUG_VERBOSE; 
    
	return quad_angle

end

def  rotate_points(pts,originX,originY,origin_direction_angle)
	
	rotated_pts = []
	angle = origin_direction_angle;
  sina= Math.sin(angle);
	cosa= Math.cos(angle);
	origin_direction_angle_d = angle_sprintf(origin_direction_angle);
	puts "origin_direction_angle = " + origin_direction_angle_d if $DEBUG_VERBOSE;
	
	pts.each do |pt|
		pt1 = []
		pt1.x = originX + ( (pt.x * cosa) - (pt.y * sina));
		pt1.y = originY + ( (pt.x * sina) + (pt.y * cosa));
		pt1.z = pt.z;
		rotated_pts.push(pt1);
	end	
	return rotated_pts;
end			

def  rotate_point(radius,originX,originY,origin_direction_angle)
	
    angle = origin_direction_angle;
    sina= Math.sin(angle);
    cosa= Math.cos(angle);
    origin_direction_angle_d = angle_sprintf(origin_direction_angle);
    puts "origin_direction_angle = " + origin_direction_angle_d if $DEBUG_VERBOSE;
    puts "originX " + originX.to_s if $DEBUG_VERBOSE; 
    puts "originY " + originY.to_s if $DEBUG_VERBOSE; 
    puts "radius " + radius.to_s if $DEBUG_VERBOSE; 
     
    rotated_pt = []
    rotated_pt.x = originX +(radius * cosa);
    rotated_pt.y = originY +( radius * sina);  
    rotated_pt.z = 0;
    puts "rotated_pt " + point_sprintf(rotated_pt) if $DEBUG_VERBOSE; 
    return rotated_pt;
  end			
  
def point_sprintf(point)
  return sprintf("(%0.5f,%0.5f,%0.5f)", point.x,point.y,point.z)
end

def sq(side)
    return 0 if(side == 0);
    rt = side * side;
    return rt;    
end
  
def get_triangle_SAS(side_b,angle_c,side_a)
  side_c  = 0;
  return side_c if(side_a == 0 || side_b == 0 || angle_C == 0);
  side_c = (sq(side_a) + sq(side_b)) - (2 * side_a * side_b * Math.cos(angle_c));
  side_c= sqrt(side_c);
  return (side_c);
  
end

def get_triangle_SSS(side_a,side_b,side_c)
    rt = [0,0,0]
    return rt if(side_a == 0 || side_b == 0 || side_c == 0);
    cos_a = (sq(side_b) + sq(side_c) - sq(side_a)) / (2 * side_b * side_c);
    cos_b = (sq(side_a) + sq(side_c) - sq(side_b)) / (2 * side_a * side_c);
    cos_c = (sq(side_a) + sq(side_b) - sq(side_c)) / (2 * side_a * side_b);
    angle_a = Math.acos(cos_a); 
    angle_b= Math.acos(cos_b);
    angle_c= Math.acos(cos_c);
    rt = [angle_b,angle_a,angle_c];
    return rt;
    
  end
  
def get_triangle_ASA(angle_b,side_c,angle_a)
  side_a  = 0;
  return side_a if(angle_a == 0 || angle_b == 0 || side_c == 0);
  angle_c = 180.degrees - angle_a - angle_b;
  #law of sines
  side_a = (side_c * Math.sin(angle_a))/ Math.sin(angle_c);
  return (side_a);
  
end
def find_plan_angle(major_pitch_angle,minor_pitch_angle,eave_angle)
  
    tan_major_pitch_angle = Math.tan(major_pitch_angle)      
    tan_minor_pitch_angle = Math.tan(minor_pitch_angle)  
    sin_eave_angle = Math.sin(eave_angle); 
    cos_eave_angle = Math.cos(eave_angle); 
    majorp_divide = tan_major_pitch_angle / tan_minor_pitch_angle;
    minorp_divide = tan_minor_pitch_angle / tan_major_pitch_angle;
    tan_major_plan_angle = (sin_eave_angle / (majorp_divide + cos_eave_angle)); 
    tan_minor_plan_angle = (sin_eave_angle / (minorp_divide + cos_eave_angle)); 

    major_plan_angle = Math.atan(tan_major_plan_angle); 
    minor_plan_angle = Math.atan(tan_minor_plan_angle); 

	if ((eave_angle > (Math::PI/2)) && (major_plan_angle < 0.0))
    	#major_plan_angle = Math::PI + major_plan_angle;
    	major_plan_angle = major_plan_angle.abs;
    	minor_plan_angle = eave_angle - major_plan_angle;
    	puts "major_angle < 0 " if $DEBUG_VERBOSE;
	end
	

	if ((eave_angle > (Math::PI/2)) && (minor_plan_angle < 0.0))
    	#minor_plan_angle = Math::PI + minor_plan_angle;
    	minor_plan_angle = minor_plan_angle.abs;
    	major_plan_angle = eave_angle - minor_plan_angle;
   		puts "minor_angle < 0 "  if $DEBUG_VERBOSE; 
    end
     plans = Array.new(2);
     plans[0]= major_plan_angle; 
     plans[1] = minor_plan_angle; 
    return plans;
  end
  
  def get_roof_framing_angles(eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle)
      
      hip_angle = Math.atan(Math.tan(major_pitch_angle)* Math.sin(major_plan_angle))
      major_hip_rafter_backing_angle = angle_sprintf(Math.atan(Math.sin(hip_angle) / Math.tan(major_plan_angle)))
      minor_hip_rafter_backing_angle = angle_sprintf(Math.atan(Math.sin(hip_angle) / Math.tan(minor_plan_angle)))

      major_jack_rafter_sidecut_angle = angle_sprintf(Math.atan(Math.cos(major_pitch_angle) / Math.tan(major_plan_angle)))
      minor_jack_rafter_sidecut_angle = angle_sprintf(Math.atan(Math.cos(minor_pitch_angle) / Math.tan(minor_plan_angle)))

      major_hip_rafter_sidecut_angle = angle_sprintf(Math.atan(Math.cos(hip_angle) / Math.tan(major_plan_angle)))
      minor_hip_rafter_sidecut_angle = angle_sprintf(Math.atan(Math.cos(hip_angle) / Math.tan(minor_plan_angle)))

      major_roof_sheathing_angle = angle_sprintf(Math.atan(Math.tan(major_plan_angle) / Math.cos(major_pitch_angle)))
      minor_roof_sheathing_angle = angle_sprintf(Math.atan(Math.tan(minor_plan_angle) / Math.cos(minor_pitch_angle)))


      
      all_text = " Eave Angle = " + angle_sprintf(eave_angle);
      all_text =   "\n Major Plan Angle = " + angle_sprintf(major_plan_angle);
      all_text += "\n Minor Plan Angle = " + angle_sprintf(minor_plan_angle);
      all_text += "\n Major Pitch Angle = " + angle_sprintf(major_pitch_angle);
      all_text += "\n Minor Pitch Angle = " + angle_sprintf(minor_pitch_angle);
      all_text += "\n Hip Rafter Angle = " + angle_sprintf(hip_angle);
      all_text += "\n Major Hip Rafter Backing Angle = " + major_hip_rafter_backing_angle;
      all_text += "\n Minor Hip Rafter Backing Angle = " + minor_hip_rafter_backing_angle;
      all_text += "\n Major Hip Rafter Sidecut Angle = " + major_hip_rafter_sidecut_angle;
      all_text += "\n Minor Hip Rafter Sidecut Angle = " + minor_hip_rafter_sidecut_angle;
      all_text += "\n Major Jack Rafter Sidecut Angle = " + major_jack_rafter_sidecut_angle;
      all_text += "\n Minor Jack Rafter Sidecut Angle = " + minor_jack_rafter_sidecut_angle;
      all_text += "\n Major Roof Sheathing Angle = " + major_roof_sheathing_angle;
      all_text += "\n Minor Roof Sheathing Angle = " + minor_roof_sheathing_angle;
      return all_text;
end
 
end # class TetrahedronTool

end # module ROOFFRAMING

#=============================================================================

if( not file_loaded?("RoofFraming.rb") )

    load("RoofFraming/plan_angle.rb")
    load("RoofFraming/Trapezoidal.rb")
    load("RoofFraming/Trapezoidal_Timber.rb")
    load("RoofFraming/Trapezoidal_Timber_Plane.rb")
    load("RoofFraming/draw_angle_dim.rb")
    load("RoofFraming/draw_arc.rb")
    load("RoofFraming/draw_circle.rb")
    load("RoofFraming/Tetrahedron_Slice.rb")
    load("RoofFraming/Slice.rb")      
    load("RoofFraming/Unequal_Hip.rb")
    load("RoofFraming/prism.rb")
    load("RoofFraming/tangent.rb")
    load("RoofFraming/drawdown.rb")
  
    submenu = UI.menu("Plugins").add_submenu("Roof Framing Geometry")
    submenu.add_item("Tetrahedron Angles") { Sketchup.active_model.select_tool ROOFFRAMING::TetrahedronTool.new }  
    submenu.add_item("Prism Plane For Hip Rafters") { Sketchup.active_model.select_tool ROOFFRAMING::PrismTool.new }  
  submenu.add_item("Prism Plane For Tangent Handrailing") { Sketchup.active_model.select_tool ROOFFRAMING::TangentTool.new }  

    submenu.add_item("Tetrahedron Slice") { Sketchup.active_model.select_tool ROOFFRAMING::TetrahedronSliceTool.new }  
   submenu.add_item("Rafter Slice") { Sketchup.active_model.select_tool ROOFFRAMING::SliceTool.new }  
 
    submenu.add_item("French Compagnon Roof Framing Kernel") { Sketchup.active_model.select_tool ROOFFRAMING::PlanAngleTool.new }  
    submenu.add_item("Trapezoidal Ground Plan ") { Sketchup.active_model.select_tool ROOFFRAMING::TrapezoidalTool.new }  
    submenu.add_item("Trapezoidal Timber") { Sketchup.active_model.select_tool ROOFFRAMING::TrapezoidalTimberTool.new }  
    submenu.add_item("Trapezoidal Timber Planes") { Sketchup.active_model.select_tool ROOFFRAMING::TrapezoidalTimberPlaneTool.new }  
    submenu.add_item("Unequal Pitched Hip Rafters") { Sketchup.active_model.select_tool ROOFFRAMING::UnequalHipTool.new }  

    submenu.add_item("Angular Dimension") { Sketchup.active_model.select_tool ROOFFRAMING::RoofAngleDimTool.new }  
     submenu.add_item("Angular Dimension Text") { Sketchup.active_model.select_tool ROOFFRAMING::RoofAngleDimToolText.new }  
    submenu.add_item("Draw Arc Center-Start-End") { Sketchup.active_model.select_tool ROOFFRAMING::DrawArcTool.new }  
    submenu.add_item("Draw Circle") { Sketchup.active_model.select_tool ROOFFRAMING::DrawCircleTool.new }  
     submenu.add_item("Draw Down Technique") { Sketchup.active_model.select_tool ROOFFRAMING::DrawDownTool.new }  

    submenu.add_item("Reload Ruby Files") { load("RoofFraming.rb"); load("RoofFraming/drawdown.rb")}  
   
end

#-----------------------------------------------------------------------------
file_loaded("RoofFraming.rb")

