# Copyright 2012 Sim Ayers
# version 1.0
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------


module ROOFFRAMING

class TrapezoidalTool < TetrahedronTool

$principal_rafter_pitch = 11.0;
$deck_angle_trapezoidal = 107.54573;
$eave_length_base = 8.5;
$eave_length_trapezoidal = 6.3125;
$show_angles = "no";
$roof_planes = "no";
$framing_angles = "no";

def initialize
       
    $principal_rafter_pitch = 11.0 if not $principal_rafter_pitch;
    $eave_length_base = 8.5 if not $eave_length_base ;
    $eave_length_trapezoidal = 6.3125 if not $eave_length_trapezoidal;
    $deck_angle_trapezoidal = 107.54573 if not $deck_angle;
    $show_angles = "no" if not $show_angles;
    $roof_planes = "no" if not $roof_planes;
    $framing_angles = "no" if not $framing_angles;

    # First prompt for the dimensions.  This is done using the inputbox method
        
    # The first step is to create some arrays which contain the prompts
    # and default values.
    prompts = ["Principal Rafter Pitch","Trapezoidal Eave Angle","Eave Length Base","Eave Length Trapezoidal","Dim Angles","Show Roof Planes","Show Roof Framing Angles"]
    #default values
    values = [$principal_rafter_pitch,$deck_angle_trapezoidal,$eave_length_base,$eave_length_trapezoidal,$show_angles,$roof_planes,$framing_angles]
    
    popups = [nil, nil, nil, nil,"yes|no","no|yes","no|yes"]
    
    # Now display the inputbox
    results = inputbox prompts, values,popups, "Trapezoidal Ground Plan";

    return if not results # This means that the user cancelled the operation
    $principal_rafter_pitch,$deck_angle_trapezoidal,$eave_length_base,$eave_length_trapezoidal,$show_angles,$roof_planes,$framing_angles = results
    
     if $deck_angle_trapezoidal < 90.0
     	UI.messagebox("Trapezoidal Eave Angle must be greater than or equal to 90.")
     	return
    end
     if $deck_angle_trapezoidal > 179.0
     	UI.messagebox("Trapezoidal Eave Angle must be less than 180.")
     	return
    end
    if ($principal_rafter_pitch <= 0.0 )
     	UI.messagebox("Principal Rafter Pitch can not be 0.")
     	return
    end
    if ($eave_length_base <= 0.0 )
     	UI.messagebox("Eave Length Base can not be 0.")
     	return
    end
    if ($eave_length_trapezoidal <= 0.0 )
     	UI.messagebox("Eave Length Trapezoidal can not be 0.")
     	return
    end
    show = true
    show = false if($show_angles == "no")
    show_roof_planes = false
    show_roof_planes = true if($roof_planes =="yes");
    show_roof_angles = false
    show_roof_angles = true if($framing_angles =="yes"); 
    
    draw_trapezoidal($principal_rafter_pitch,$deck_angle_trapezoidal,$eave_length_base,$eave_length_trapezoidal,show,show_roof_planes,show_roof_angles);  
    
    
end


 
 # draw the trapezoidal ground plan folded
def draw_trapezoidal(principal_rafter_pitch,deck_angle,eave_length_base,eave_length_trapezoidal,show_angles,show_roof_planes,show_roof_angles)  

    model = Sketchup.active_model
    # this enables undo of the whole operation as a unit
    model.start_operation "Trapezoidal Ground Plan"

    # create a new group in which we will draw our dimension entities
    model_ents = model.active_entities
    group = model_ents.add_group
    ents = group.entities
    group.name="Trapezoidal"
    wplayer=model.layers.add("Trapezoidal Ground Plan")
    wplayer.visible=true
    group.layer=wplayer
    group.casts_shadows=false
    group.receives_shadows=false
	
    eave_angle = 90.degrees
    eave_angle_trapezoidal = deck_angle.degrees;
    eave_angle_t1 =  eave_angle_trapezoidal - 90.degrees;
    eave_angle_t2 = 90.degrees - eave_angle_t1;

    eave_length_trapezoidal_length = eave_length_trapezoidal + (eave_length_base * Math.tan(eave_angle_t1));

    principal_rafter_pitch_angle = Math.atan(principal_rafter_pitch / 12);     	

    major_run = eave_length_base /2;
    major_rise = major_run * Math.tan(principal_rafter_pitch_angle);
    minor_run = eave_length_trapezoidal /2;   

    major_pitch_angle = principal_rafter_pitch_angle;     
    minor_pitch_angle = Math.atan(major_rise / minor_run);

    plan_angles = Array.new(2);
    plan_angles = find_plan_angle(major_pitch_angle,minor_pitch_angle,eave_angle)   
    major_plan_angle = plan_angles[0]; 
    minor_plan_angle = plan_angles[1]; 


    hip_angle = Math.atan(Math.tan(principal_rafter_pitch_angle)* Math.sin(major_plan_angle))
    major_side = minor_run
    minor_side = major_run
    hip_run_length = major_side / Math.cos(major_plan_angle)	

    puts "new trapezoidal dimensions " if $DEBUG_VERBOSE;
	
	
	eave_angle_d = angle_sprintf(eave_angle_trapezoidal );
	puts "eave angle trapezoidal = " + eave_angle_d if $DEBUG_VERBOSE;
  eave_angle_d = angle_sprintf(eave_angle_t1 );
	puts "eave angle trapezoidal t1 = " + eave_angle_d if $DEBUG_VERBOSE;
  eave_angle_d = angle_sprintf(eave_angle_t2 );
	puts "eave angle trapezoidal t2 = " + eave_angle_d if $DEBUG_VERBOSE;

  major_d = angle_sprintf(major_plan_angle);
	minor_d = angle_sprintf(minor_plan_angle);
	puts "major d = " + major_d if $DEBUG_VERBOSE;
	puts "minor d = " + minor_d if $DEBUG_VERBOSE;


	principal_rafter_pitch_angle_d = angle_sprintf(principal_rafter_pitch_angle);
	minor_pitch_angle_d = angle_sprintf(minor_pitch_angle);
	puts "principal rafter angle = " + principal_rafter_pitch_angle_d if $DEBUG_VERBOSE;
	puts "minor pitch angle = " + minor_pitch_angle_d if $DEBUG_VERBOSE;
	hip_angle_d = angle_sprintf(hip_angle);
	puts "hip rafter angle = " + hip_angle_d if $DEBUG_VERBOSE;
	puts "hip run = " + length_sprintf(hip_run_length) if $DEBUG_VERBOSE;
	
  m = length_sprintf(major_side)
  puts "major_side = " + m if $DEBUG_VERBOSE;
  m = length_sprintf(minor_side)
  puts "minor_side = " + m if $DEBUG_VERBOSE;
    
	framing_angle_text= get_roof_framing_angles(90.degrees,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle);


	
	p1 = [0,0,0]
	radius = eave_length_base/6; #radius used for angluar text position
  @pts = []
  @pts[0] = [p1.x,p1.y,0]
  @pts[1] = [ p1.x + eave_length_base,p1.y ,0 ]
  vec11 = @pts[1].vector_to @pts[0]
	vector = Geom::Vector3d.new 0,0,1
  vector2 = vector.normalize!
	t = ents.add_text(framing_angle_text, @pts[1],vector2) if show_roof_angles;
	#t.leader_type = 2

        
    dd = major_plan_angle; # plan angle
    ss = principal_rafter_pitch_angle; # roof pitch angle
    r1 = Math.atan(Math.tan(ss) * Math.sin(dd)); # hip rafter angle
    ee = Math.atan(Math.cos(ss) / Math.tan(dd)); # jack rafter angle
    bb = Math.atan(Math.sin(r1) / Math.tan(dd)); # hip rafter backing angle
    run = (eave_length_base/2); # common rafter run
    rise = run * Math.tan(ss); # major common rafter rise
    
    mid_point = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2),0]
    mid_point_z = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2),rise]

    trapezoidal_span = eave_length_trapezoidal + ((eave_length_base/2) * Math.tan(eave_angle_t1));

    mid_point_t = [p1.x+ (eave_length_base/2),p1.y+ trapezoidal_span,0]

    point_A=  [p1.x,p1.y,0]
    point_B=  [ p1.x ,p1.y + eave_length_trapezoidal_length , 0]
    point_C = [ p1.x + eave_length_base ,p1.y + eave_length_trapezoidal , 0]
    point_D = [ p1.x + eave_length_base ,p1.y, 0]
    point_E = [ p1.x ,p1.y + (eave_length_trapezoidal/2) , 0]
    point_F = [ p1.x+ eave_length_base ,p1.y + (eave_length_trapezoidal/2) , 0]
    point_G = [ p1.x+ (eave_length_base/2) ,p1.y + trapezoidal_span , 0]
    point_H = [ p1.x+ (eave_length_base/2) ,p1.y, 0]
    point_K = mid_point
    
    a_run = mid_point.distance point_G 
    b_rafter = a_run * Math.cos(eave_angle_t1) 
    b_run = b_rafter * Math.cos(eave_angle_t1) 
    b_rise = b_rafter * Math.sin(eave_angle_t1) 
    point_L = [p1.x+ (eave_length_base/2)+b_rise,p1.y+ (eave_length_trapezoidal/2)+b_run,0]
    m = length_sprintf(b_rafter)
    puts "b_rafter = " + m if $DEBUG_VERBOSE;
    m = length_sprintf(b_run)
    puts "b_run = " + m if $DEBUG_VERBOSE;
    m = length_sprintf(b_rise)
    puts "b_rise = " + m if $DEBUG_VERBOSE;

    b_rafter = rise 
    base = mid_point.distance point_L 
    trapezoidal_angle_LM = Math.atan(rise / base)
    b_run = b_rafter * Math.cos(trapezoidal_angle_LM) 
    b_rise = b_rafter * Math.sin(trapezoidal_angle_LM) 
    point_M = [p1.x+ (eave_length_base/2)-b_run,p1.y+ (eave_length_trapezoidal/2)+b_rise,0]
    rotate_angle = 90.degrees  + eave_angle_t2
    point_M = rotate_point(rise,mid_point.x,mid_point.y ,rotate_angle)
    
    
    trapezoidal_angle_LM_d = angle_sprintf(trapezoidal_angle_LM);
    puts "trapezoidal_angle_LM = " + trapezoidal_angle_LM_d if $DEBUG_VERBOSE;

    trapezoidal_angle_GK = Math.atan(rise / a_run)
    trapezoidal_angle_GK_d = angle_sprintf(trapezoidal_angle_GK);
    puts "trapezoidal_angle_GK = " + trapezoidal_angle_GK_d if $DEBUG_VERBOSE; 

        
    trapezoidal_run = mid_point.distance point_L 
    trapezoidal_rafter_angle = Math.atan(rise/trapezoidal_run);
    trapezoidal_rafter = trapezoidal_run / Math.cos(trapezoidal_rafter_angle);
    trapezoidal_leg = trapezoidal_rafter * Math.tan(trapezoidal_rafter_angle);
    rotate_angle = trapezoidal_rafter_angle  + eave_angle_t1
    point_Q = rotate_point(trapezoidal_leg,point_M.x,point_M.y ,-rotate_angle)
    
    plan_angles = Array.new(2);
    plan_angles = find_plan_angle(major_pitch_angle, trapezoidal_rafter_angle ,eave_angle_trapezoidal)   
    major_plan_angle2 = plan_angles[0]; 
    minor_plan_angle2 = plan_angles[1]; 
    framing_angle_text = get_roof_framing_angles(eave_angle_trapezoidal ,major_pitch_angle, trapezoidal_rafter_angle ,major_plan_angle2,minor_plan_angle2);
    point_C2 = point_C;
    #point_C2.x +=50;
    #point_C2.y +=50;
    t = ents.add_text(framing_angle_text, point_C2,vector2) if show_roof_angles;

    plan_angles = Array.new(2);
    plan_angles = find_plan_angle(major_pitch_angle, trapezoidal_rafter_angle ,eave_angle_t2)   
    major_plan_angle2 = plan_angles[0]; 
    minor_plan_angle2 = plan_angles[1]; 
    point_B2 = point_B;
    #point_B2.x -=50;
    #point_B2.y +=50;
    framing_angle_text = get_roof_framing_angles(eave_angle_t2 ,major_pitch_angle, trapezoidal_rafter_angle ,major_plan_angle2,minor_plan_angle2);
    t = ents.add_text(framing_angle_text, point_B2,vector2) if show_roof_angles;


      # draw trapezoidal ground plan
    pts = []
    pts[0] = point_A
    pts[1] = point_B
    pts[2] = point_C
    pts[3] = point_D
    pts[4] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
    
    # draw rise
        
    pts1 = mid_point
    pts2 = mid_point_z
    new_rafter = ents.add_cline pts1, pts2    
      
     # draw hip rafter runs
        
    pts1 = point_A
    pts2 = mid_point
    new_rafter = ents.add_cline pts1, pts2    
   dimension_angle(point_B,point_A,mid_point,radius, ents ) if show_angles;
   dimension_angle(point_D,point_A,mid_point,radius, ents ) if show_angles;
   
    pts1 = point_B
    pts2 = mid_point
    new_rafter = ents.add_cline pts1, pts2
    pts1 = point_C
    pts2 = mid_point
    new_rafter = ents.add_cline pts1, pts2
    pts1 = point_D
    pts2 = mid_point
    new_rafter = ents.add_cline pts1, pts2

    # draw hip rafter 
        
    pts1 = point_A
    pts2 = mid_point_z
    new_rafter = ents.add_line pts1, pts2    
    dimension_angle(mid_point,point_A,mid_point_z,radius, ents ) if show_angles;
    
    pts1 = point_B
    pts2 = mid_point_z
    new_rafter = ents.add_line pts1, pts2
     dimension_angle(mid_point,point_B,mid_point_z,radius, ents ) if show_angles;
     
    pts1 = point_C
    pts2 = mid_point_z
    new_rafter = ents.add_line pts1, pts2
     dimension_angle(mid_point,point_C,mid_point_z,radius, ents ) if show_angles;
     
    pts1 = point_D
    pts2 = mid_point_z
    new_rafter = ents.add_line pts1, pts2
     dimension_angle(mid_point,point_D,mid_point_z,radius, ents ) if show_angles;
    
    # draw rafter runs
    pts1 = point_E
    pts2 = mid_point
    new_rafter = ents.add_cline pts1, pts2
    pts1 = point_F
    pts2 = mid_point
    new_rafter = ents.add_cline pts1, pts2
    pts1 = point_G
    pts2 = mid_point
    new_rafter = ents.add_cline pts1, pts2
    pts1 = point_H
    pts2 = mid_point
    new_rafter = ents.add_cline pts1, pts2
    
    
    # draw rafters 
    pts1 = point_E
    pts2 = mid_point_z
    new_rafter = ents.add_cline pts1, pts2
    dimension_angle(mid_point,point_E,mid_point_z,radius, ents ) if show_angles;

    pts1 = point_F
    pts2 = mid_point_z
    new_rafter = ents.add_cline pts1, pts2
    dimension_angle(mid_point,point_F,mid_point_z,radius, ents ) if show_angles;

    pts1 = point_G
    pts2 = mid_point_z
    new_rafter = ents.add_cline pts1, pts2
    dimension_angle(mid_point,point_G,mid_point_z,radius, ents ) if show_angles;

    pts1 = point_H
    pts2 = mid_point_z
    new_rafter = ents.add_cline pts1, pts2
    dimension_angle(mid_point,point_H,mid_point_z,radius, ents ) if show_angles;
     
     if(show_roof_planes)
        # draw trapezoidal roof planes
    pts = []
    pts[0] = point_A
    pts[1] = mid_point
    pts[2] = mid_point_z
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    
    pts = []
    pts[0] = point_B
    pts[1] = mid_point
    pts[2] = mid_point_z
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    pts = []
    pts[0] = point_C
    pts[1] = mid_point
    pts[2] = mid_point_z
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    pts = []
    pts[0] = point_D
    pts[1] = mid_point
    pts[2] = mid_point_z
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    
    pts = []
    pts[0] = point_E
    pts[1] = mid_point
    pts[2] = mid_point_z
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    pts = []
    pts[0] = point_G
    pts[1] = mid_point
    pts[2] = mid_point_z
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    pts = []
    pts[0] = point_F
    pts[1] = mid_point
    pts[2] = mid_point_z
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    pts = []
    pts[0] = point_H
    pts[1] = mid_point
    pts[2] = mid_point_z
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    
     pts = []
    pts[0] = point_A
    pts[1] = mid_point_z
    pts[2] = point_D
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    
      pts = []
    pts[0] = point_A
    pts[1] = mid_point_z
    pts[2] = point_B
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
      pts = []
    pts[0] = point_B
    pts[1] = mid_point_z
    pts[2] = point_C
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
      pts = []
    pts[0] = point_C
    pts[1] = mid_point_z
    pts[2] = point_D
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    
  end
   # draw DP line and projection plane for skewed trapezoidal common rafter
     if (eave_angle_trapezoidal  > 90.degrees)
        pts1 = point_L
        pts2 = point_Q
        new_rafter = ents.add_cline pts1, pts2
        pts1 = point_L
        pts2 = point_M
        new_rafter = ents.add_cline pts1, pts2
        pts1 = point_M
        pts2 = point_Q
        new_rafter = ents.add_cline pts1, pts2
        pts1 = point_L
        pts2 = point_Q
        new_rafter = ents.add_cline pts1, pts2
        pts1 = point_G
        pts2 = point_Q
        new_rafter = ents.add_cline pts1, pts2
        pts1 = point_Q
        pts2 = mid_point_z
        new_rafter = ents.add_cline pts1, pts2
        pts1 = point_L
        pts2 = mid_point_z
        new_rafter = ents.add_cline pts1, pts2
         dimension_angle(mid_point,point_L,mid_point_z,radius, ents ) if show_angles;
           dimension_angle(mid_point_z,point_G,point_Q,radius*1.5, ents ) if show_angles;
           
        a_run = mid_point.distance point_G         
        intersecting_angle = dimension_angle_test(mid_point,point_G,point_Q);
        puts "intersecting_angle = " + intersecting_angle.to_s if $DEBUG_VERBOSE; 

        y = a_run * Math.tan(intersecting_angle.degrees);
        pt = [mid_point.x - y,mid_point.y,mid_point.z]
        puts "intersecting pt " + point_sprintf(pt) if $DEBUG_VERBOSE; 
        pts1 = pt
        pts2 = mid_point_z
        new_rafter = ents.add_cline pts1, pts2
        dimension_angle(mid_point,mid_point_z,pt,radius, ents ) if show_angles;
        
        if(show_roof_planes)
            pts = []
            pts[0] = point_G
            pts[1] = mid_point_z
            pts[2] = point_Q
            pts[3] = pts[0]
            new_rafter = ents.add_face pts
        end
        
      end  
      
       trap_run = mid_point.distance point_L 
    trap_rafter_angle = Math.atan(rise/trap_run);

    plan_angles = Array.new(2);
    plan_angles = find_plan_angle(major_pitch_angle,trap_rafter_angle ,eave_angle_trapezoidal)   
    major_plan_angle2 = plan_angles[0]; 
    minor_plan_angle2 = plan_angles[1]; 
    minor_trapezoidal_plan_angle =  minor_plan_angle2;

    plan_angles = Array.new(2);
    plan_angles = find_plan_angle(major_pitch_angle,trap_rafter_angle ,eave_angle_t2)   
    major_plan_angle2 = plan_angles[0]; 
    minor_plan_angle2 = plan_angles[1]; 
    minor_trapezoidal_plan_angle2 = minor_plan_angle2;    


    roof_sheathing_angle_right = Math.atan(Math.tan(minor_trapezoidal_plan_angle) / Math.cos(trapezoidal_angle_LM));
    roof_sheathing_angle_left = Math.atan(Math.tan(minor_trapezoidal_plan_angle2) / Math.cos(trapezoidal_angle_LM));
    
    roof_sheathing_run_base = point_L.distance point_C;
    trapezoidal_rafter_length = roof_sheathing_run_base  * Math.tan(roof_sheathing_angle_right);
    a_run = mid_point.distance point_G 
    b_rafter = a_run * Math.cos(eave_angle_t1) 
    b_run = b_rafter * Math.cos(eave_angle_t1) 
    b_rise = b_rafter * Math.sin(eave_angle_t1) 
    s_run = trapezoidal_rafter_length * Math.cos(eave_angle_t2);
    s_rise = trapezoidal_rafter_length * Math.sin(eave_angle_t2);
    point_roof_surface =[p1.x+ (eave_length_base/2)+b_rise+s_run,p1.y+ (eave_length_trapezoidal/2)+b_run+s_rise,0];
    
    #draw real roof surface
    pts1 = point_B
    pts2 = point_roof_surface
    new_rafter = ents.add_cline pts1, pts2
    pts1 = point_C
    pts2 = point_roof_surface
    new_rafter = ents.add_cline pts1, pts2
    pts1 = point_L
    pts2 = point_roof_surface
    new_rafter = ents.add_cline pts1, pts2
    
    pts1 = point_G
    pts2 = point_roof_surface
    new_rafter = ents.add_cline pts1, pts2
     pts1 = point_K
    pts2 = point_L
    new_rafter = ents.add_cline pts1, pts2

    dimension_angle(point_G,point_roof_surface,point_L,radius, ents ) if show_angles;
      dimension_angle(point_Q,point_G,point_K,radius, ents ) if show_angles;

    Sketchup.active_model.rendering_options["ModelTransparency"]=true 
    view = Sketchup.active_model.active_view 
    view.field_of_view = 30
    #result = Sketchup.send_action 10501; # set view to Top
    view.zoom_extents 

    group.to_component 
    model.commit_operation
    
   
end # end function
   
 
   
   
end # class TrapezoidalTool

end #module ROOFFRAMING

file_loaded("RoofFraming/Trapezoidal.rb")
