# Copyright 2012 Sim Ayers
# version 1.0
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------


module ROOFFRAMING

class TrapezoidalTimberTool < TetrahedronTool

$principal_rafter_pitch = 12.0;
$deck_angle_trapezoidal = 107.0;
$eave_length_base = 104.0;
$eave_length_trapezoidal = 77.0;
$roof_planes = "no";
$framing_angles = "no";
$show_internal = "no"
$principal_rafter_depth = 11.5;
$principal_rafter_thickness = 5.5;

def initialize
       
    $principal_rafter_pitch = 12.0 if not $principal_rafter_pitch;
    $eave_length_base = 104.0 if not $eave_length_base ;
    $eave_length_trapezoidal = 77.0 if not $eave_length_trapezoidal;
    $deck_angle_trapezoidal = 104.0 if not $deck_angle;
    $roof_planes = "no" if not $roof_planes;
    $framing_angles = "no" if not $framing_angles;
    $show_internal = "no" if not $show_internal;
    $principal_rafter_depth = 11.5 if not $principal_rafter_depth;
    $principal_rafter_thickness = 5.5 if not $principal_rafter_thickness;

    # First prompt for the dimensions.  This is done using the inputbox method
        
    # The first step is to create some arrays which contain the prompts
    # and default values.
    prompts = ["Principal Rafter Pitch","Trapezoidal Eave Angle","Eave Length Base","Eave Length Trapezoidal",
    "Depth of Principal Rafter","Thickness of Principal Rafter","Show Roof Planes",
    "Show Roof Framing Angles","Show Internal Ceiling Planes"]
    
    #default values
    values = [$principal_rafter_pitch,$deck_angle_trapezoidal,$eave_length_base,$eave_length_trapezoidal,$principal_rafter_depth,$principal_rafter_thickness,
    $roof_planes,$framing_angles,$show_internal]
    
    popups = [nil, nil, nil, nil,nil,nil,"no|yes","no|yes","no|yes"]
    
    # Now display the inputbox
    results = inputbox prompts, values,popups,"Trapezoidal Timber";

    return if not results # This means that the user cancelled the operation
    $principal_rafter_pitch,$deck_angle_trapezoidal,$eave_length_base,$eave_length_trapezoidal,$principal_rafter_depth,$principal_rafter_thickness,
    $roof_planes,$framing_angles,$show_internal= results
    
     if $deck_angle_trapezoidal < 90.0
     	UI.messagebox("Trapezoidal Eave Angle must be greater than or equal to 90.")
     	return
    end
     if $deck_angle_trapezoidal > 179.0
     	UI.messagebox("Trapezoidal Eave Angle must be less than 180.")
     	return
    end
    if ($principal_rafter_pitch <= 0.0 )
     	UI.messagebox("Principal Rafter Pitch can not be 0.")
     	return
    end
    if ($eave_length_base <= 0.0 )
     	UI.messagebox("Eave Length Base can not be 0.")
     	return
    end
    if ($eave_length_trapezoidal <= 0.0 )
     	UI.messagebox("Eave Length Trapezoidal can not be 0.")
     	return
    end
    show = false 
    show_roof_planes = false
    show_roof_planes = true if($roof_planes =="yes");
    show_roof_angles = false
    show_roof_angles = true if($framing_angles =="yes"); 
    show_internal_planes = false
    show_internal_planes = true if($show_internal =="yes");
    
    draw_trapezoidal($principal_rafter_pitch,$deck_angle_trapezoidal,$eave_length_base,
    $eave_length_trapezoidal,show,show_roof_planes,show_roof_angles,show_internal_planes,$principal_rafter_depth,$principal_rafter_thickness);     
     
end
 
 # draw the trapezoidal ground plan folded
def draw_trapezoidal(principal_rafter_pitch,deck_angle,eave_length_base,eave_length_trapezoidal,show_angles,show_roof_planes,
  show_roof_angles,show_internal_planes,principal_rafter_depth,principal_rafter_thickness)  

        model = Sketchup.active_model;
        # this enables undo of the whole operation as a unit
        model.start_operation "Trapezoidal Timber";

        # create a new group in which we will draw our dimension entities
        model_ents = model.active_entities;
        group = model_ents.add_group;
        ents = group.entities;
        group.name="Trapezoidal Timber";
        wplayer=model.layers.add("Trapezoidal Timber");
        wplayer.visible=true;
        group.layer=wplayer;
        group.casts_shadows=false;
        group.receives_shadows=false;

        eave_angle = 90.degrees
        eave_angle_trapezoidal = deck_angle.degrees;
        eave_angle_t1 =  eave_angle_trapezoidal - 90.degrees;
        eave_angle_t2 = 90.degrees - eave_angle_t1;

        eave_length_trapezoidal_length = eave_length_trapezoidal + (eave_length_base * Math.tan(eave_angle_t1));

        principal_rafter_pitch_angle = Math.atan(principal_rafter_pitch / 12);     	

        major_run = eave_length_base /2;
        major_rise = major_run * Math.tan(principal_rafter_pitch_angle);
        minor_run = eave_length_trapezoidal /2;   

        major_pitch_angle = principal_rafter_pitch_angle;     
        minor_pitch_angle = Math.atan(major_rise / minor_run);

        plan_angles = Array.new(2);
        plan_angles = find_plan_angle(major_pitch_angle,minor_pitch_angle,eave_angle);   
        major_plan_angle = plan_angles[0]; 
        minor_plan_angle = plan_angles[1]; 


        hip_angle = Math.atan(Math.tan(principal_rafter_pitch_angle)* Math.sin(major_plan_angle));
        major_side = minor_run;
        minor_side = major_run;
        hip_run_length = major_side / Math.cos(major_plan_angle);	

        puts "new trapezoidal timber dimensions " if $DEBUG_VERBOSE;


        eave_angle_d = angle_sprintf(eave_angle_trapezoidal );
        puts "eave angle trapezoidal = " + eave_angle_d if $DEBUG_VERBOSE;
        eave_angle_d = angle_sprintf(eave_angle_t1 );
        puts "eave angle trapezoidal t1 = " + eave_angle_d if $DEBUG_VERBOSE;
        eave_angle_d = angle_sprintf(eave_angle_t2 );
        puts "eave angle trapezoidal t2 = " + eave_angle_d if $DEBUG_VERBOSE;

        major_d = angle_sprintf(major_plan_angle);
        minor_d = angle_sprintf(minor_plan_angle);
        puts "major d = " + major_d if $DEBUG_VERBOSE;
        puts "minor d = " + minor_d if $DEBUG_VERBOSE;


        principal_rafter_pitch_angle_d = angle_sprintf(principal_rafter_pitch_angle);
        minor_pitch_angle_d = angle_sprintf(minor_pitch_angle);
        puts "principal rafter angle = " + principal_rafter_pitch_angle_d if $DEBUG_VERBOSE;
        puts "minor pitch angle = " + minor_pitch_angle_d if $DEBUG_VERBOSE;
        hip_angle_d = angle_sprintf(hip_angle);
        puts "hip rafter angle = " + hip_angle_d if $DEBUG_VERBOSE;
        puts "hip run = " + length_sprintf(hip_run_length) if $DEBUG_VERBOSE;

        m = length_sprintf(major_side);
        puts "major_side = " + m if $DEBUG_VERBOSE;
        m = length_sprintf(minor_side);
        puts "minor_side = " + m if $DEBUG_VERBOSE;

        framing_angle_text = get_roof_framing_angles(90.degrees,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle);

        p1 = [0,0,0];
        radius = eave_length_base/6; #radius used for angluar text position
        @pts = [];
        @pts[0] = [p1.x,p1.y,0];
        @pts[1] = [ p1.x + eave_length_base,p1.y,0];
        vec11 = @pts[1].vector_to @pts[0];
        vector = Geom::Vector3d.new(0,0,1);
        vector2 = vector.normalize!
        t = ents.add_text(framing_angle_text, @pts[1],vector2) if show_roof_angles;

        dd = major_plan_angle; # plan angle
        ss = principal_rafter_pitch_angle; # roof pitch angle
        r1 = Math.atan(Math.tan(ss) * Math.sin(dd)); # hip rafter angle
        ee = Math.atan(Math.cos(ss) / Math.tan(dd)); # jack rafter angle
        bb = Math.atan(Math.sin(r1) / Math.tan(dd)); # hip rafter backing angle
        run = (eave_length_base/2); # common rafter run
        rise = run * Math.tan(ss); # major common rafter rise

        mid_point = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2),0];
        mid_point_z = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2),rise];

        trapezoidal_span = eave_length_trapezoidal + ((eave_length_base/2) * Math.tan(eave_angle_t1));

        mid_point_t = [p1.x+ (eave_length_base/2),p1.y+ trapezoidal_span,0];

        point_A=  [p1.x,p1.y,0];
        point_B=  [ p1.x ,p1.y + eave_length_trapezoidal_length ,0];
        point_C = [ p1.x + eave_length_base ,p1.y + eave_length_trapezoidal ,0];
        point_D = [ p1.x + eave_length_base ,p1.y,0];
        point_E = [ p1.x ,p1.y + (eave_length_trapezoidal/2),0];
        point_F = [ p1.x+ eave_length_base ,p1.y + (eave_length_trapezoidal/2),0];
        point_G = [ p1.x+ (eave_length_base/2) ,p1.y + trapezoidal_span,0];
        point_H = [ p1.x+ (eave_length_base/2) ,p1.y, 0];
        point_K = mid_point;

        a_run = mid_point.distance point_G;
        b_rafter = a_run * Math.cos(eave_angle_t1); 
        b_run = b_rafter * Math.cos(eave_angle_t1); 
        b_rise = b_rafter * Math.sin(eave_angle_t1); 
        point_L = [p1.x+ (eave_length_base/2)+b_rise,p1.y+ (eave_length_trapezoidal/2)+b_run,0];
        m = length_sprintf(b_rafter);
        puts "Point L Rafter = " + m if $DEBUG_VERBOSE;
        m = length_sprintf(b_run);
        puts "Point L Run = " + m if $DEBUG_VERBOSE;
        m = length_sprintf(b_rise);
        puts "Point L Rise = " + m if $DEBUG_VERBOSE;

        trap_run = mid_point.distance point_L; 
        trap_rafter_angle = Math.atan(rise/trap_run);

        plan_angles = Array.new(2);
        plan_angles = find_plan_angle(major_pitch_angle,trap_rafter_angle ,eave_angle_trapezoidal);   
        major_plan_angle2 = plan_angles[0]; 
        minor_plan_angle2 = plan_angles[1]; 
        minor_trapezoidal_plan_angle =  minor_plan_angle2;
        major_trapezoidal_plan_angle = major_plan_angle2;

        framing_angle_text = get_roof_framing_angles(eave_angle_trapezoidal ,major_pitch_angle,trap_rafter_angle ,major_plan_angle2,minor_plan_angle2);

        t = ents.add_text(framing_angle_text, point_C,vector2) if show_roof_angles;

        plan_angles = Array.new(2);
        plan_angles = find_plan_angle(major_pitch_angle,trap_rafter_angle ,eave_angle_t2)   
        major_plan_angle2 = plan_angles[0]; 
        minor_plan_angle2 = plan_angles[1]; 
        minor_trapezoidal_plan_angle2 = minor_plan_angle2;
        major_trapezoidal_plan_angle2 = major_plan_angle2;
        framing_angle_text = get_roof_framing_angles(eave_angle_t2 ,major_pitch_angle,trap_rafter_angle ,major_plan_angle2,minor_plan_angle2);
        t = ents.add_text(framing_angle_text, point_B,vector2) if show_roof_angles;

        b_rafter = rise; 
        base = mid_point.distance point_L; 
        trapezoidal_angle_LM = Math.atan(rise / base);
        b_run = b_rafter * Math.cos(trapezoidal_angle_LM); 
        b_rise = b_rafter * Math.sin(trapezoidal_angle_LM); 
        point_M = [p1.x+ (eave_length_base/2)-b_run,p1.y+ (eave_length_trapezoidal/2)+b_rise,0];
        rotate_angle = 90.degrees  + eave_angle_t2;
        point_M = rotate_point(rise,mid_point.x,mid_point.y ,rotate_angle);

        trapezoidal_angle_LM_d = angle_sprintf(trapezoidal_angle_LM);
        puts "trapezoidal_angle_LM = " + trapezoidal_angle_LM_d if $DEBUG_VERBOSE;

        trapezoidal_angle_GK = Math.atan(rise / a_run)
        trapezoidal_angle_GK_d = angle_sprintf(trapezoidal_angle_GK);
        puts "trapezoidal_angle_GK = " + trapezoidal_angle_GK_d if $DEBUG_VERBOSE; 


        trapezoidal_run = mid_point.distance point_L 
        trapezoidal_rafter_angle = Math.atan(rise/trapezoidal_run);
        trapezoidal_rafter = trapezoidal_run / Math.cos(trapezoidal_rafter_angle);
        trapezoidal_leg = trapezoidal_rafter * Math.tan(trapezoidal_rafter_angle);
        rotate_angle = trapezoidal_rafter_angle  + eave_angle_t1
        point_Q = rotate_point(trapezoidal_leg,point_M.x,point_M.y ,-rotate_angle)
        puts "Trapezoidal_Rafter = " + trapezoidal_rafter.to_s if $DEBUG_VERBOSE; 
        tri_base_GL = point_G.distance point_L 
        roof_surface_rotation_angle = Math.atan( trapezoidal_rafter / tri_base_GL) 
        roof_surface_rotation_angle_d = angle_sprintf(roof_surface_rotation_angle);
        puts "roof_surface_rotation_angle = " + roof_surface_rotation_angle_d if $DEBUG_VERBOSE; 
              
        # draw trapezoidal ground plan
        pts = [];
        pts[0] = point_A;
        pts[1] = point_B;
        pts[2] = point_C;
        pts[3] = point_D;
        pts[4] = pts[0];
        #new_rafter = ents.add_face pts
        new_rafter = ents.add_line pts[0], pts[1]   
        new_rafter = ents.add_line pts[1], pts[2]   
        new_rafter = ents.add_line pts[2], pts[3]   
        new_rafter = ents.add_line pts[3], pts[0]   

        dimension_angle(pts[0],pts[1],pts[2],radius, ents ) if show_angles;
        dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;

        # draw rise

        pts1 = mid_point
        pts2 = mid_point_z
        new_rafter = ents.add_cline(pts1,pts2);    

        # draw hip rafter runs

        pts1 = point_A
        pts2 = mid_point
        new_rafter = ents.add_cline(pts1,pts2);    
        dimension_angle(point_B,point_A,mid_point,radius, ents ) if show_angles;
        dimension_angle(point_D,point_A,mid_point,radius, ents ) if show_angles;

        pts1 = point_B
        pts2 = mid_point
        new_rafter = ents.add_cline(pts1,pts2);
        pts1 = point_C
        pts2 = mid_point
        new_rafter = ents.add_cline(pts1,pts2);
        pts1 = point_D
        pts2 = mid_point
        new_rafter = ents.add_cline(pts1,pts2);

        # draw hip rafter 

        pts1 = point_A
        pts2 = mid_point_z
        new_rafter = ents.add_line pts1, pts2    
        dimension_angle(mid_point,point_A,mid_point_z,radius, ents ) if show_angles;

        pts1 = point_B
        pts2 = mid_point_z
        new_rafter = ents.add_line pts1, pts2
        dimension_angle(mid_point,point_B,mid_point_z,radius, ents ) if show_angles;
     
        pts1 = point_C;
        pts2 = mid_point_z;
        new_rafter = ents.add_line(pts1,pts2);
        dimension_angle(mid_point,point_C,mid_point_z,radius, ents ) if show_angles;

        pts1 = point_D;
        pts2 = mid_point_z;
        new_rafter = ents.add_line(pts1,pts2);
        dimension_angle(mid_point,point_D,mid_point_z,radius, ents ) if show_angles;

        # draw rafter runs
        pts1 = point_E;
        pts2 = mid_point;
        new_rafter = ents.add_cline(pts1,pts2);
        pts1 = point_F;
        pts2 = mid_point;
        new_rafter = ents.add_cline(pts1,pts2);
        pts1 = point_G;
        pts2 = mid_point;
        new_rafter = ents.add_cline(pts1,pts2);
        pts1 = point_H;
        pts2 = mid_point;
        new_rafter = ents.add_cline(pts1,pts2);


        # draw rafters 
        pts1 = point_E
        pts2 = mid_point_z
        new_rafter = ents.add_cline(pts1,pts2);
        dimension_angle(mid_point,point_E,mid_point_z,radius, ents ) if show_angles;

        pts1 = point_F
        pts2 = mid_point_z
        new_rafter = ents.add_cline(pts1,pts2);
        dimension_angle(mid_point,point_F,mid_point_z,radius, ents ) if show_angles;

        pts1 = point_G
        pts2 = mid_point_z
        new_rafter = ents.add_cline(pts1,pts2);
        dimension_angle(point_Q,point_G,mid_point_z,radius, ents ) if show_angles;

        pts1 = point_H
        pts2 = mid_point_z
        new_rafter = ents.add_cline(pts1,pts2);
        dimension_angle(mid_point,point_H,mid_point_z,radius, ents ) if show_angles;
        dimension_angle(point_K,point_B,point_L,radius, ents ) if show_angles;
        dimension_angle(point_K,point_B,point_E,radius, ents ) if show_angles;
        dimension_angle(point_K,point_C,point_F,radius, ents ) if show_angles;
        dimension_angle(point_K,point_C,point_L,radius, ents ) if show_angles;

        if(show_roof_planes)
          # draw trapezoidal roof planes
          pts = []
          pts[0] = point_A
          pts[1] = mid_point
          pts[2] = mid_point_z
          pts[3] = pts[0]
          new_rafter = ents.add_face pts

          pts = []
          pts[0] = point_B
          pts[1] = mid_point
          pts[2] = mid_point_z
          pts[3] = pts[0]
          new_rafter = ents.add_face pts
          pts = []
          pts[0] = point_C
          pts[1] = mid_point
          pts[2] = mid_point_z
          pts[3] = pts[0]
          new_rafter = ents.add_face pts
          pts = []
          pts[0] = point_D
          pts[1] = mid_point
          pts[2] = mid_point_z
          pts[3] = pts[0]
          new_rafter = ents.add_face pts

          pts = []
          pts[0] = point_E
          pts[1] = mid_point
          pts[2] = mid_point_z
          pts[3] = pts[0]
          new_rafter = ents.add_face pts
          pts = []
          pts[0] = point_G
          pts[1] = mid_point
          pts[2] = mid_point_z
          pts[3] = pts[0]
          new_rafter = ents.add_face pts
          pts = []
          pts[0] = point_F
          pts[1] = mid_point
          pts[2] = mid_point_z
          pts[3] = pts[0]
          new_rafter = ents.add_face pts
          pts = []
          pts[0] = point_H
          pts[1] = mid_point
          pts[2] = mid_point_z
          pts[3] = pts[0]
          new_rafter = ents.add_face pts

          pts = []
          pts[0] = point_A
          pts[1] = mid_point_z
          pts[2] = point_D
          pts[3] = pts[0]
          new_rafter = ents.add_face pts
    
          pts = []
          pts[0] = point_A
          pts[1] = mid_point_z
          pts[2] = point_B
          pts[3] = pts[0]
          new_rafter = ents.add_face pts
          pts = []
          pts[0] = point_B
          pts[1] = mid_point_z
          pts[2] = point_C
          pts[3] = pts[0]
          new_rafter = ents.add_face pts
          pts = []
          pts[0] = point_C
          pts[1] = mid_point_z
          pts[2] = point_D
          pts[3] = pts[0]
          new_rafter = ents.add_face pts
        end
        
        # draw DP line and projection plane for skewed trapezoidal common rafter
     if (eave_angle_trapezoidal  > 90.degrees)
          pts1 = point_L
          pts2 = point_Q
          new_rafter = ents.add_cline(pts1,pts2);
          pts1 = point_L
          pts2 = point_M
          new_rafter = ents.add_cline(pts1,pts2);
          pts1 = point_M
          pts2 = point_Q
          new_rafter = ents.add_cline(pts1,pts2);
          pts1 = point_L
          pts2 = point_Q
          new_rafter = ents.add_cline(pts1,pts2);
          pts1 = point_G
          pts2 = point_Q
          new_rafter = ents.add_cline(pts1,pts2);
          pts1 = point_K
          pts2 = point_L
          new_rafter = ents.add_cline(pts1,pts2);
          a_run = mid_point.distance point_G         
          intersecting_angle = dimension_angle_test(mid_point,point_G,point_Q);
          puts "intersecting_angle = " + intersecting_angle.to_s if $DEBUG_VERBOSE; 

          y = a_run * Math.tan(intersecting_angle.degrees);
          pt = [mid_point.x - y,mid_point.y,mid_point.z]
          puts "intersecting pt " + point_sprintf(pt) if $DEBUG_VERBOSE; 
          pts1 = pt
          pts2 = mid_point_z

        end
        
        # draw principal rafters above trapezoidal
        #principal_rafter_depth,principal_rafter_thickness
        offset = eave_length_trapezoidal_length / 10; # diplay principal rafters at offset
        #calculate the length of the rafter plumbcut
        plumbcut = principal_rafter_depth / Math.cos(principal_rafter_pitch_angle); 
        levelcut = principal_rafter_depth / Math.cos(90.degrees - principal_rafter_pitch_angle); 
        puts "main plumbcut " + length_sprintf(plumbcut) if $DEBUG_VERBOSE; 
        puts "main levelcut " + length_sprintf(levelcut) if $DEBUG_VERBOSE; 

        point_e=  [ p1.x ,p1.y + eave_length_trapezoidal_length +offset, 0]
        point_f=  [ p1.x + eave_length_base,p1.y + eave_length_trapezoidal_length +offset, 0]
        point_e2=  [p1.x+levelcut,p1.y+eave_length_trapezoidal_length+offset,0]
        point_f2=  [p1.x-levelcut+eave_length_base,p1.y+eave_length_trapezoidal_length+offset, 0]

        point_g=  [ p1.x + (eave_length_base/2),p1.y + eave_length_trapezoidal_length +offset+rise, 0]

        pts1 = point_e
        pts2 = point_f
        new_rafter = ents.add_cline(pts1,pts2);
        pts1 = point_g
        pts2 = point_K
        new_rafter = ents.add_cline(pts1,pts2);
        pts = []
        pts[0] = point_e
        pts[1] = point_g
        pts[2] =  [ p1.x + (eave_length_base/2),p1.y + eave_length_trapezoidal_length +offset+rise-plumbcut, 0]
        pts[3] = [ p1.x+ levelcut ,p1.y + eave_length_trapezoidal_length +offset, 0]
        pts[4] = pts[0]

        new_rafter = ents.add_face pts
        #new_rafter.reverse! 

        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        p_r_t = -(principal_rafter_thickness);
        new_rafter.pushpull p_r_t

        pts = []
        pts[0] = point_f
        pts[1] = point_g
        pts[2] =  [ p1.x + (eave_length_base/2),p1.y + eave_length_trapezoidal_length +offset+rise-plumbcut, 0]
        pts[3] = [ p1.x+eave_length_base- levelcut ,p1.y + eave_length_trapezoidal_length +offset, 0]
        pts[4] = pts[0]

        new_rafter = ents.add_face pts
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        new_rafter.pushpull p_r_t


        #draw construction lines from trapezoidal to principal rafters
        pts1 = point_e
        pts2 = point_B
        new_rafter = ents.add_cline(pts1,pts2);
        pts1 = point_f
        pts2 = point_C
        new_rafter = ents.add_cline(pts1,pts2);
      
      
        principal_rafter_thickness_2 = principal_rafter_thickness /2

        point_E1 = [ p1.x ,p1.y + (eave_length_trapezoidal/2)+principal_rafter_thickness_2 , 0]
        point_E2 = [ p1.x+levelcut ,p1.y + (eave_length_trapezoidal/2)+principal_rafter_thickness_2 , 0]
        point_E3 = [ p1.x+levelcut ,p1.y + (eave_length_trapezoidal/2)-principal_rafter_thickness_2 , 0]
        point_E4 = [ p1.x ,p1.y + (eave_length_trapezoidal/2)-principal_rafter_thickness_2 , 0]

        point_F1 = [ p1.x+ eave_length_base ,p1.y + (eave_length_trapezoidal/2)+principal_rafter_thickness_2 , 0]
        point_F2 = [ p1.x+ eave_length_base-levelcut,p1.y + (eave_length_trapezoidal/2)+principal_rafter_thickness_2 , 0]
        point_F3 = [ p1.x+ eave_length_base-levelcut,p1.y + (eave_length_trapezoidal/2)-principal_rafter_thickness_2 , 0]
        point_F4 = [ p1.x+ eave_length_base,p1.y + (eave_length_trapezoidal/2)-principal_rafter_thickness_2 , 0]

        point_KZ1 = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2)+principal_rafter_thickness_2 ,rise]
        point_KZ2 = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2)-principal_rafter_thickness_2 ,rise]
        point_KZ3 = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2)-principal_rafter_thickness_2 ,rise-plumbcut]
        point_KZ4 = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2)+principal_rafter_thickness_2 ,rise-plumbcut]
        #draw foot print of rafters & plumb face
        new_rafter = ents.add_face [point_E1,point_E2,point_E3,point_E4,point_E1]
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        new_rafter = ents.add_face [point_F1,point_F2,point_F3,point_F4,point_F1]
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        new_rafter = ents.add_face [point_KZ1,point_KZ2,point_KZ3,point_KZ4,point_KZ1] # plumb face
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        #draw left rafter sides
        new_rafter = ents.add_face [point_E1,point_KZ1,point_KZ4,point_E2,point_E1]
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        new_rafter = ents.add_face [point_E4,point_KZ2,point_KZ3,point_E3,point_E4]
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        #draw left rafter top & bottom
        new_rafter = ents.add_face [point_E1,point_KZ1,point_KZ2,point_E4,point_E1] # top
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        new_rafter = ents.add_face [point_E2,point_KZ4,point_KZ3,point_E3,point_E2] #bottom
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 

        #draw Right rafter sides
        new_rafter = ents.add_face [point_F1,point_KZ1,point_KZ4,point_F2,point_F1]
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        new_rafter = ents.add_face [point_F4,point_KZ2,point_KZ3,point_F3,point_F4]
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        #draw Right rafter top & bottom
        new_rafter = ents.add_face [point_F1,point_KZ1,point_KZ2,point_F4,point_F1] # top
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 
        new_rafter = ents.add_face [point_F2,point_KZ4,point_KZ3,point_F3,point_F2] # bottom
        new_rafter.material= Sketchup::Color.new(153,84,61) 
        new_rafter.back_material= Sketchup::Color.new(153,84,61) 

        # minor common rafter
        minor_run_y = (eave_length_trapezoidal/2)-principal_rafter_thickness_2;
        minor_rise = minor_run_y * Math.tan(minor_pitch_angle);
        #calculate the depth of the minor rafter
        minor_rafter_depth = plumbcut * Math.cos(minor_pitch_angle); 
        minor_levelcut = minor_rafter_depth / Math.cos(90.degrees - minor_pitch_angle); 
        minor_plumbcut = plumbcut

        puts "minor_rafter_depth " + length_sprintf(minor_rafter_depth) if $DEBUG_VERBOSE; 
        puts "minor_levelcut " + length_sprintf(minor_levelcut) if $DEBUG_VERBOSE; 

        point_H1 = [p1.x+ (eave_length_base/2)-principal_rafter_thickness_2,p1.y ,0]
        point_H2 = [p1.x+ (eave_length_base/2)-principal_rafter_thickness_2,p1.y+minor_levelcut,0]
        point_H3 = [p1.x+ (eave_length_base/2)+principal_rafter_thickness_2,p1.y+minor_levelcut ,0]
        point_H4 = [p1.x+ (eave_length_base/2)+principal_rafter_thickness_2,p1.y,0]

        point_MKZ1 = [p1.x+ (eave_length_base/2)-principal_rafter_thickness_2,p1.y+ (eave_length_trapezoidal/2)-principal_rafter_thickness_2 ,minor_rise]
        point_MKZ2 = [p1.x+ (eave_length_base/2)+principal_rafter_thickness_2,p1.y+ (eave_length_trapezoidal/2)-principal_rafter_thickness_2 ,minor_rise]
        point_MKZ3 = [p1.x+ (eave_length_base/2)+principal_rafter_thickness_2,p1.y+ (eave_length_trapezoidal/2)-principal_rafter_thickness_2 ,minor_rise-minor_plumbcut]
        point_MKZ4 = [p1.x+ (eave_length_base/2)-principal_rafter_thickness_2,p1.y+ (eave_length_trapezoidal/2)-principal_rafter_thickness_2 ,minor_rise-minor_plumbcut]

        #draw minor rafter foot print of rafters & plumb face
        new_rafter = ents.add_face [point_H1,point_H2,point_H3,point_H4,point_H1] # foot print
        new_rafter = ents.add_face [point_MKZ1,point_MKZ2,point_MKZ3,point_MKZ4,point_MKZ1] # plumbface
        #draw minor rafter sides
        new_rafter = ents.add_face [point_H1,point_MKZ1,point_MKZ4,point_H2,point_H1] 
        new_rafter = ents.add_face [point_H4,point_MKZ2,point_MKZ3,point_H3,point_H4] 
        #draw minor rafter top & bottom
        new_rafter = ents.add_face [point_H1,point_MKZ1,point_MKZ2,point_H4,point_H1] #top
        new_rafter = ents.add_face [point_H2,point_MKZ4,point_MKZ3,point_H3,point_H2] #bottom

        #draw a construction line on the bottom of the minor rafter to check for the internal plane
        point_H1_a = [p1.x+ (eave_length_base/2),p1.y+minor_levelcut ,0]
        point_H1_b = [p1.x+ (eave_length_base/2),p1.y+(eave_length_trapezoidal/2) ,rise-plumbcut]
        new_rafter = ents.add_cline point_H1_a, point_H1_b

        #draw construction lines for the internal plane
        #left
        pts1 = point_e2
        pts2 = [p1.x+ levelcut,p1.y+minor_levelcut ,0]
        new_rafter = ents.add_cline(pts1,pts2);
        #right
        pts1 = point_f2
        pts2 = [p1.x+ eave_length_base-levelcut,p1.y+minor_levelcut ,0]
        new_rafter = ents.add_cline(pts1,pts2);
        #bottom
        pts1 = [p1.x+ levelcut,p1.y+minor_levelcut ,0]
        pts2 = [p1.x+ eave_length_base-levelcut,p1.y+minor_levelcut ,0]
        new_rafter = ents.add_cline(pts1,pts2);
        #top
        inner_base_plane_length = eave_length_base - (levelcut *2);
        inner_base_plane_height = inner_base_plane_length * Math.tan(minor_plan_angle);
        outer_base_plane_height = inner_base_plane_length * Math.tan(eave_angle_t1);
        pts1 = [p1.x+eave_length_base- levelcut,p1.y+minor_levelcut + inner_base_plane_height,0]
        pts2 = [p1.x+levelcut,p1.y+minor_levelcut + inner_base_plane_height+outer_base_plane_height,0]
        new_rafter = ents.add_cline(pts1,pts2);

        mid_point_plumbcut = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2),rise- plumbcut];
      
      #roof_surface_rotation_angle

        roof_surface_rotation_angle_b = 90.degrees - roof_surface_rotation_angle;
        puts "roof_surface_rotation_angle " + roof_surface_rotation_angle.radians.to_s if $DEBUG_VERBOSE; 
        puts "roof_surface_rotation_angle_b " + roof_surface_rotation_angle_b.radians.to_s if $DEBUG_VERBOSE; 

        kgq_angle = dimension_angle_test(point_K,point_G,point_Q);
        puts "plane_rotation_hor " + kgq_angle.to_s if $DEBUG_VERBOSE; 
        g_rotate_angle = dimension_angle_test(point_Q,point_G,mid_point_z);
        puts "G_rotate_angle " + g_rotate_angle.to_s if $DEBUG_VERBOSE; 
        roof_sheathing_angle_right = Math.atan(Math.tan(minor_trapezoidal_plan_angle) / Math.cos(trapezoidal_angle_LM));
        roof_sheathing_angle_left = Math.atan(Math.tan(minor_trapezoidal_plan_angle2) / Math.cos(trapezoidal_angle_LM));
        puts "minor_trapezoidal_plan_angle_right " +minor_trapezoidal_plan_angle.radians.to_s if $DEBUG_VERBOSE; 
        puts "minor_trapezoidal_plan_angle_left " +minor_trapezoidal_plan_angle2.radians.to_s if $DEBUG_VERBOSE; 
        puts "roof_sheathing_angle_right " +roof_sheathing_angle_right.radians.to_s if $DEBUG_VERBOSE; 
        puts "roof_sheathing_angle_left " +roof_sheathing_angle_left.radians.to_s if $DEBUG_VERBOSE; 

        roof_sheathing_run_base = point_L.distance point_C;
        trapezoidal_rafter_length = roof_sheathing_run_base  * Math.tan(roof_sheathing_angle_right);
        a_run = mid_point.distance point_G 
        b_rafter = a_run * Math.cos(eave_angle_t1) 
        b_run = b_rafter * Math.cos(eave_angle_t1) 
        b_rise = b_rafter * Math.sin(eave_angle_t1) 
        s_run = trapezoidal_rafter_length * Math.cos(eave_angle_t2);
        s_rise = trapezoidal_rafter_length * Math.sin(eave_angle_t2);
        point_roof_surface =[p1.x+ (eave_length_base/2)+b_rise+s_run,p1.y+ (eave_length_trapezoidal/2)+b_run+s_rise,0];

        #draw real roof surface
        pts1 = point_B
        pts2 = point_roof_surface
        new_rafter = ents.add_cline(pts1,pts2);
        pts1 = point_C
        pts2 = point_roof_surface
        new_rafter = ents.add_cline(pts1,pts2);
        pts1 = point_L
        pts2 = point_roof_surface
        new_rafter = ents.add_cline(pts1,pts2);

        pts1 = point_G
        pts2 = point_roof_surface
        new_rafter = ents.add_cline(pts1,pts2);

        g_rotate_angle = g_rotate_angle.degrees;
        plane_rotation_hor = kgq_angle.degrees;
        plate_rotation = eave_angle_t1-plane_rotation_hor;

        g1_eave = principal_rafter_thickness_2 / Math.cos(roof_surface_rotation_angle_b)
        puts "g1_eave " + length_sprintf(g1_eave) if $DEBUG_VERBOSE; 
        g1_run = g1_eave * Math.cos(eave_angle_t1)
        g1_rise = g1_eave * Math.sin(eave_angle_t1)
        puts "g1_run " + length_sprintf(g1_run) if $DEBUG_VERBOSE; 
        puts "g1_rise " + length_sprintf(g1_rise) if $DEBUG_VERBOSE; 
        m = point_C.distance point_G;
        m_run = m - g1_eave;
        skewed_rafter_right = get_triangle_ASA(roof_surface_rotation_angle,m_run,roof_sheathing_angle_right)
        skewed_rafter_left = get_triangle_ASA(90.degrees + roof_surface_rotation_angle_b,m_run,roof_sheathing_angle_left)

        puts "skewed_rafter_right " + length_sprintf(skewed_rafter_right) if $DEBUG_VERBOSE; 
        puts "skewed_rafter_left " + length_sprintf(skewed_rafter_left) if $DEBUG_VERBOSE; 
        skewed_rafter_right_rise =  skewed_rafter_right * Math.sin(g_rotate_angle)
        skewed_rafter_left_rise =  skewed_rafter_left * Math.sin(g_rotate_angle)
        puts "skewed_rafter_right_rise " + length_sprintf(skewed_rafter_right_rise) if $DEBUG_VERBOSE; 
        puts "skewed_rafter_left_rise " + length_sprintf(skewed_rafter_left_rise) if $DEBUG_VERBOSE; 

        puts "plate_rotation  " +plate_rotation.radians.to_s if $DEBUG_VERBOSE; 
        skewed_beam_on_ground_plan =  g1_eave * Math.cos(plate_rotation )
        puts "skewed_beam_on_ground_plan " + length_sprintf(skewed_beam_on_ground_plan) if $DEBUG_VERBOSE; 

        c1a_run = levelcut * Math.tan(eave_angle_t1)
        c1b_run = levelcut * Math.tan(minor_trapezoidal_plan_angle - eave_angle_t1)
        c1 = c1a_run + c1b_run;
        c1_perpendicular = c1 * Math.cos(eave_angle_t1)
        angle_j1 = eave_angle_t1 - plane_rotation_hor;
        foot_print_g  = c1_perpendicular / Math.cos( angle_j1)
        puts "c1_perpendicular  " + length_sprintf(c1_perpendicular) if $DEBUG_VERBOSE; 
        puts "foot_print_g " + length_sprintf(foot_print_g) if $DEBUG_VERBOSE; 

        internal_y = c1_perpendicular * Math.cos(eave_angle_t1)
        internal_x = c1_perpendicular * Math.sin(eave_angle_t1)

        internal_point_L = [point_L.x - internal_x,point_L.y - internal_y,0];

        #calculate the length of the trapezoidal common rafter levecut     

        trap_rafter_depth = c1_perpendicular * Math.sin(trap_rafter_angle); 
        trap_levelcut = c1_perpendicular
      
      
        puts "trap_rafter_depth " + length_sprintf(trap_rafter_depth) if $DEBUG_VERBOSE; 
        puts "trap_levelcut " + length_sprintf(trap_levelcut) if $DEBUG_VERBOSE;   

        skewed_foot_print_length = foot_print_g

        #draw G foot print
        #angle_b = 90.degrees - kgq_angle.degrees;
        g_x = skewed_foot_print_length * Math.cos(90.degrees - plane_rotation_hor);
        g_y = skewed_foot_print_length * Math.sin(90.degrees - plane_rotation_hor);

        point_G_footprint_1= [ p1.x+ (eave_length_base/2)-g1_run ,p1.y + trapezoidal_span+g1_rise , 0]
        point_G_footprint_2= [ p1.x+ (eave_length_base/2)+g1_run ,p1.y + trapezoidal_span-g1_rise , 0]


        point_G_footprint_3= [ p1.x+ (eave_length_base/2)+g1_run -g_x,p1.y + trapezoidal_span-g1_rise -g_y, 0]
        point_G_footprint_4= [ p1.x+ (eave_length_base/2)-g1_run-g_x ,p1.y + trapezoidal_span+g1_rise-g_y , 0]

        mid_point_footprint_internal =   [ p1.x+ (eave_length_base/2) - g_x,p1.y + trapezoidal_span - g_y, 0]

        #new_rafter = ents.add_face [point_G_footprint_1,point_G_footprint_2,point_G_footprint_3,point_G_footprint_4,point_G_footprint_1]
        new_rafter = ents.add_cline point_G_footprint_2,point_G_footprint_3
        new_rafter = ents.add_cline point_G_footprint_1,point_G_footprint_4

        #draw trapezoidal skewed common rafter from point G

        #trap_run_1 =  ((eave_length_base/2)+g1_run) * Math.tan(eave_angle_t1) + (eave_length_trapezoidal/2)-principal_rafter_thickness_2    
        #trap_run_2 =  ((eave_length_base/2)-g1_run) * Math.tan(eave_angle_t1) + (eave_length_trapezoidal/2)-principal_rafter_thickness_2      
        m = point_K.distance point_G;
        m -= principal_rafter_thickness_2;
        trap_run_1 =  m + g1_rise;    
        trap_run_2 =  m - g1_rise;   
      
      
        puts "trap_run_1 " + length_sprintf(trap_run_1) if $DEBUG_VERBOSE; 
        puts "trap_run_2 " + length_sprintf(trap_run_2) if $DEBUG_VERBOSE; 

        trap_rise_1 = trap_run_1 * Math.tan(trapezoidal_angle_GK);
        trap_rise_2 = trap_run_2 * Math.tan(trapezoidal_angle_GK);      

        puts "trap_rise_1 " + length_sprintf(trap_rise_1) if $DEBUG_VERBOSE; 
        puts "trap_rise_2 " + length_sprintf(trap_rise_2) if $DEBUG_VERBOSE; 


        #top of skewed rafter
        point_GKZ1 = [p1.x+ (eave_length_base/2)- g1_run,p1.y+ (eave_length_trapezoidal/2)+principal_rafter_thickness_2 ,trap_rise_1]
        point_GKZ2 = [p1.x+ (eave_length_base/2)+ g1_run,p1.y+ (eave_length_trapezoidal/2)+principal_rafter_thickness_2 ,trap_rise_2]


        #bottom of skewed rafter
        tr = mid_point.distance point_G 
        plane_tilt = Math.atan((tr * Math.tan(plane_rotation_hor))/rise)
        puts "plane_tilt  " + plane_tilt.radians.to_s if $DEBUG_VERBOSE; 

        #Angle C = arctan( tan ( A )  sin ( D ))

        internal_plane_slope = Math.atan(Math.tan(trapezoidal_angle_LM) * Math.sin(eave_angle_t1));

        plane_tilt_C = 90.degrees - plane_tilt + internal_plane_slope;

        angle_a_law_of_sines = plane_tilt;
        angle_c_law_of_sines = plane_tilt_C;
        angle_b_law_of_sines = 180.degrees - angle_a_law_of_sines - angle_c_law_of_sines;

        side_a_law_of_sines = (plumbcut * Math.sin(plane_tilt)) / Math.sin(plane_tilt_C);
        side_b_law_of_sines = (plumbcut * Math.sin(angle_b_law_of_sines)) / Math.sin(plane_tilt_C);
        #h = a  sin( B )
        h = side_b_law_of_sines * Math.sin(angle_b_law_of_sines);
        p_run = h;
        p_rise = p_run / Math.tan(plane_tilt);
        puts "p_rise " + p_rise.to_s if $DEBUG_VERBOSE; 
        puts "skewed_plumb_cut_length " + length_sprintf(side_b_law_of_sines) if $DEBUG_VERBOSE; 

        skewed_plumb_cut_length = side_b_law_of_sines;          
     
      
      
=begin     
             
      # top of skewed rafter      foot print of rafter
     2 ------1
      |     |                       3 ------4 
      |     |                       2       1
      |     |
     4 ----- 3
=end


        x_1=  p1.x + (eave_length_base/2) + g1_run - g_x;
        x_2 = p1.x + (eave_length_base/2) - g1_run - g_x;

        point_G_footprint_3= [x_1,p1.y + trapezoidal_span - g1_rise - g_y,0]
        point_G_footprint_4= [x_2,p1.y + trapezoidal_span + g1_rise - g_y,0]


        p_rise = skewed_plumb_cut_length * Math.cos(plane_tilt) 
        puts "p_rise " + p_rise.to_s if $DEBUG_VERBOSE; 

        point_GKZ4 = [ point_G_footprint_3.x,p1.y + (eave_length_trapezoidal/2) + principal_rafter_thickness_2,trap_rise_2 - p_rise]
        point_GKZ3 = [ point_G_footprint_4.x,p1.y + (eave_length_trapezoidal/2) + principal_rafter_thickness_2,trap_rise_1 - p_rise]


        puts "point_G_footprint_1 " + point_sprintf(point_G_footprint_1) if $DEBUG_VERBOSE; 
        puts "point_G_footprint_2 " + point_sprintf(point_G_footprint_2) if $DEBUG_VERBOSE; 
        puts "point_G_footprint_3 " + point_sprintf(point_G_footprint_3) if $DEBUG_VERBOSE; 
        puts "point_G_footprint_4 " + point_sprintf(point_G_footprint_4) if $DEBUG_VERBOSE; 

        puts "point_GKZ1 " + point_sprintf(point_GKZ1) if $DEBUG_VERBOSE; 
        puts "point_GKZ2 " + point_sprintf(point_GKZ2) if $DEBUG_VERBOSE; 
        puts "point_GKZ3 " + point_sprintf(point_GKZ3) if $DEBUG_VERBOSE; 
        puts "point_GKZ4 " + point_sprintf(point_GKZ4) if $DEBUG_VERBOSE; 

        skewed_rafter_miter_angle_foot = dimension_angle_test(point_Q,point_G,mid_point_z);
        puts "skewed_rafter_miter_angle_foot = " + skewed_rafter_miter_angle_foot.to_s if $DEBUG_VERBOSE; 

        skewed_rafter_bevel_angle_foot = dimension_angle_test(point_G,point_roof_surface,point_L);
        puts "skewed_rafter_bevel_angle_foot = " + skewed_rafter_bevel_angle_foot.to_s if $DEBUG_VERBOSE; 

        puts "internal_plane_slope " + angle_sprintf(internal_plane_slope) if $DEBUG_VERBOSE; 

        skewed_rafter_miter_angle_top = dimension_angle_test(point_GKZ2,point_GKZ4,point_G_footprint_3) - 90;
        puts "skewed_rafter_miter_angle_foot = " + skewed_rafter_miter_angle_top.to_s if $DEBUG_VERBOSE; 

        skewed_r = dimension_angle_test(point_GKZ2,point_GKZ1,point_GKZ3) -90;
        puts "skewed_r = " + skewed_r.to_s if $DEBUG_VERBOSE; 
        skewed_r1 = dimension_angle_test(point_GKZ1,point_GKZ2,point_GKZ4) -90;
        puts "skewed_r1 = " + skewed_r1.to_s if $DEBUG_VERBOSE; 

        group2 = model_ents.add_group;
        ents2 = group2.entities;
        group2.name="Skewed Rafter";

        #draw skewed rafter top & bottom
        new_rafter = ents2.add_face [point_G_footprint_1,point_GKZ1,point_GKZ2,point_G_footprint_2,point_G_footprint_1] #top    
        new_rafter.material= Sketchup::Color.new(204,0,0)

        new_rafter = ents2.add_face [point_GKZ1,point_GKZ2,point_GKZ4,point_GKZ3,point_GKZ1] #plumbcut    
        new_rafter.material= Sketchup::Color.new(204,0,0)
      
        new_rafter = ents2.add_face [point_G_footprint_3,point_GKZ4,point_GKZ3,point_G_footprint_4,point_G_footprint_3] #bottom    not coplanar
        new_rafter.material= Sketchup::Color.new(204,0,0)

        new_rafter = ents2.add_face [point_G_footprint_3,point_GKZ4,point_GKZ2,point_G_footprint_2,point_G_footprint_3] #side
        new_rafter.material= Sketchup::Color.new(204,0,0)

        new_rafter = ents2.add_face [point_G_footprint_4,point_GKZ3,point_GKZ1,point_G_footprint_1,point_G_footprint_4] #side
        new_rafter.material= Sketchup::Color.new(204,0,0)

        new_rafter = ents2.add_face [point_G_footprint_1,point_G_footprint_2,point_G_footprint_3,point_G_footprint_4,point_G_footprint_1] #foot print    
        new_rafter.material= Sketchup::Color.new(204,0,0)
      
        # copy and rotate skewed rafter
        gp = group2.copy;
        origin = gp.transformation.origin;
        ctr =  point_G.to_a
        tro = Geom::Transformation.translation(origin-ctr)
        point_C1 = point_G;
        point_C1.x = point_C.x + radius;
        ctr =  point_C1.to_a
        trt=Geom::Transformation.translation(ctr-origin.to_a)
        gp.transform! trt
        gp.transform! tro

        rot_ang = trapezoidal_angle_GK; #g_rotate_angle;
        z_axis=gp.transformation.zaxis
        trr=Geom::Transformation.rotation(ctr,X_AXIS,rot_ang)
        gp.transform! trr

        rot_ang = 90.degrees - plane_tilt; #g_rotate_angle;
        z_axis=gp.transformation.zaxis
        trr=Geom::Transformation.rotation(ctr,Y_AXIS,rot_ang)
        gp.transform! trr
        
  if(show_internal_planes)
        #bottom
        pts = []
        pts[0] = [p1.x+ levelcut,p1.y+minor_levelcut ,0]
        pts[1] = [p1.x+ eave_length_base-levelcut,p1.y+minor_levelcut ,0]
        pts[2] = mid_point_plumbcut
        pts[3] = pts[0]
        new_rafter = ents.add_face pts
        #left
        pts = []
        pts[0] = [p1.x+ levelcut,p1.y+minor_levelcut ,0]
        pts[1] =  [p1.x+levelcut,p1.y+minor_levelcut + inner_base_plane_height+outer_base_plane_height,0]
        pts[2] = mid_point_plumbcut
        pts[3] = pts[0]
        new_rafter = ents.add_face pts
        #right
        pts = []
        pts[0] = [p1.x+eave_length_base - levelcut,p1.y+minor_levelcut ,0]
        pts[1] =  [p1.x+eave_length_base - levelcut,p1.y+minor_levelcut + inner_base_plane_height,0]
        pts[2] = mid_point_plumbcut
        pts[3] = pts[0]
        new_rafter = ents.add_face pts
        #top
        pts = []
        pts[0] =  [p1.x+eave_length_base - levelcut,p1.y+minor_levelcut + inner_base_plane_height,0]
        pts[1] = [p1.x+levelcut,p1.y+minor_levelcut + inner_base_plane_height+outer_base_plane_height,0]
        pts[2] = mid_point_plumbcut
        pts[3] = pts[0]
        new_rafter = ents.add_face pts
        # internal plane line of skewed rafter
        new_rafter = ents.add_line mid_point_footprint_internal,mid_point_plumbcut;
        new_rafter = ents.add_line internal_point_L,mid_point_plumbcut; 
      end
 
        group3 = model_ents.add_group;
        group3.name="Hip Rafter Left";
 
        # draw hip rafters for principal rafters -- left
        dx = 1;
        dy = 1;
        draw_trapezoidal_hip(dx,dy,group3,p1,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,principal_rafter_thickness,principal_rafter_depth,major_run,minor_run);
        
        group4 = model_ents.add_group;
        group4.name="Hip Rafter Right";
 
        dx = -1;
        dy = 1;
        p1.x += eave_length_base;
        draw_trapezoidal_hip(dx,dy,group4,p1,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,principal_rafter_thickness,principal_rafter_depth,major_run,minor_run);
        major_plan_angle2 

=begin
        needs lots of work
        dx = -1;
        dy = -1;
        p1.y += eave_length_trapezoidal;
        draw_trapezoidal_hip(dx,dy,ents,p1,eave_angle_trapezoidal,major_pitch_angle,trapezoidal_angle_LM,major_trapezoidal_plan_angle,minor_trapezoidal_plan_angle,
        principal_rafter_thickness,principal_rafter_depth,major_run,trapezoidal_run);
=end
 
     

      Sketchup.active_model.rendering_options["ModelTransparency"]=false;
      #Sketchup.active_model.rendering_options["ModelTransparency"]=true if show_internal_planes || show_roof_planes;
      view = Sketchup.active_model.active_view 
      view.field_of_view = 30
      #result = Sketchup.send_action 10501; # set view to Top
      view.zoom_extents 

      group.to_component 
      model.commit_operation

   
end # end function

# draw hip rafters for principal rafters
def draw_trapezoidal_hip(dx,dy,group,origin,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,rafter_thickness,rafter_depth,
  major_run,minor_run)   

      @entity = group.entities;

      #S = major_pitch_angle
      #SS =minor_pitch_angle
      hip_angle = Math.atan(Math.tan(major_pitch_angle)* Math.sin(major_plan_angle))
      hip_run_length = major_run / Math.cos(major_plan_angle);
      #S Hip Rafter Offset
      hrom_S = ((rafter_thickness * Math.sin(major_plan_angle)) / Math.sin(180.degrees - major_plan_angle - minor_plan_angle)) * Math.cos(minor_plan_angle);
      #SS Hip Rafter Offset
      hrom_SS = ((rafter_thickness * Math.sin(minor_plan_angle)) / Math.sin(180.degrees - major_plan_angle - minor_plan_angle)) * Math.cos(major_plan_angle);
      #S Hip Rafter Offset Run
      hror_S = hrom_S / Math.sin(major_plan_angle);
      #SS Hip Rafter OffsetRun
      hror_SS = hrom_SS / Math.sin(minor_plan_angle);
      
      plumbcut = rafter_depth / Math.cos(major_pitch_angle); 
      levelcut = rafter_depth / Math.cos(90.degrees - major_pitch_angle); 
      minor_rafter_depth = plumbcut * Math.cos(minor_pitch_angle); 
      minor_levelcut = minor_rafter_depth / Math.cos(90.degrees - minor_pitch_angle); 
      minor_plumbcut = plumbcut;      
      
      point_S = [origin.x,origin.y+(hror_S*dy),0]; # major hip rafter side planing point at eave line
      point_SS = [origin.x+(hror_SS*dx),origin.y,0]; # minor hip rafter side planing point at eave line

      run_y = minor_run - (rafter_thickness/2);
      run_x = major_run - ((rafter_thickness/2) /Math.tan(minor_plan_angle));
      puts "run_x = " + run_x.to_s if $DEBUG_VERBOSE; 
      puts "run_y = " + run_y.to_s if $DEBUG_VERBOSE; 
      
      planning_point_x = major_run - (rafter_thickness/2);
      planning_point_y = minor_run - (rafter_thickness/2);
      planning_point_z = planning_point_x * Math.tan(major_pitch_angle); 
      planning_point_z2 = planning_point_y * Math.tan(minor_pitch_angle); 
      planning_point_z_internal = planning_point_z2-plumbcut;
 
      major_plan_hip_offset_point_x = ((run_y- hror_S)* Math.tan(major_plan_angle));
      major_plan_hip_offset_point_y = minor_run - (rafter_thickness/2);
      
      minor_plan_hip_offset_point_x = major_run - (rafter_thickness/2);      
      minor_plan_hip_offset_point_y = ((major_run - (rafter_thickness/2)- hror_SS)* Math.tan(minor_plan_angle));     

      point_K = [origin.x+(run_x*dx),origin.y+(run_y*dy),0]; # center point of hip rafter on ground level
      point_KZ = [origin.x+(run_x*dx),origin.y+(run_y*dy),run_x*Math.tan(major_pitch_angle)];# center point of hip rafter at hip rafter rise/elevation

      point_Major_HS = [origin.x+(major_plan_hip_offset_point_x * dx),origin.y+(major_plan_hip_offset_point_y*dy),0];
      point_Minor_HSS = [origin.x+(minor_plan_hip_offset_point_x*dx),origin.y+(minor_plan_hip_offset_point_y*dy),0];
     
     point_planning_Z = [origin.x+(planning_point_x*dx),origin.y+(planning_point_y*dy),planning_point_z];
     point_planning_Z2 = [origin.x+(planning_point_x*dx),origin.y+(planning_point_y*dy),planning_point_z2];
      point_planning_Z2_internal = [origin.x+(planning_point_x*dx),origin.y+(planning_point_y*dy),planning_point_z_internal];
  

      point_Major_HZS = [origin.x+(major_plan_hip_offset_point_x*dx),origin.y+(major_plan_hip_offset_point_y*dy),major_plan_hip_offset_point_x*Math.tan(major_pitch_angle)];
      point_Minor_HZSS = [origin.x+(minor_plan_hip_offset_point_x*dx),origin.y+(minor_plan_hip_offset_point_y*dy),minor_plan_hip_offset_point_y*Math.tan(minor_pitch_angle)];
      point_Major_HZS_internal = [origin.x+(major_plan_hip_offset_point_x*dx),origin.y+(major_plan_hip_offset_point_y*dy),major_plan_hip_offset_point_x * Math.tan(major_pitch_angle)-plumbcut];
      point_Minor_HZSS_internal = [origin.x+(minor_plan_hip_offset_point_x*dx),origin.y+(minor_plan_hip_offset_point_y*dy),minor_plan_hip_offset_point_y * Math.tan(minor_pitch_angle)-plumbcut];
      point_KZ_internal = [origin.x+(run_x*dx),origin.y+(run_y*dy),run_x * Math.tan(major_pitch_angle)-plumbcut];

      
      point_Major_HS_internal = [origin.x+(levelcut*dx),origin.y+((minor_levelcut+hror_S)*dy),0];
      point_Minor_HSS_internal = [origin.x+((levelcut+hror_SS)*dx),origin.y+(minor_levelcut*dy),0];
      point_internal = [origin.x+(levelcut*dx),origin.y+(minor_levelcut*dy),0]; 
   
=begin 
      new_rafter = entity.add_line point_KZ, point_K
      new_rafter = entity.add_line origin, point_KZ

      new_rafter = entity.add_line point_S,point_Major_HS
      new_rafter = entity.add_line point_SS,point_Minor_HSS

      new_rafter = entity.add_line point_Major_HZS,point_Major_HS
      #new_rafter = entity.add_line point_Minor_HZSS,point_Minor_HSS

      new_rafter = entity.add_line point_Major_HZS,point_S
      new_rafter = entity.add_line point_Minor_HZSS,point_SS
      
        new_rafter = entity.add_line point_planning_Z,point_planning_Z2
=end


      @new_rafter = @entity.add_face [origin,point_S,point_Major_HZS,point_KZ,origin] #top of rafter    left side
       #status = @new_rafter.reverse!
       status = @new_rafter.back_material=[255,216,101]       
      @new_rafter.material= Sketchup::Color.new(255,216,101)
      
      if(point_planning_Z2 == point_KZ)
        @new_rafter = @entity.add_face [origin,point_SS,point_Minor_HZSS,point_KZ,origin] #top of rafter    right side
      else
        @new_rafter = @entity.add_face [origin,point_SS,point_Minor_HZSS,point_planning_Z2,point_KZ,origin] #top of rafter    right side
      end
      
      
       status = @new_rafter.back_material=[255,216,101]
      @new_rafter.material= Sketchup::Color.new(255,216,101)

      @new_rafter = @entity.add_face [point_S,point_Major_HS_internal,point_Major_HZS_internal,point_Major_HZS,point_S] # left side
       status = @new_rafter.back_material=[255,216,101]
      @new_rafter.material= Sketchup::Color.new(255,216,101)

      @new_rafter = @entity.add_face [point_SS,point_Minor_HSS_internal,point_Minor_HZSS_internal,point_Minor_HZSS,point_SS] # right side
       status = @new_rafter.back_material=[255,216,101]
      @new_rafter.material= Sketchup::Color.new(255,216,101)

      @new_rafter = @entity.add_face [ point_internal,point_Major_HS_internal,point_Major_HZS_internal,point_KZ_internal,point_internal] # left side bottom
       status = @new_rafter.back_material=[255,216,101]
      @new_rafter.material= Sketchup::Color.new(255,216,101)
       
     if(point_planning_Z2_internal == point_KZ_internal)
      @new_rafter = @entity.add_face [ point_internal,point_Minor_HSS_internal,point_Minor_HZSS_internal,point_KZ_internal,point_internal] # right side bottom
     else
         @new_rafter = @entity.add_face [ point_internal,point_Minor_HSS_internal,point_Minor_HZSS_internal,point_planning_Z2_internal,point_KZ_internal,point_internal] # right side bottom
    end
      
       status = @new_rafter.back_material=[255,216,101]
      @new_rafter.material= Sketchup::Color.new(255,216,101)

      @new_rafter = @entity.add_face [origin,point_S,point_Major_HS_internal,point_internal,point_Minor_HSS_internal,point_SS,origin] # foot print bottom
       status = @new_rafter.back_material=[255,216,101]
      @new_rafter.material= Sketchup::Color.new(255,216,101)

      @new_rafter = @entity.add_face [point_Major_HZS_internal,point_Major_HZS,point_KZ,point_KZ_internal,point_Major_HZS_internal]; #plumcut left side
       status = @new_rafter.back_material=[255,216,101]
      @new_rafter.material= Sketchup::Color.new(255,216,101)

      @new_rafter = @entity.add_face [point_Minor_HZSS_internal,point_Minor_HZSS,point_planning_Z2,point_planning_Z2_internal,point_Minor_HZSS_internal]; #plumcut right side on minor rafter
       status = @new_rafter.back_material=[255,216,101]
      @new_rafter.material= Sketchup::Color.new(255,216,101)
      
    if(point_planning_Z2_internal != point_KZ_internal)
        @new_rafter = @entity.add_face [point_KZ,point_KZ_internal,point_planning_Z2_internal,point_planning_Z2,point_KZ]; #plumcut right side  on main rafter
        status = @new_rafter.back_material=[255,216,101]
        @new_rafter.material= Sketchup::Color.new(255,216,101)
    end

      
=begin      
       # copy and rotate skewed rafter
        @@gp = group.copy;
        @origin2 = @@gp.transformation.origin;
        @ctr =  origin.to_a
        @tro = Geom::Transformation.translation(@origin2-@ctr)
        @point_C1 = origin;
        #@point_C1.x += 50*dx*-1;
        @ctr =  @point_C1.to_a
        @trt=Geom::Transformation.translation(@ctr-@origin2.to_a)
        @@gp.transform! @trt
        @@gp.transform! @tro
    

        @rot_ang = 180.degrees; 
        @z_axis=@@gp.transformation.zaxis
        @trr=Geom::Transformation.rotation(@ctr,Z_AXIS,@rot_ang)
        @@gp.transform! @trr
   
=end

  end
    
 
end # class TrapezoidalTimberTool

end #module ROOFFRAMING

file_loaded("RoofFraming/Trapezoidal_Timber.rb")
