# Copyright 2012 Sim Ayers
# version 1.0
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------


module ROOFFRAMING

class UnequalHipTool < TetrahedronTool

$principal_rafter_pitch = 9.0;
$minor_rafter_pitch = 12.0;
$deck_angle_trapezoidal = 90.0;
$eave_length_base = 196.0;
$eave_length_trapezoidal = 96.0;
$roof_planes = "no";
$framing_angles = "no";
$show_internal = "no"
$major_rafter_depth = 9.5;
$major_rafter_width = 1.5;
$hip_rafter_width = 3.5;
$jack_rafter_spacing = 24;
$ridge_width = 3.5;
$hip_rafter_depth = 11.5;
$seatcut_length = 3.5;
$overhang_run = 24;

def initialize
       
    $principal_rafter_pitch = 9.0 if not $principal_rafter_pitch;
    $minor_rafter_pitch = 12.0 if not $minor_rafter_pitch;
    
    $eave_length_base = 104.0 if not $eave_length_base ;
    $eave_length_trapezoidal = 196.0 if not $eave_length_trapezoidal;
    $deck_angle_trapezoidal = 90.0 if not $deck_angle;
    $roof_planes = "no" if not $roof_planes;
    $framing_angles = "no" if not $framing_angles;
    $show_internal = "no" if not $show_internal;
    $major_rafter_depth = 9.5 if not $major_rafter_depth;
    $major_rafter_width = 1.5 if not $major_rafter_width;
    
    $hip_rafter_depth = 11.5 if not $hip_rafter_depth;
    $hip_rafter_width = 3.5 if not $hip_rafter_width;
    $jack_rafter_spacing =24 if not $jack_rafter_spacing;
    $ridge_width = 3.5 if not $ridge_width;
   $seatcut_length = 3.5 if not $seatcut_length;
  $overhang_run = 24 if not $overhang_run;
    

    # First prompt for the dimensions.  This is done using the inputbox method
        
    # The first step is to create some arrays which contain the prompts
    # and default values.
    prompts = ["Major Rafter Pitch","Minor Rafter Pitch","Roof Eave Angle","Major Rafter Run",
    "Depth of Major Rafter","Width of Major Rafter","Depth of Hip Rafter","Width of Hip Rafter",
    "Ridge Width", "Jack Rafter Spacing","Major Seatcut Length","Major Overhang Run"]
    
    #default values
    values = [$principal_rafter_pitch,$minor_rafter_pitch,$deck_angle_trapezoidal,$eave_length_base,$major_rafter_depth,$major_rafter_width,
    $hip_rafter_depth,$hip_rafter_width,$ridge_width,$jack_rafter_spacing,$seatcut_length,$overhang_run]
    
    popups = [nil,nil,nil, nil, nil,nil,nil,nil,nil,nil,nil,nil]
    
    # Now display the inputbox
    results = inputbox prompts, values,popups, "UnequalPitched Hip Rafter";

    return if not results # This means that the user cancelled the operation
    $principal_rafter_pitch,$minor_rafter_pitch,$deck_angle_trapezoidal,$eave_length_base,$major_rafter_depth,$major_rafter_width,
    $hip_rafter_depth,$hip_rafter_width,$ridge_width,$jack_rafter_spacing,$seatcut_length,$overhang_run= results
    
     if $deck_angle_trapezoidal < 90.0
     	UI.messagebox("Trapezoidal Eave Angle must be greater than or equal to 90.")
     	return
    end
     if $deck_angle_trapezoidal > 179.0
     	UI.messagebox("Trapezoidal Eave Angle must be less than 180.")
     	return
    end
    if ($principal_rafter_pitch <= 0.0 )
     	UI.messagebox("Principal Rafter Pitch can not be 0.")
     	return
    end
    if ($eave_length_base <= 0.0 )
     	UI.messagebox("Eave Length Base can not be 0.")
     	return
    end
       
    show = false 
    show_roof_planes = false
    show_roof_planes = true if($roof_planes =="yes");
    show_roof_angles = false
    show_roof_angles = true if($framing_angles =="yes"); 
    show_internal_planes = false
    show_internal_planes = true if($show_internal =="yes");
    
    draw_unequal_hip($principal_rafter_pitch,$minor_rafter_pitch,$deck_angle_trapezoidal,$eave_length_base,$hip_rafter_depth,$hip_rafter_width,$ridge_width,$jack_rafter_spacing,
    $seatcut_length,$overhang_run,$major_rafter_depth,$major_rafter_width);     
     
end
 
 # draw the trapezoidal ground plan folded
def draw_unequal_hip(principal_rafter_pitch,minor_rafter_pitch,deck_angle,eave_length_base,hip_rafter_depth,hip_rafter_width,ridge_width,jack_rafter_spacing,
  seat_cut_length,overhang_run,major_rafter_depth,major_rafter_width)  

        model = Sketchup.active_model;
        # this enables undo of the whole operation as a unit
        model.start_operation "Trapezoidal Timber";

        # create a new group in which we will draw our dimension entities
        model_ents = model.active_entities;
        group = model_ents.add_group;
        ents = group.entities;
        group.name="Trapezoidal Timber";
        wplayer=model.layers.add("Trapezoidal Timber");
        wplayer.visible=true;
        group.layer=wplayer;
        group.casts_shadows=false;
        group.receives_shadows=false;

      $DEBUG_VERBOSE = false;

        eave_angle = 90.degrees
        eave_angle_trapezoidal = deck_angle.degrees;
        eave_angle_t1 =  eave_angle_trapezoidal - 90.degrees;
        eave_angle_t2 = 90.degrees - eave_angle_t1;
        p1 = [0,0,0];
        jack_rafter_width =  major_rafter_width;
        
       principal_rafter_pitch_angle = Math.atan(principal_rafter_pitch / 12);     	
        minor_rafter_pitch99 = Math.atan(minor_rafter_pitch / 12); 
        major_run = eave_length_base;
        major_rise = major_run * Math.tan(principal_rafter_pitch_angle);
         eave_length_base  = major_run *2
        minor_run = major_rise / Math.tan(minor_rafter_pitch99 );
        #minor_run = eave_length_trapezoidal /2;   
        eave_length_trapezoidal = minor_run *2;

        eave_length_trapezoidal_length = eave_length_trapezoidal + (eave_length_base * Math.tan(eave_angle_t1));

 
        major_pitch_angle = principal_rafter_pitch_angle;     
        minor_pitch_angle = Math.atan(major_rise / minor_run);

        plan_angles = Array.new(2);
        plan_angles = find_plan_angle(major_pitch_angle,minor_pitch_angle,eave_angle);   
        major_plan_angle = plan_angles[0]; 
        minor_plan_angle = plan_angles[1]; 


        hip_angle = Math.atan(Math.tan(principal_rafter_pitch_angle)* Math.sin(major_plan_angle));
        major_side = minor_run;
        minor_side = major_run;
        hip_run_length = major_side / Math.cos(major_plan_angle);	

        puts "new unequal pitched hip dimensions " if $DEBUG_VERBOSE;


        eave_angle_d = angle_sprintf(eave_angle_trapezoidal );
        puts "eave angle trapezoidal = " + eave_angle_d if $DEBUG_VERBOSE;
        eave_angle_d = angle_sprintf(eave_angle_t1 );
        puts "eave angle trapezoidal t1 = " + eave_angle_d if $DEBUG_VERBOSE;
        eave_angle_d = angle_sprintf(eave_angle_t2 );
        puts "eave angle trapezoidal t2 = " + eave_angle_d if $DEBUG_VERBOSE;

        major_d = angle_sprintf(major_plan_angle);
        minor_d = angle_sprintf(minor_plan_angle);
        puts "major d = " + major_d if $DEBUG_VERBOSE;
        puts "minor d = " + minor_d if $DEBUG_VERBOSE;


        principal_rafter_pitch_angle_d = angle_sprintf(principal_rafter_pitch_angle);
        minor_pitch_angle_d = angle_sprintf(minor_pitch_angle);
        puts "principal rafter angle = " + principal_rafter_pitch_angle_d if $DEBUG_VERBOSE;
        puts "minor pitch angle = " + minor_pitch_angle_d if $DEBUG_VERBOSE;
        hip_angle_d = angle_sprintf(hip_angle);
        puts "hip rafter angle = " + hip_angle_d if $DEBUG_VERBOSE;
        puts "hip run = " + length_sprintf(hip_run_length) if $DEBUG_VERBOSE;

        m = length_sprintf(major_side);
        puts "major_side = " + m if $DEBUG_VERBOSE;
        m = length_sprintf(minor_side);
        puts "minor_side = " + m if $DEBUG_VERBOSE;

        dd = major_plan_angle; # plan angle
        ss = principal_rafter_pitch_angle; # roof pitch angle
        r1 = Math.atan(Math.tan(ss) * Math.sin(dd)); # hip rafter angle
        ee = Math.atan(Math.cos(ss) / Math.tan(dd)); # jack rafter angle
        bb = Math.atan(Math.sin(r1) / Math.tan(dd)); # hip rafter backing angle
        run = (eave_length_base/2); # common rafter run
        rise = run * Math.tan(ss); # major common rafter rise

        major_rafter_width_2 = major_rafter_width /2
        ridge_width = ridge_width;
        ridge_width_2  = ridge_width /2;
        major_height_above_plate = 0;
        rafter_rise = rise;
        major_rafter_depth =  major_rafter_depth;
        major_rafter_run = eave_length_base/2;
       rafter_rise = major_rafter_run * (Math.tan(major_pitch_angle));
       
       minor_rafter_run = rafter_rise /(Math.tan(minor_pitch_angle));
		     
       minor_rafter_length = minor_rafter_run / (Math.cos(minor_pitch_angle));
		    major_rafter_length = major_rafter_run / (Math.cos(major_pitch_angle));
		     
       
        #major_rafter_length = major_rafter_depth / (Math.cos(major_pitch_angle));
		    #minor_rafter_length = major_rafter_depth / (Math.cos(minor_pitch_angle));
	
        major_plumb_height = major_rafter_depth / (Math.cos(major_pitch_angle));
		    minor_plumb_height = major_rafter_depth / (Math.cos(minor_pitch_angle));
		    major_height_below_plate = seat_cut_length * (Math.tan(major_pitch_angle));
		    major_height_above_plate = major_plumb_height - major_height_below_plate;
		    
        hap = major_height_above_plate ;
		    ridge_depth = major_plumb_height;
        top_of_ridge = rafter_rise + major_height_above_plate -((ridge_width*0.5) * Math.tan(major_pitch_angle));
        
		    major_ridge_run = ((ridge_width * Math.sin(minor_pitch_angle)) / Math.sin(Math::PI - major_pitch_angle - minor_pitch_angle)) * Math.cos(major_pitch_angle);
		    minor_ridge_run = ((ridge_width * Math.sin(major_pitch_angle)) / Math.sin(Math::PI - major_pitch_angle - minor_pitch_angle)) * Math.cos(minor_pitch_angle);
				
        minor_ridge_run = (ridge_width * 0.5)/ Math.tan(major_plan_angle);

		    minor_rafter_to_ridge = minor_rafter_length - (minor_ridge_run / Math.cos(minor_pitch_angle));
		    major_rafter_to_ridge = major_rafter_length - (ridge_width / Math.cos(major_pitch_angle));
		 
     	 d =  major_plan_angle;
		   dd = minor_plan_angle;
		   s =  major_pitch_angle;
		   ss = minor_pitch_angle;
		     
		   offset_HROM = ((hip_rafter_width * Math.sin(d)) / Math.sin(Math::PI - d - dd)) * Math.cos(dd); # Hip_Rafter_Offset_Main_Perpendicular
		   offset_HROA = ((hip_rafter_width * Math.sin(dd)) / Math.sin(Math::PI - d - dd)) * Math.cos(d); # Hip_Rafter_Offset_Adjacent_Perpendicular
		   offset_HRORM = offset_HROM / Math.sin(d); # # Hip_Rafter_Offset_Main_along eave
		   offset_HRORA = offset_HROA / Math.sin(dd);
		   offset_HRORMR = offset_HRORM * Math.sin(dd);
		   offset_HRORMR_Hip_Rafter = offset_HRORMR / Math.cos(hip_angle);
		  
      major_sheathing_angle = Math.atan(Math.tan(major_plan_angle) / Math.cos(major_pitch_angle));
		  minor_sheathing_angle = Math.atan(Math.tan(minor_plan_angle) / Math.cos(minor_pitch_angle));
		      
		       	
		   major_jack_rafter_length_diff = jack_rafter_spacing * (Math.tan(major_sheathing_angle));
		   minor_jack_rafter_length_diff = jack_rafter_spacing * (Math.tan(minor_sheathing_angle));
		  p2m = Math.atan(Math.cos(s) / Math.tan(d));
		  p2a = Math.atan(Math.cos(ss) / Math.tan(dd));
		  jack_JRODM = offset_HRORM * Math.tan(Math::PI/2 - p2m);
		  jack_JRODA = offset_HRORA * Math.tan(Math::PI/2 - p2a);
		 	jack_JRLM = (Math.tan(Math::PI/2 - p2m) * (jack_rafter_spacing + (jack_rafter_width* 0.5))) - jack_JRODM;
		  jack_JRLA = (Math.tan(Math::PI/2 - p2a) * (jack_rafter_spacing + (jack_rafter_width* 0.5))) - jack_JRODA;
		    
          

        mid_point = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2),0];
        mid_point_z = [p1.x+ (eave_length_base/2),p1.y+ (eave_length_trapezoidal/2),rise+hap];

        trapezoidal_span = eave_length_trapezoidal + ((eave_length_base/2) * Math.tan(eave_angle_t1));

        mid_point_t = [p1.x+ (eave_length_base/2),p1.y+ trapezoidal_span,0];

        point_A=  [p1.x,p1.y,0];
        point_B=  [ p1.x ,p1.y + eave_length_trapezoidal_length ,0];
        point_C = [ p1.x + eave_length_base ,p1.y + eave_length_trapezoidal ,0];
        point_D = [ p1.x + eave_length_base ,p1.y,0];
        point_E = [ p1.x ,p1.y + (eave_length_trapezoidal/2),0];
        point_F = [ p1.x+ eave_length_base ,p1.y + (eave_length_trapezoidal/2),0];
        point_G = [ p1.x+ (eave_length_base/2) ,p1.y + trapezoidal_span,0];
        point_H = [ p1.x+ (eave_length_base/2) ,p1.y, 0];
        point_K = mid_point;

           
        # draw trapezoidal ground plan
        pts = [];
        pts[0] = point_A;
        pts[1] = point_B;
        pts[2] = point_C;
        pts[3] = point_D;
        pts[4] = pts[0];
        #new_rafter = ents.add_face pts
        new_rafter = ents.add_line pts[0], pts[1]   
        new_rafter = ents.add_line pts[1], pts[2]   
        new_rafter = ents.add_line pts[2], pts[3]   
        new_rafter = ents.add_line pts[3], pts[0]   

     
        # draw rise

        pts1 = mid_point
        pts2 = mid_point_z
        new_rafter = ents.add_cline(pts1,pts2);    

        # draw hip rafter runs

        pts1 = point_A
        pts2 = mid_point
        new_rafter = ents.add_cline(pts1,pts2);    
   
        pts1 = point_D
        pts2 = mid_point
        new_rafter = ents.add_line pts1, pts2    
        
         # draw rafter runs
        pts1 = point_E;
        pts2 = mid_point;
        new_rafter = ents.add_cline(pts1,pts2);
        pts1 = point_F;
        pts2 = mid_point;
        new_rafter = ents.add_cline(pts1,pts2);     
        pts1 = point_H;
        pts2 = mid_point;
        new_rafter = ents.add_cline(pts1,pts2);    
        
        
        point_A=  [p1.x,p1.y,hap];
        point_B=  [ p1.x ,p1.y + eave_length_trapezoidal_length ,hap];
        point_C = [ p1.x + eave_length_base ,p1.y + eave_length_trapezoidal ,hap];
        point_D = [ p1.x + eave_length_base ,p1.y,hap];
        point_E = [ p1.x ,p1.y + (eave_length_trapezoidal/2),hap];
        point_F = [ p1.x+ eave_length_base ,p1.y + (eave_length_trapezoidal/2),hap];
        point_G = [ p1.x+ (eave_length_base/2) ,p1.y + trapezoidal_span,hap];
        point_H = [ p1.x+ (eave_length_base/2) ,p1.y, hap];
        point_K = mid_point;


        # draw hip rafter 

        pts1 = point_A
        pts2 = mid_point_z
        new_rafter = ents.add_line pts1, pts2    
   
        pts1 = point_D
        pts2 = mid_point_z
        new_rafter = ents.add_line pts1, pts2
       
        

        # draw rafters 
        pts1 = point_E
        pts2 = mid_point_z
        new_rafter = ents.add_cline(pts1,pts2);
  
        pts1 = point_F
        pts2 = mid_point_z
        new_rafter = ents.add_cline(pts1,pts2);
   
        pts1 = point_H
        pts2 = mid_point_z
        new_rafter = ents.add_cline(pts1,pts2);

  
      
       
      
        # draw principal rafters above trapezoidal
        #major_rafter_depth,major_rafter_width
        #calculate the length of the rafter plumbcut
        plumbcut = major_rafter_depth / Math.cos(principal_rafter_pitch_angle); 
        puts "main plumbcut " + length_sprintf(plumbcut) if $DEBUG_VERBOSE; 
           
  
        seatcut = seat_cut_length;
        
        point_E1a = [ p1.x ,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM , hap]
        point_E2a = [ p1.x ,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM , 0]
        point_E3a = [ p1.x+seatcut ,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,0]
        
        point_E1b = [ p1.x ,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM , hap]
        point_E2b = [ p1.x ,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM , 0]
        point_E3b = [ p1.x+seatcut ,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,0]
     
       point_F1a = [ p1.x+ eave_length_base  ,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM , hap]
        point_F2a = [ p1.x + eave_length_base ,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM , 0]
        point_F3a = [ p1.x-seatcut+ eave_length_base  ,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,0]
        
        point_F1b = [ p1.x + eave_length_base ,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM , hap]
        point_F2b = [ p1.x + eave_length_base ,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM , 0]
        point_F3b = [ p1.x-seatcut+ eave_length_base  ,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,0]
     
        major_overhang_rise = overhang_run * Math.tan(major_pitch_angle);
		    minor_overhang_rise = major_overhang_rise / Math.tan(minor_pitch_angle);
		      
        point_OE1a = [ p1.x-overhang_run,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,hap-major_overhang_rise]
        point_OE2a = [ p1.x-overhang_run,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,hap-major_overhang_rise-plumbcut]
        point_OE3a = [ p1.x ,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,-major_height_below_plate ]
        
        point_OE1b = [ p1.x-overhang_run,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,hap-major_overhang_rise]
        point_OE2b = [ p1.x-overhang_run,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,hap-major_overhang_rise-plumbcut]
        point_OE3b = [ p1.x ,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,-major_height_below_plate ]
       
       point_OF1a = [ p1.x+overhang_run+ eave_length_base,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,hap-major_overhang_rise]
        point_OF2a = [ p1.x+overhang_run+ eave_length_base,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,hap-major_overhang_rise-plumbcut]
        point_OF3a = [ p1.x+ eave_length_base,p1.y + (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,-major_height_below_plate ]
        
        point_OF1b = [ p1.x+overhang_run+ eave_length_base,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,hap-major_overhang_rise]
        point_OF2b = [ p1.x+overhang_run+ eave_length_base,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,hap-major_overhang_rise-plumbcut]
        point_OF3b = [ p1.x+ eave_length_base,p1.y + (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,-major_height_below_plate ]
     

        point_KZ1 = [p1.x+ (eave_length_base/2)-ridge_width_2 ,p1.y+ (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,top_of_ridge]
        point_KZ2 = [p1.x+ (eave_length_base/2)-ridge_width_2 ,p1.y+ (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,top_of_ridge]
        point_KZ3 = [p1.x+ (eave_length_base/2)-ridge_width_2 ,p1.y+ (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,top_of_ridge-plumbcut]
        point_KZ4 = [p1.x+ (eave_length_base/2)-ridge_width_2 ,p1.y+ (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,top_of_ridge-plumbcut]
        
        point_KZ1R = [p1.x+ (eave_length_base/2)+ridge_width_2 ,p1.y+ (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,top_of_ridge]
        point_KZ2R = [p1.x+ (eave_length_base/2)+ridge_width_2 ,p1.y+ (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,top_of_ridge]
        point_KZ3R = [p1.x+ (eave_length_base/2)+ridge_width_2 ,p1.y+ (eave_length_trapezoidal/2)-major_rafter_width_2+offset_HRORM ,top_of_ridge-plumbcut]
        point_KZ4R = [p1.x+ (eave_length_base/2)+ridge_width_2 ,p1.y+ (eave_length_trapezoidal/2)+major_rafter_width_2+offset_HRORM ,top_of_ridge-plumbcut]
    
        # draw ridge
        
      
        point_RZ1 = [p1.x+ (eave_length_base/2)-(ridge_width/2),p1.y+ minor_run-minor_ridge_run,top_of_ridge]
        point_RZ2 = [p1.x+ (eave_length_base/2)-(ridge_width/2),p1.y+ minor_run-minor_ridge_run ,top_of_ridge-ridge_depth]
        point_RZ3 = [p1.x+ (eave_length_base/2)+(ridge_width/2),p1.y+ minor_run-minor_ridge_run ,top_of_ridge]
        point_RZ4 = [p1.x+ (eave_length_base/2)+(ridge_width/2),p1.y+ minor_run-minor_ridge_run ,top_of_ridge-ridge_depth]
        
        point_R2Z1 = [p1.x+ (eave_length_base/2)-(ridge_width/2),p1.y+ (minor_run*2) -minor_ridge_run ,top_of_ridge]
        point_R2Z2 = [p1.x+ (eave_length_base/2)-(ridge_width/2),p1.y+ (minor_run*2)-minor_ridge_run ,top_of_ridge-ridge_depth]
        point_R2Z3 = [p1.x+ (eave_length_base/2)+(ridge_width/2),p1.y+ (minor_run*2)-minor_ridge_run ,top_of_ridge]
        point_R2Z4 = [p1.x+ (eave_length_base/2)+(ridge_width/2),p1.y+ (minor_run*2)-minor_ridge_run ,top_of_ridge-ridge_depth]
        
        
     
       new_rafter = ents.add_face [point_RZ1,point_RZ2,point_RZ4,point_RZ3]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        
        new_rafter = ents.add_face [point_R2Z1,point_R2Z2,point_R2Z4,point_R2Z3]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        
        new_rafter = ents.add_face [point_RZ1,point_RZ2,point_R2Z2,point_R2Z1,point_RZ1]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        
       new_rafter = ents.add_face [point_RZ3,point_RZ4,point_R2Z4,point_R2Z3,point_RZ3]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        
        new_rafter = ents.add_face [point_RZ1,point_R2Z1,point_R2Z3,point_RZ3,point_RZ1]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
       
       new_rafter = ents.add_face [point_RZ2,point_R2Z2,point_R2Z4,point_RZ4,point_RZ2]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        
   
        
        #draw foot print of rafters & plumb face
        new_rafter = ents.add_face [point_E1a,point_E2a,point_E2b,point_E1b,point_E1a]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
      
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        new_rafter = ents.add_face [point_F1a,point_F2a,point_F2b,point_F1b,point_F1a]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        new_rafter = ents.add_face [point_KZ1,point_KZ2,point_KZ3,point_KZ4,point_KZ1] # plumb face
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        
        #draw over hang
        new_rafter = ents.add_face [point_E1a,point_OE1a,point_OE2a,point_OE3a,point_E1a]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        new_rafter = ents.add_face [point_E1b,point_OE1b,point_OE2b,point_OE3b,point_E1b]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        new_rafter = ents.add_face [point_OE1a,point_OE2a,point_OE2b,point_OE1b]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        
        new_rafter = ents.add_face [point_E1a,point_OE1a,point_OE1b,point_E1b]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
      
        new_rafter = ents.add_face [point_OE2a,point_OE3a,point_OE3b,point_OE2b]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
      
      
      
         new_rafter = ents.add_face [point_F1a,point_OF1a,point_OF2a,point_OF3a,point_F1a]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
          new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        new_rafter = ents.add_face [point_F1b,point_OF1b,point_OF2b,point_OF3b,point_F1b]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
          new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        new_rafter = ents.add_face [point_OF1a,point_OF2a,point_OF2b,point_OF1b]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
          new_rafter.back_material= Sketchup::Color.new(153,122,91) 
            new_rafter = ents.add_face [point_F1a,point_OF1a,point_OF1b,point_F1b]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        
        new_rafter = ents.add_face [point_OF2a,point_OF3a,point_OF3b,point_OF2b]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
      
      
        #draw left rafter sides
        new_rafter = ents.add_face [point_E1a,point_KZ1,point_KZ4,point_E3a,point_E2a,point_E1a]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        new_rafter = ents.add_face [point_E1b,point_KZ2,point_KZ3,point_E3b,point_E2b,point_E1b]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        
        #draw left rafter top & bottom
        new_rafter = ents.add_face [point_E1a,point_KZ1,point_KZ2,point_E1b,point_E1a] # top
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        new_rafter = ents.add_face [point_E3a,point_KZ4,point_KZ3,point_E3b,point_E3a] #bottom
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 

        #draw Right rafter sides
       new_rafter = ents.add_face [point_F1a,point_KZ1R,point_KZ4R,point_F3a,point_F2a,point_F1a]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        new_rafter = ents.add_face [point_F1b,point_KZ2R,point_KZ3R,point_F3b,point_F2b,point_F1b]
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        #draw Right rafter top & bottom
       new_rafter = ents.add_face [point_F1a,point_KZ1R,point_KZ2R,point_F1b,point_F1a] # top
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
        new_rafter = ents.add_face [point_F3a,point_KZ4R,point_KZ3R,point_F3b,point_F3a] #bottom
        new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 

        # minor common rafter
       minor_rafter_run = rafter_rise /(Math.tan(minor_pitch_angle));
		   minor_rafter_length = minor_rafter_run / (Math.cos(minor_pitch_angle));
		   minor_overhang_rafter_length = overhang_run / (Math.cos(minor_pitch_angle));
		      
        minor_run_y = minor_rafter_run - minor_ridge_run ;
        minor_rise = minor_run_y * Math.tan(minor_pitch_angle);
        #calculate the depth of the minor rafter
        #minor_rafter_depth = plumbcut * Math.cos(minor_pitch_angle); 
         minor_rafter_depth = major_rafter_depth;
      
   	     
        minor_plumbcut = minor_plumb_height;
        minor_hbp = minor_plumbcut - hap;
       minor_seatcut = minor_hbp / Math.tan(minor_pitch_angle); 
        minor_height_below_plate = minor_hbp;
    
        puts "minor_rafter_depth " + length_sprintf(minor_rafter_depth) if $DEBUG_VERBOSE; 
        puts "minor_seatcut " + length_sprintf(minor_seatcut) if $DEBUG_VERBOSE; 
   puts "minor_rafter run " + length_sprintf(minor_rafter_run) if $DEBUG_VERBOSE; 
   puts "minor_rafter run y " + length_sprintf(minor_run_y) if $DEBUG_VERBOSE; 
 puts "minor_ridge run  " + length_sprintf(minor_ridge_run) if $DEBUG_VERBOSE; 
puts "major_ridge run  " + length_sprintf(major_ridge_run) if $DEBUG_VERBOSE; 
puts "ridge runs total  " + length_sprintf(minor_ridge_run+major_ridge_run) if $DEBUG_VERBOSE; 
major_rafter_width_2 = ridge_width * 0.5;
        point_H1a = [p1.x+ (eave_length_base/2)-major_rafter_width_2,p1.y ,hap]
        point_H2a = [p1.x+ (eave_length_base/2)-major_rafter_width_2,p1.y ,0]
        point_H3a = [p1.x+ (eave_length_base/2)-major_rafter_width_2,p1.y+minor_seatcut ,0]
   
        point_H1b = [p1.x+ (eave_length_base/2)+major_rafter_width_2,p1.y ,hap]
        point_H2b =[p1.x+ (eave_length_base/2)+major_rafter_width_2,p1.y ,0]
        point_H3b = [p1.x+ (eave_length_base/2)+major_rafter_width_2,p1.y+minor_seatcut ,0]
   
        point_MKZ1 = [p1.x+ (eave_length_base/2)-major_rafter_width_2,p1.y+ minor_rafter_run - minor_ridge_run ,minor_rise+hap]
        point_MKZ2 = [p1.x+ (eave_length_base/2)+major_rafter_width_2,p1.y+ minor_rafter_run - minor_ridge_run ,minor_rise+hap]
        point_MKZ3 = [p1.x+ (eave_length_base/2)+major_rafter_width_2,p1.y+ minor_rafter_run - minor_ridge_run ,minor_rise-minor_plumbcut+hap]
        point_MKZ4 = [p1.x+ (eave_length_base/2)-major_rafter_width_2,p1.y+ minor_rafter_run - minor_ridge_run,minor_rise-minor_plumbcut+hap]

        minor_overhang_run = major_overhang_rise / Math.tan(minor_pitch_angle);
        
        minor_overhang_rise = minor_overhang_run * Math.tan(minor_pitch_angle);
     

        point_OH1a = [p1.x+ (eave_length_base/2)-major_rafter_width_2,p1.y-minor_overhang_run ,hap-minor_overhang_rise]
        point_OH2a = [p1.x+ (eave_length_base/2)-major_rafter_width_2,p1.y-minor_overhang_run ,hap-minor_overhang_rise-minor_plumbcut]
        point_OH3a = [p1.x+ (eave_length_base/2)-major_rafter_width_2,p1.y ,-minor_height_below_plate]
   
        point_OH1b = [p1.x+ (eave_length_base/2)+major_rafter_width_2,p1.y-minor_overhang_run ,hap-minor_overhang_rise]
        point_OH2b = [p1.x+ (eave_length_base/2)+major_rafter_width_2,p1.y-minor_overhang_run ,hap-minor_overhang_rise-minor_plumbcut]
        point_OH3b = [p1.x+ (eave_length_base/2)+major_rafter_width_2,p1.y ,-minor_height_below_plate]
   

 
        #draw minor rafter foot print of rafters & plumb face
        new_rafter = ents.add_face [point_H1a,point_H2a,point_H2b,point_H1b,point_H1a] # foot print
          new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     
        new_rafter = ents.add_face [point_MKZ1,point_MKZ2,point_MKZ3,point_MKZ4,point_MKZ1] # plumbface
          new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     
        #draw minor rafter sides
        new_rafter = ents.add_face [point_H1a,point_MKZ1,point_MKZ4,point_H3a,point_H2a,point_H1a] 
          new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     
        new_rafter = ents.add_face [point_H1b,point_MKZ2,point_MKZ3,point_H3b,point_H2b,point_H1b] 
          new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     
        #draw minor rafter top & bottom
        new_rafter = ents.add_face [point_H1a,point_MKZ1,point_MKZ2,point_H1b,point_H1a] #top
          new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     
        new_rafter = ents.add_face [point_H3a,point_MKZ4,point_MKZ3,point_H3b,point_H3a] #bottom     
          new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     

   #draw minor rafter overhang
          new_rafter = ents.add_face [point_H1a,point_OH1a,point_OH2a,point_OH3a,point_H1a]
            new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     
          new_rafter = ents.add_face [point_H1b,point_OH1b,point_OH2b,point_OH3b,point_H1b]
            new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     
          new_rafter = ents.add_face [point_OH1a,point_OH2a,point_OH2b,point_OH1b]
            new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     
          new_rafter = ents.add_face [point_H1a,point_OH1a,point_OH1b,point_H1b]
            new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     
          new_rafter = ents.add_face [point_OH2a,point_OH3a,point_OH3b,point_OH2b]
            new_rafter.material= Sketchup::Color.new(153,122,91) 
        new_rafter.back_material= Sketchup::Color.new(153,122,91) 
     
    
        group3 = model_ents.add_group;
        group3.name="Hip Rafter Left";
 
 
      p1.z = hap;
        # draw hip rafters for principal rafters -- left
        dx = 1;
        dy = 1;
        draw_trapezoidal_hip(dx,dy,group3,p1,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,
        hip_rafter_width,hip_rafter_depth,major_run,minor_run,hap,ridge_width,top_of_ridge,overhang_run);
        
        group4 = model_ents.add_group;
        group4.name="Hip Rafter Right";
 
        dx = -1;
        dy = 1;
        p1.x += eave_length_base;
        draw_trapezoidal_hip(dx,dy,group4,p1,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,
        hip_rafter_width,hip_rafter_depth,major_run,minor_run,hap,ridge_width,top_of_ridge,overhang_run);



        group4 = model_ents.add_group;
        group4.name="Major Jack Rafters Left";
 
 
      p1.z = hap;
        # draw hip rafters for principal rafters -- left
        dx = 1;
        dy = 1;
         p1.x = 0;
        draw_jack_rafters_major(dx,dy,group4,p1,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,
        hip_rafter_width,hip_rafter_depth,major_run,minor_run,hap,ridge_width,top_of_ridge,jack_rafter_spacing,
        seatcut,overhang_run,major_rafter_depth,major_rafter_width);
      
        group5 = model_ents.add_group;
        group5.name="Major Jack Rafters Right";
 

     # draw hip rafters for principal rafters -- left
        dx = -1;
        dy = 1;
         p1.x += eave_length_base;
        draw_jack_rafters_major(dx,dy,group5,p1,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,
        hip_rafter_width,hip_rafter_depth,major_run,minor_run,hap,ridge_width,top_of_ridge,jack_rafter_spacing,
        seatcut,overhang_run,major_rafter_depth,major_rafter_width);
    
    
        group6 = model_ents.add_group;
        group6.name="Minor Jack Rafters Left";
 
 
      p1.z = hap;
        # draw hip rafters for principal rafters -- left
        dx = 1;
        dy = 1;
         p1.x = 0;
        draw_jack_rafters_minor(dx,dy,group6,p1,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,
        hip_rafter_width,hip_rafter_depth,major_run,minor_run,hap,ridge_width,top_of_ridge,jack_rafter_spacing,
        seatcut,overhang_run,major_rafter_depth,major_rafter_width);
    
    
          group7 = model_ents.add_group;
        group7.name="Minor Jack Rafters Left";
 
 
      p1.z = hap;
        # draw hip rafters for principal rafters -- left
        dx = -1;
        dy = 1;
         p1.x = eave_length_base;
        draw_jack_rafters_minor(dx,dy,group7,p1,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,
        hip_rafter_width,hip_rafter_depth,major_run,minor_run,hap,ridge_width,top_of_ridge,jack_rafter_spacing,
        seatcut,overhang_run,major_rafter_depth,major_rafter_width);
    
    
    
    
    

      Sketchup.active_model.rendering_options["ModelTransparency"]=false;
      #Sketchup.active_model.rendering_options["ModelTransparency"]=true if show_internal_planes || show_roof_planes;
      view = Sketchup.active_model.active_view 
      view.field_of_view = 30
      #result = Sketchup.send_action 10501; # set view to Top
      view.zoom_extents 

      group.to_component 
      model.commit_operation

   
end # end function


# draw jack rafters major
def draw_jack_rafters_minor(dx,dy,group,origin,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,hip_rafter_width,hip_rafter_depth,
  major_run,minor_run,hap,ridge_width,top_of_ridge,jack_rafter_spacing,
  seatcut,overhang_run,major_rafter_depth,major_rafter_width)   

      @entity = group.entities;

       d =  major_plan_angle;
		   dd = minor_plan_angle;
		   s =  major_pitch_angle;
		   ss = minor_pitch_angle;
       hip_angle = Math.atan(Math.tan(major_pitch_angle)* Math.sin(major_plan_angle));
      plumbcut = major_rafter_depth / Math.cos(minor_pitch_angle); 
      height_below_plate = plumbcut - hap;     
      jack_rafter_width = major_rafter_width;
		  major_sheathing_angle = Math.atan(Math.tan(major_plan_angle) / Math.cos(major_pitch_angle));
		  minor_sheathing_angle = Math.atan(Math.tan(minor_plan_angle) / Math.cos(minor_pitch_angle));
		  major_jack_rafter_length_diff = jack_rafter_spacing * (Math.tan(major_sheathing_angle));
		  minor_jack_rafter_length_diff = jack_rafter_spacing * (Math.tan(minor_sheathing_angle));
      
      offset_HROM = ((hip_rafter_width * Math.sin(d)) / Math.sin(Math::PI - d - dd)) * Math.cos(dd); # Hip_Rafter_Offset_Main_Perpendicular
		   offset_HROA = ((hip_rafter_width * Math.sin(dd)) / Math.sin(Math::PI - d - dd)) * Math.cos(d); # Hip_Rafter_Offset_Adjacent_Perpendicular
		  
      offset_HRORM = offset_HROM / Math.sin(d); # # Hip_Rafter_Offset_Main_along eave
		   offset_HRORA = offset_HROA / Math.sin(dd);# Hip_Rafter_Offset_Adhacent_along eave
       
		   offset_HRORMR = offset_HRORM * Math.sin(dd); # jack rafter run deduction for hip rafter backing offset
		   offset_HRORMR_Hip_Rafter = offset_HRORMR / Math.cos(hip_angle);
		
		  p2m = Math.atan(Math.cos(s) / Math.tan(d));
		  p2a = Math.atan(Math.cos(ss) / Math.tan(dd));
		  jack_JRODM = offset_HRORM * Math.tan(Math::PI/2 - p2m);
		  jack_JRODA = offset_HRORA * Math.tan(Math::PI/2 - p2a);
		 	jack_JRLM = (Math.tan(Math::PI/2 - p2m) * (jack_rafter_spacing + (jack_rafter_width* 0.5))) - jack_JRODM;
		  jack_JRLA = (Math.tan(Math::PI/2 - p2a) * (jack_rafter_spacing + (jack_rafter_width* 0.5))) - jack_JRODA;
		    
        jack_rafter_width2 = jack_rafter_width * 0.5;
        major_overhang_rise = overhang_run * Math.tan(major_pitch_angle);
        minor_overhang_run = major_overhang_rise / Math.tan(minor_pitch_angle);
        
        minor_overhang_rise = minor_overhang_run * Math.tan(minor_pitch_angle);
        
        minor_plumbcut = plumbcut;
        minor_hbp = minor_plumbcut - hap;
       minor_seatcut = minor_hbp / Math.tan(minor_pitch_angle); 
        minor_height_below_plate = minor_hbp;
    
          overhang_run_y = overhang_run / Math.tan(major_plan_angle);
          overhang_run_x = overhang_run;
          overhang_run_z =minor_overhang_rise;

          i = 0;
          spacing = (major_run /jack_rafter_spacing)
          ocspacing = jack_rafter_spacing;
       	
    for i in 1..spacing 
          
          rafterrun =(( jack_rafter_spacing*i)+jack_rafter_width2-offset_HRORA) * Math.tan(minor_plan_angle) ;
          z1 = hap + (rafterrun * Math.tan(minor_pitch_angle)) -plumbcut;
          z2 = hap + (rafterrun * Math.tan(minor_pitch_angle));
          z3 = hap - overhang_run_z;  
          z4 =  hap - overhang_run_z -  plumbcut;
          z5 = hap - plumbcut;
           
          pts = []
          pts[0] = [origin.x+(((ocspacing*i)+jack_rafter_width2)*dx),origin.y,0];
          pts[1] = [origin.x+(((ocspacing*i)+jack_rafter_width2)*dx),origin.y+seatcut,0];
          pts[2] = [origin.x+(((ocspacing*i)+jack_rafter_width2)*dx),origin.y+rafterrun,z1];
          pts[3] = [origin.x+(((ocspacing*i)+jack_rafter_width2)*dx),origin.y+rafterrun,z2];
          pts[4] = [origin.x+(((ocspacing*i)+jack_rafter_width2)*dx),origin.y-minor_overhang_run,z3];
          pts[5] = [origin.x+(((ocspacing*i)+jack_rafter_width2)*dx),origin.y-minor_overhang_run,z4];
          pts[6] = [origin.x+(((ocspacing*i)+jack_rafter_width2)*dx),origin.y,z5];
          pts[7] = [origin.x+(((ocspacing*i)+jack_rafter_width2)*dx),origin.y,0];

          @new_jackrafter = @entity.add_face pts;
          status = @new_jackrafter.back_material=[153,137,122] ;      
          @new_jackrafter.material= Sketchup::Color.new(153,137,122);
          
          rafterrun =(( jack_rafter_spacing*i)-jack_rafter_width2-offset_HRORA) * Math.tan(minor_plan_angle) ;
          z1 = hap + (rafterrun * Math.tan(minor_pitch_angle)) -plumbcut;
          z2 = hap + (rafterrun * Math.tan(minor_pitch_angle));
          z3 = hap - overhang_run_z;  
          z4 =  hap - overhang_run_z -  plumbcut;
          z5 = hap - plumbcut;
           
          pts2 = []
          pts2[0] = [origin.x+(((ocspacing*i)-jack_rafter_width2)*dx),origin.y,0];
          pts2[1] = [origin.x+(((ocspacing*i)-jack_rafter_width2)*dx),origin.y+seatcut,0];
          pts2[2] = [origin.x+(((ocspacing*i)-jack_rafter_width2)*dx),origin.y+rafterrun,z1];
          pts2[3] = [origin.x+(((ocspacing*i)-jack_rafter_width2)*dx),origin.y+rafterrun,z2];
          pts2[4] = [origin.x+(((ocspacing*i)-jack_rafter_width2)*dx),origin.y-minor_overhang_run,z3];
          pts2[5] = [origin.x+(((ocspacing*i)-jack_rafter_width2)*dx),origin.y-minor_overhang_run,z4];
          pts2[6] = [origin.x+(((ocspacing*i)-jack_rafter_width2)*dx),origin.y,z5];
          pts2[7] = [origin.x+(((ocspacing*i)-jack_rafter_width2)*dx),origin.y,0];

          @new_jackrafter = @entity.add_face pts2;
          status = @new_jackrafter.back_material=[153,137,122] ;      
          @new_jackrafter.material= Sketchup::Color.new(153,137,122);
          
        @new_jackrafter = @entity.add_face [pts[4],pts[5],pts2[5],pts2[4],pts[4]];
          status = @new_jackrafter.back_material=[153,137,122] ;      
          @new_jackrafter.material= Sketchup::Color.new(153,137,122);
 
        @new_jackrafter = @entity.add_face [pts[4],pts[3],pts2[3],pts2[4],pts[4]];
          status = @new_jackrafter.back_material=[153,137,122] ;      
          @new_jackrafter.material= Sketchup::Color.new(153,137,122);


          #@new_jackrafter.pushpull major_rafter_width;
          sp = jack_rafter_spacing * (i+1);

          if(( sp + (major_rafter_width *2)) > major_run)
            break
          end
    	 
        end
    
  
   
  end
    


# draw jack rafters major
def draw_jack_rafters_major(dx,dy,group,origin,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,hip_rafter_width,hip_rafter_depth,
  major_run,minor_run,hap,ridge_width,top_of_ridge,jack_rafter_spacing,
  seatcut,overhang_run,major_rafter_depth,major_rafter_width)   

      @entity = group.entities;


       d =  major_plan_angle;
		   dd = minor_plan_angle;
		   s =  major_pitch_angle;
		   ss = minor_pitch_angle;
       hip_angle = Math.atan(Math.tan(major_pitch_angle)* Math.sin(major_plan_angle));
      plumbcut = major_rafter_depth / Math.cos(major_pitch_angle); 
      height_below_plate = plumbcut - hap;     
      jack_rafter_width = major_rafter_width;
		  major_sheathing_angle = Math.atan(Math.tan(major_plan_angle) / Math.cos(major_pitch_angle));
		  minor_sheathing_angle = Math.atan(Math.tan(minor_plan_angle) / Math.cos(minor_pitch_angle));
		  major_jack_rafter_length_diff = jack_rafter_spacing * (Math.tan(major_sheathing_angle));
		  minor_jack_rafter_length_diff = jack_rafter_spacing * (Math.tan(minor_sheathing_angle));
      
      offset_HROM = ((hip_rafter_width * Math.sin(d)) / Math.sin(Math::PI - d - dd)) * Math.cos(dd); # Hip_Rafter_Offset_Main_Perpendicular
		   offset_HROA = ((hip_rafter_width * Math.sin(dd)) / Math.sin(Math::PI - d - dd)) * Math.cos(d); # Hip_Rafter_Offset_Adjacent_Perpendicular
		   
       offset_HRORM = offset_HROM / Math.sin(d); # # Hip_Rafter_Offset_Main_along eave
      offset_HRORA = offset_HROA / Math.sin(dd);# Hip_Rafter_Offset_Adhacent_along eave
      
      puts "offset_HROM = " +offset_HROM.to_s if $DEBUG_VERBOSE; 
      puts "offset_HRORM = " +offset_HRORM.to_s if $DEBUG_VERBOSE; 
      
      puts "offset_HROA = " +offset_HROA.to_s if $DEBUG_VERBOSE; 
      puts "offset_HRORA = " +offset_HRORA.to_s if $DEBUG_VERBOSE; 
   
 		
		   offset_HRORMR = offset_HRORM * Math.sin(dd); # jack rafter run deduction for hip rafter backing offset
		   offset_HRORMR_Hip_Rafter = offset_HRORMR / Math.cos(hip_angle);
		  
      puts "offset_HRORMR = " +offset_HRORMR.to_s if $DEBUG_VERBOSE; 
      puts "offset_HRORMR_Hip_Rafter = " +offset_HRORMR_Hip_Rafter.to_s if $DEBUG_VERBOSE; 
    
		  p2m = Math.atan(Math.cos(s) / Math.tan(d));
		  p2a = Math.atan(Math.cos(ss) / Math.tan(dd));
		  jack_JRODM = offset_HRORM * Math.tan(Math::PI/2 - p2m);
		  jack_JRODA = offset_HRORA * Math.tan(Math::PI/2 - p2a);
		 	jack_JRLM = (Math.tan(Math::PI/2 - p2m) * (jack_rafter_spacing + (jack_rafter_width* 0.5))) - jack_JRODM;
		  jack_JRLA = (Math.tan(Math::PI/2 - p2a) * (jack_rafter_spacing + (jack_rafter_width* 0.5))) - jack_JRODA;
		    
        jack_rafter_width2 = jack_rafter_width * 0.5;

          overhang_run_y = overhang_run / Math.tan(major_plan_angle);
          overhang_run_x = overhang_run;
          overhang_run_z = overhang_run * Math.tan(major_pitch_angle);

          i = 0;
          spacing = (minor_run /jack_rafter_spacing)
          ocspacing = jack_rafter_spacing;
       	   rafterrun =(( jack_rafter_spacing)+jack_rafter_width2-offset_HRORM) * Math.tan(major_plan_angle) ;
         puts "rafterrun = " +rafterrun.to_s if $DEBUG_VERBOSE; 
   
    for i in 1..spacing 
          
          rafterrun =(( jack_rafter_spacing*i)+jack_rafter_width2-offset_HRORM) * Math.tan(major_plan_angle) ;
          z1 = hap + (rafterrun * Math.tan(major_pitch_angle)) -plumbcut;
          z2 = hap + (rafterrun * Math.tan(major_pitch_angle));
          z3 = hap - overhang_run_z;  
          z4 =  hap - overhang_run_z -  plumbcut;
          z5 = hap - plumbcut;
           
          pts = []
          pts[0] = [origin.x,origin.y+(ocspacing*i)+jack_rafter_width2,0];
          pts[1] = [origin.x+(seatcut*dx),origin.y+(ocspacing*i)+jack_rafter_width2,0];
          pts[2] = [origin.x+(rafterrun*dx),origin.y+(ocspacing*i)+jack_rafter_width2,z1];
          pts[3] = [origin.x+(rafterrun*dx),origin.y+(ocspacing*i)+jack_rafter_width2,z2];
          pts[4] = [origin.x-(overhang_run*dx),origin.y+(ocspacing*i)+jack_rafter_width2,z3];
          pts[5] = [origin.x-(overhang_run*dx),origin.y+(ocspacing*i)+jack_rafter_width2,z4];
          pts[6] = [origin.x,origin.y+(ocspacing*i)+jack_rafter_width2,z5];
          pts[7] = [origin.x,origin.y+(ocspacing*i)+jack_rafter_width2,0];

          @new_jackrafter = @entity.add_face pts;
          status = @new_jackrafter.back_material=[153,137,122] ;      
          @new_jackrafter.material= Sketchup::Color.new(153,137,122);
          
          rafterrun =(( jack_rafter_spacing*i)-jack_rafter_width2-offset_HRORM) * Math.tan(major_plan_angle) ;
          z1 = hap + (rafterrun * Math.tan(major_pitch_angle)) -plumbcut;
          z2 = hap + (rafterrun * Math.tan(major_pitch_angle));
          z3 = hap - overhang_run_z;  
          z4 =  hap - overhang_run_z -  plumbcut;
          z5 = hap - plumbcut;
           
          pts2 = []
          pts2[0] = [origin.x,origin.y+(ocspacing*i)-jack_rafter_width2,0];
          pts2[1] = [origin.x+(seatcut*dx),origin.y+(ocspacing*i)-jack_rafter_width2,0];
          pts2[2] = [origin.x+(rafterrun*dx),origin.y+(ocspacing*i)-jack_rafter_width2,z1];
          pts2[3] = [origin.x+(rafterrun*dx),origin.y+(ocspacing*i)-jack_rafter_width2,z2];
          pts2[4] = [origin.x-(overhang_run*dx),origin.y+(ocspacing*i)-jack_rafter_width2,z3];
          pts2[5] = [origin.x-(overhang_run*dx),origin.y+(ocspacing*i)-jack_rafter_width2,z4];
          pts2[6] = [origin.x,origin.y+(ocspacing*i)-jack_rafter_width2,z5];
          pts2[7] = [origin.x,origin.y+(ocspacing*i)-jack_rafter_width2,0];

          @new_jackrafter = @entity.add_face pts2;
          status = @new_jackrafter.back_material=[153,137,122] ;      
          @new_jackrafter.material= Sketchup::Color.new(153,137,122);
          
        @new_jackrafter = @entity.add_face [pts[4],pts[5],pts2[5],pts2[4],pts[4]];
          status = @new_jackrafter.back_material=[153,137,122] ;      
          @new_jackrafter.material= Sketchup::Color.new(153,137,122);
 
        @new_jackrafter = @entity.add_face [pts[4],pts[3],pts2[3],pts2[4],pts[4]];
          status = @new_jackrafter.back_material=[153,137,122] ;      
          @new_jackrafter.material= Sketchup::Color.new(153,137,122);


          #@new_jackrafter.pushpull major_rafter_width;
          sp = jack_rafter_spacing * (i+1);

          if(( sp + (major_rafter_width *2)) > minor_run)
            break
          end
    	 
        end
    
  
   
  end
    
 # draw hip rafters for principal rafters
def draw_trapezoidal_hip(dx,dy,group,origin,eave_angle,major_pitch_angle,minor_pitch_angle,major_plan_angle,minor_plan_angle,hip_rafter_width,hip_rafter_depth,
  major_run,minor_run,hap,ridge_width,top_of_ridge,overhang_run )   

      @entity = group.entities;


      #S = major_pitch_angle
      #SS =minor_pitch_angle
      hip_angle = Math.atan(Math.tan(major_pitch_angle)* Math.sin(major_plan_angle))
      hip_rafter_run = major_run / Math.cos(major_plan_angle);
      #S Hip Rafter Offset
      hrom_S = ((hip_rafter_width * Math.sin(major_plan_angle)) / Math.sin(180.degrees - major_plan_angle - minor_plan_angle)) * Math.cos(minor_plan_angle);
      #SS Hip Rafter Offset
      hrom_SS = ((hip_rafter_width * Math.sin(minor_plan_angle)) / Math.sin(180.degrees - major_plan_angle - minor_plan_angle)) * Math.cos(major_plan_angle);
      #S Hip Rafter Offset Run along eave
      hror_S = hrom_S / Math.sin(major_plan_angle);
      #SS Hip Rafter OffsetRun along eave 
      hror_SS = hrom_SS / Math.sin(minor_plan_angle);
      
      hip_rafter_length = hip_rafter_run / (Math.cos(hip_angle));
      plumbcut = hip_rafter_depth / Math.cos(major_pitch_angle); 
      minor_height_below_plate = plumbcut - hap;
      seatcut = minor_height_below_plate / Math.tan(minor_pitch_angle); 
		  major_ridge_run = ((ridge_width * Math.sin(minor_pitch_angle)) / Math.sin(Math::PI - major_pitch_angle - minor_pitch_angle)) * Math.cos(major_pitch_angle);
		  minor_ridge_run = ((ridge_width * Math.sin(major_pitch_angle)) / Math.sin(Math::PI - major_pitch_angle - minor_pitch_angle)) * Math.cos(minor_pitch_angle);
	    h_HRORMR = hror_S * Math.sin(major_pitch_angle);
		  h_HRORMR_Hip_Rafter = h_HRORMR / Math.cos(hip_angle);
      
      minor_h_HRORMR = hror_SS * Math.sin(minor_pitch_angle);
		  minor_h_HRORMR_Hip_Rafter = minor_h_HRORMR / Math.cos(hip_angle);    
      
		  major_sheathing_angle = Math.atan(Math.tan(major_plan_angle) / Math.cos(major_pitch_angle));
		  minor_sheathing_angle = Math.atan(Math.tan(minor_plan_angle) / Math.cos(minor_pitch_angle));
		  major_hipsidecut_angle = Math.atan(Math.cos(hip_angle) / Math.tan(major_plan_angle));
      minor_hipsidecut_angle = Math.atan(Math.cos(hip_angle) / Math.tan(minor_plan_angle));
      hawk_R5 = Math.atan(Math.tan(hip_angle) * Math.cos(major_plan_angle)); 
      hawk_R5_minor = Math.atan(Math.tan(hip_angle) * Math.cos(minor_plan_angle)); 
   
      hip_side_cut_length = hror_S; #(hip_rafter_width * 0.5)/ Math.sin(major_hipsidecut_angle);
      hip_side_cut_run  =  hip_side_cut_length * Math.cos(hawk_R5);
      hip_side_cut_rise  =  hip_side_cut_length * Math.sin(hawk_R5);
      
      hip_side_cut_run  =   hror_S; # * Math.tan(major_plan_angle);
      hip_side_cut_rise  =  hip_side_cut_run * Math.tan(major_pitch_angle); 
      
      
      minor_hip_side_cut_run  =  hror_SS * Math.tan(minor_plan_angle);      
      minor_hip_side_cut_rise  =  minor_hip_side_cut_run * Math.tan(minor_pitch_angle);      
      
      major_Hip_Rafter_Plumb_Backing_Angle = Math.atan(Math.tan(major_pitch_angle) * Math.cos(major_plan_angle));
      minor_Hip_Rafter_Plumb_Backing_Angle = Math.atan(Math.tan(minor_pitch_angle) * Math.cos(minor_plan_angle));
      
          
     major_plumb_backing_height = (hip_rafter_width * 0.5)* Math.tan(major_Hip_Rafter_Plumb_Backing_Angle);
    minor_plumb_backing_height = (hip_rafter_width * 0.5)* Math.tan(minor_Hip_Rafter_Plumb_Backing_Angle);
 
    major_plumbcut_height = plumbcut - major_plumb_backing_height;
    minor_plumbcut_height = plumbcut - minor_plumb_backing_height;
 
 
 
      puts "hip_side_cut_length  " + length_sprintf(hip_side_cut_length); 
      puts "hip_side_cut_run  " + length_sprintf(hip_side_cut_run); 
      puts "hip_side_cut_rise  " + length_sprintf(hip_side_cut_rise); 
        puts "major_plumb_backing_height  " + length_sprintf(major_plumb_backing_height); 
        
           puts "hror_S  " + length_sprintf(hror_S); 
             puts "hror_SS  " + length_sprintf(hror_SS); 
                     puts "minor_hip_side_cut_run  " + length_sprintf(minor_hip_side_cut_run); 
                  puts " minor_hip_side_cut_rise  " + length_sprintf( minor_hip_side_cut_rise); 
        
        puts "major_plumbcut_height  " + length_sprintf(major_plumbcut_height); 
        puts "minor_plumbcut_height  " + length_sprintf(minor_plumbcut_height); 
             
      angle_d = angle_sprintf(major_hipsidecut_angle );
      puts "major_hipsidecut_angle = " + angle_d if $DEBUG_VERBOSE;
      
          major_ridge_rafter = major_ridge_run / Math.cos(major_pitch_angle);
          hip_ridge_deduction = major_ridge_rafter / Math.sin(major_sheathing_angle);

          hip_rafter_length_to_ridge = hip_rafter_length - hip_ridge_deduction + h_HRORMR_Hip_Rafter;

          hip_rafter_run_to_ridge_x = major_run - (ridge_width * 0.5);
          hip_rafter_run_to_ridge_y =minor_run - ((ridge_width * 0.5)/ Math.tan(major_plan_angle));

          hip_rafter_run_to_hip_side_cut = hip_rafter_run_to_ridge_x;

          point_S = [origin.x,origin.y+(hror_S*dy),hap]; # major hip rafter side planing point at eave line
          point_SS = [origin.x+(hror_SS*dx),origin.y,hap]; # minor hip rafter side planing point at eave line

          point_CS = [origin.x,origin.y,hap- plumbcut]; # minor hip rafter side planing point at eave line

          point_Sp = [origin.x,origin.y+(hror_S*dy),hap-plumbcut]; # major hip rafter side planing point at eave line
          point_SSp = [origin.x+(hror_SS*dx),origin.y,hap-plumbcut]; # minor hip rafter side planing point at eave line

          run_y = hip_rafter_run_to_ridge_y;
          run_x = hip_rafter_run_to_ridge_x;

          run_y_sidecut = hip_rafter_run_to_ridge_y+hip_side_cut_run ;
          run_x_sidecut = hip_rafter_run_to_ridge_x;
          run_z_sidecut = hip_rafter_run_to_ridge_x;

          puts "run_x = " + run_x.to_s if $DEBUG_VERBOSE; 
          puts "run_y = " + run_y.to_s if $DEBUG_VERBOSE; 
          
          
           overhang_run_y = overhang_run / Math.tan(major_plan_angle);
           overhang_run_x = overhang_run;
           overhang_run_z = overhang_run * Math.tan(major_pitch_angle);
             
           point_OHg = [origin.x-(overhang_run_x*dx),origin.y-overhang_run_y,hap-overhang_run_z];
           point_OHgp = [origin.x-(overhang_run_x*dx),origin.y-overhang_run_y,hap-overhang_run_z- plumbcut];
           corner = [origin.x,origin.y,-hap];
           
           point_OHgS = [origin.x-(overhang_run_x*dx),origin.y+(hror_S*dy)-overhang_run_y,hap-overhang_run_z]; 
           point_OHgSp = [origin.x-(overhang_run_x*dx),origin.y+(hror_S*dy)-overhang_run_y,hap-overhang_run_z- plumbcut]; 
         
           
          point_OHgSS = [origin.x+(hror_SS*dx)-(overhang_run_x*dx),origin.y-overhang_run_y,hap-overhang_run_z]; 
          point_OHgSSp = [origin.x+(hror_SS*dx)-(overhang_run_x*dx),origin.y-overhang_run_y,hap-overhang_run_z-plumbcut]; 


          point_KZ = [origin.x+(run_x*dx),origin.y+(run_y*dy),top_of_ridge];# center point of hip rafter at hip rafter rise/elevation
          point_KZZ = [origin.x+(run_x*dx),origin.y+(run_y*dy),top_of_ridge- plumbcut];# center point of hip rafter at hip rafter rise/elevation

          point_KZL = [origin.x+(run_x*dx),origin.y+( run_y_sidecut *dy),top_of_ridge];# hip rafter side cut
          point_KZLP = [origin.x+(run_x*dx),origin.y+( run_y_sidecut *dy),top_of_ridge+hip_side_cut_rise- plumbcut ];# hip rafter side cut
          point_KZLSS = [origin.x+(run_x*dx),origin.y+( run_y-minor_hip_side_cut_run *dy),top_of_ridge-minor_hip_side_cut_rise];# hip rafter side cut fix
          point_KZLPSS = [origin.x+(run_x*dx),origin.y+( run_y-minor_hip_side_cut_run *dy),top_of_ridge-minor_hip_side_cut_rise- plumbcut];# hip rafter side cut


          #@new_rafter = @entity.add_face [origin,point_S,point_Major_KZ,point_KZ,origin] #top of rafter    left side
          @new_rafter = @entity.add_line [origin,point_KZ] #top of rafter   center line

          @new_rafter = @entity.add_line [point_S,point_KZL] #top of rafter    right side

          @new_rafter = @entity.add_line [point_SS,point_KZLSS] #top of rafter    left side

          @new_rafter = @entity.add_line [point_Sp,point_KZLP] # long point of rafter

          @new_rafter = @entity.add_line [point_KZL,point_KZLP] #plumb cut line upper

          @new_rafter = @entity.add_line [point_KZLSS,point_KZLPSS] #plumb cut line lower

          @new_rafter = @entity.add_line [point_KZL,point_KZLSS] #line across the top of the rafter
          @new_rafter = @entity.add_line [point_KZLP,point_KZLPSS] #line across the bottom of the rafter


          @new_rafter = @entity.add_line [point_SSp,point_KZLPSS] #short point of rafter

          @new_rafter = @entity.add_line [point_SS,point_SSp] #foot plumb line 

          @new_rafter = @entity.add_line [point_S,point_Sp] #foot plumb line

          @new_rafter = @entity.add_line [origin,point_CS] #top of rafter    left side

          @new_rafter = @entity.add_line [point_CS,point_KZZ] # bottom of rafter

  @new_rafter = @entity.add_line [origin,point_S] 
@new_rafter = @entity.add_line [origin,point_SS] 
  @new_rafter = @entity.add_line [point_CS,point_Sp] 
@new_rafter = @entity.add_line [point_CS,point_SSp] 


       @new_rafter = @entity.add_face [point_Sp,point_S,point_KZL,point_KZLP,point_Sp] #top of rafter    left side
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
  
          @new_rafter = @entity.add_face [point_SSp,point_SS,point_KZLSS,point_KZLPSS,point_SSp] #top of rafter    left side
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
      
     
     @new_rafter = @entity.add_face [point_KZLPSS,point_KZLSS,point_KZ,point_KZZ,point_KZLPSS] #top of rafter    left side
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
       
       @new_rafter = @entity.add_face [point_KZLP,point_KZL,point_KZ,point_KZZ,point_KZLP] #top of rafter    left side
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
   
      
      @new_rafter = @entity.add_face [point_SSp,point_SS,origin,point_CS,point_SSp] #top of rafter    left side
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
      
     @new_rafter = @entity.add_face [point_Sp,point_S,origin,point_CS,point_Sp] #top of rafter    left side
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
  
 @new_rafter = @entity.add_face [origin,point_S,point_KZL,point_KZ,origin] #top of rafter    left side
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
  
         @new_rafter = @entity.add_face [origin,point_SS,point_KZLSS,point_KZ,origin] #top of rafter    left side
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
  
      @new_rafter = @entity.add_face [origin,point_OHg,point_OHgp, point_CS] #top of rafter    left side
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
    
    @new_rafter = @entity.add_face [point_OHg,point_OHgp, point_OHgSp,point_OHgS,point_OHg] 
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
  
@new_rafter = @entity.add_face [point_OHg,point_OHgp, point_OHgSSp,point_OHgSS,point_OHg] 
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
  
  @new_rafter = @entity.add_face [point_OHgS,point_OHgSp,point_Sp,point_S,point_OHgS] 
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
      @new_rafter = @entity.add_face [point_OHgSS,point_OHgSSp,point_SSp,point_SS,point_OHgSS] 
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
   
      @new_rafter = @entity.add_face [origin,point_OHg,point_OHgS,point_S,origin] 
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
   
    @new_rafter = @entity.add_face [origin,point_OHg,point_OHgSS,point_SS,origin] 
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
      
       @new_rafter = @entity.add_face [point_CS,point_OHgp,point_OHgSp,point_Sp,point_CS] 
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
   
    @new_rafter = @entity.add_face [point_CS,point_OHgp,point_OHgSSp,point_SSp,point_CS] 
      status = @new_rafter.back_material=[153,137,122]       
      @new_rafter.material= Sketchup::Color.new(153,137,122)
      
      
      
   
  end
    
end # class TrapezoidalTimberTool

end #module ROOFFRAMING

file_loaded("RoofFraming/Unequal_Hip.rb")
