# Copyright 2012 Sim Ayers
# version 1.0
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------


module ROOFFRAMING

class DrawDownTool < TetrahedronTool

$profile_slope_angle = 51.00;
$ground_plan_angle = 60.00;
$ground_plan_base = "800mm";
$profile_rafter_width = "40mm";
$profile_rafter_depth = "80mm";
$main_hip_rafter_width = "50mm";
$main_hip_rafter_depth = "80mm";
$timber = "no";
$timber_angle = 45.00;
$timber_type = "Saint Andrews Cross";

def initialize
       
    $profile_slope_angle = 51.00 if not $profile_slope_angle;
    $ground_plan_angle = 60.00 if not $ground_plan_angle;
    $ground_plan_base = "800mm" if not$ground_plan_base;
    $profile_rafter_width = "40mm" if not $profile_rafter_width;
    $profile_rafter_depth = "80mm" if not $profile_rafter_depth;
    $main_hip_rafter_width = "50mm" if not $main_hip_rafter_width;
    $main_hip_rafter_depth = "80mm" if not $main_hip_rafter_depth;
    $timber = "no" if not $timber;
    $timber_angle = "45.00" if not $timber_angle;
    $timber_type = "Saint Andrews Cross" if not $timber_type;

    # First prompt for the dimensions.  This is done using the inputbox method
        
    # The first step is to create some arrays which contain the prompts
    # and default values.
    prompts = ["Profile Rafter Slope Angle","Ground Plan Angle","Ground Plan Base Length",
    "Profile Rafter Width","Profile Rafter Depth","Hip Rafter Width"," Hip Rafter Depth",
    "Insert Timber","Timber Type","Timber Rotation Angle"]
    #default values
    values = [$profile_slope_angle,$ground_plan_angle,$ground_plan_base,$profile_rafter_width,$profile_rafter_depth,$main_hip_rafter_width,$main_hip_rafter_depth,
    $timber,$timber_type,$timber_angle]
    
    popups = [nil, nil, nil, nil,nil, nil, nil,"no|yes","Saint Andrews Cross|Single Timber",nil]
        
    # Now display the inputbox
    results = inputbox prompts, values,popups, "Draw Down Technique";

    return if not results # This means that the user cancelled the operation
    $profile_slope_angle,$ground_plan_angle,$ground_plan_base,$profile_rafter_width,$profile_rafter_depth,$main_hip_rafter_width,
    $main_hip_rafter_depth,$timber,$timber_type,$timber_angle= results;
    
       
     p1 = [0,0,0];

    	draw_down_plan(p1,$profile_slope_angle,$ground_plan_angle,$ground_plan_base.to_l,$profile_rafter_width.to_l,
      $profile_rafter_depth.to_l,$main_hip_rafter_width.to_l,$main_hip_rafter_depth.to_l);  
    
    if($timber == "yes") 
    	draw_timber(p1,$profile_slope_angle,$ground_plan_angle,$ground_plan_base.to_l,$profile_rafter_width.to_l,
      $profile_rafter_depth.to_l,$main_hip_rafter_width.to_l,$main_hip_rafter_depth.to_l,$timber_type,$timber_angle);  
    end
    
    
end

def   	draw_timber(p1,profile_slope_angle,ground_plan_angle,ground_plan_base,profile_rafter_width,profile_rafter_depth,main_hip_rafter_width,
  main_hip_rafter_depth,timber_type,timber_angle)

     model = Sketchup.active_model;
    # this enables undo of the whole operation as a unit
    model.start_operation "Timber";

    # create a new group in which we will draw our dimension entities
    model_ents = model.active_entities;
    group = model_ents.add_group;
    ents = group.entities;
    group.name="Timber";
    wplayer=model.layers.add("Timber");
    wplayer.visible=true;
    group.layer=wplayer;
    group.casts_shadows=false;
    group.receives_shadows=false;
    puts "new timber" if $DEBUG_VERBOSE;
    
    plan_angle = ground_plan_angle.degrees;
    profile_rafter_slope_angle = profile_slope_angle.degrees;
    profile_rafter_run = (ground_plan_base/2) * Math.tan( plan_angle);
    profile_rafter_length = profile_rafter_run / Math.cos( profile_rafter_slope_angle);
    roof_sheathing_angle = Math.atan(Math.tan(plan_angle) / Math.cos(profile_rafter_slope_angle));
    hip_angle = Math.atan(Math.tan(profile_rafter_slope_angle) * Math.sin(plan_angle));
    hip_rafter_length = profile_rafter_length / Math.sin(roof_sheathing_angle);
 
    x4 = ground_plan_base / 4;
  
    p2 = [p1.x + ground_plan_base,p1.y,0];   
    p2_mid = [p1.x + (ground_plan_base/2),p1.y,0];     
    p3 = [p1.x + (ground_plan_base/2),p1.y+profile_rafter_run,0];    
  
    x1 = 0;
    y1 = 0;
    z1 = 0;
    x2 = profile_rafter_width;
    z2 = profile_rafter_depth;
    y2 = profile_rafter_length;
    y2 = hip_rafter_length  if(timber_type =="Saint Andrews Cross");
  
    # draw face  for timber
      pts = []
      pts[0] = [x1, y1, z1];
      pts[1] = [x2, y1, z1];
      pts[2] = [x2, y2, z1];
      pts[3] = [x1, y2, z1];
      pts[4] = pts[0]
      new_face = ents.add_face pts;
      depth = profile_rafter_depth * -1;
      new_face.pushpull depth;
      
      rot_ang = -(Math::PI/2  - timber_angle.degrees);
      z_axis=group.transformation.zaxis;
      ctr =  pts[0].to_a;
      trr=Geom::Transformation.rotation(ctr,Z_AXIS,rot_ang);
      group.transform! trr;
      
      if(timber_type =="Saint Andrews Cross");
        gp = model_ents.add_group;
        gp.name="Rafter Right";
        entity = gp.entities;
        pts = []
        pts[0] = [p2.x, y1, z1];
        pts[1] = [p2.x-x2, y1, z1];
        pts[2] = [p2.x-x2, y2, z1];
        pts[3] = [p2.x, y2, z1];
        pts[4] = pts[0]
        new_face = entity.add_face pts;
        new_face.pushpull depth;
        rot_ang = (Math::PI/2  - timber_angle.degrees);
        z_axis=gp.transformation.zaxis;
        ctr =  pts[0].to_a;
        trr=Geom::Transformation.rotation(ctr,Z_AXIS,rot_ang);
        gp.transform! trr;     
      end
      
    group.to_component;
    model.commit_operation;
  end # end function
  
# draw the ground plan
def   	draw_down_plan(p1,profile_slope_angle,ground_plan_angle,ground_plan_base,profile_rafter_width,profile_rafter_depth,main_hip_rafter_width,main_hip_rafter_depth)  
  

    model = Sketchup.active_model;
    # this enables undo of the whole operation as a unit
    model.start_operation "Draw Down";

    # create a new group in which we will draw our dimension entities
    model_ents = model.active_entities;
    group = model_ents.add_group;
    ents = group.entities;
    group.name="Draw Down";
    wplayer=model.layers.add("Draw Down");
    wplayer.visible=true;
    group.layer=wplayer;
    group.casts_shadows=false;
    group.receives_shadows=false;
	
       
  plan_angle = ground_plan_angle.degrees;
  profile_rafter_slope_angle = profile_slope_angle.degrees;
  
	puts "new Draw Down Technique" if $DEBUG_VERBOSE;
	   
		
	#p1 = [0,0,0]
	radius = ground_plan_base/3; #radius used for angluar text position
  @pts = []
  @pts[0] = [p1.x,p1.y,0]
  @pts[1] = [ p1.x + ground_plan_base,p1.y ,0 ]
  vec11 = @pts[1].vector_to @pts[0]
	vector = Geom::Vector3d.new 0,0,1
  vector2 = vector.normalize!

  profile_rafter_run = (ground_plan_base/2) * Math.tan( plan_angle);
  profile_rafter_length = profile_rafter_run / Math.cos( profile_rafter_slope_angle);
  roof_sheathing_angle = Math.atan(Math.tan(plan_angle) / Math.cos(profile_rafter_slope_angle));
  hip_angle = Math.atan(Math.tan(profile_rafter_slope_angle) * Math.sin(plan_angle));
  hip_rafter_backing_angle = Math.atan(Math.sin(hip_angle) / Math.tan(plan_angle));
 
  p2 = [p1.x + ground_plan_base,p1.y,0];   
  p2_mid = [p1.x + (ground_plan_base/2),p1.y,0];     
  p3 = [p1.x + (ground_plan_base/2),p1.y+profile_rafter_run,0];    
      
      # draw base lines
   ents.add_line [p1,p2];
   ents.add_line [p2,p3];
   ents.add_line [p3,p1];
   ents.add_line [p3,p2_mid];
   
   d = plan_angle;
   dd = plan_angle;
   
  offset_HROM = ((main_hip_rafter_width * Math.sin(d)) / Math.sin(Math::PI - d - dd)) * Math.cos(dd); # Hip_Rafter_Offset_Main_Perpendicular
	offset_HROA = ((main_hip_rafter_width * Math.sin(dd)) / Math.sin(Math::PI - d - dd)) * Math.cos(d); # Hip_Rafter_Offset_Adjacent_Perpendicular
	
  hip_shift_at_peak = offset_HROM * Math.tan(plan_angle);  
       
  offset_HRORM = offset_HROM / Math.sin(d); # # Hip_Rafter_Offset_Main_along eave
  offset_HRORA = offset_HROA / Math.sin(dd);# Hip_Rafter_Offset_Adhacent_along eave
  vertical_HROR = offset_HRORM * Math.tan(roof_sheathing_angle );
  development_line_depth = profile_rafter_depth * Math.tan(profile_rafter_slope_angle );
  development_line_offset = profile_rafter_depth * Math.tan(hip_rafter_backing_angle );
  backing_line_height = development_line_offset / Math.cos(roof_sheathing_angle );
  profile_rise = profile_rafter_run * Math.tan(profile_rafter_slope_angle );
  plumb_cut = profile_rafter_depth / Math.cos(profile_rafter_slope_angle );
  level_cut = profile_rafter_depth / Math.sin(profile_rafter_slope_angle );
  hip_level_cut = main_hip_rafter_depth / Math.sin(hip_angle );
  square_line_base = profile_rafter_depth / Math.tan(profile_rafter_slope_angle );
  square_line_base_sine = square_line_base * Math.sin(profile_rafter_slope_angle );
      
   p4 = [p1.x + offset_HRORM,p1.y,0];  
   x = (ground_plan_base/2) - offset_HRORM;
   y = x * Math.tan(plan_angle);
   p5 = [p1.x + (ground_plan_base/2), p1.y+y,0];  
   ents.add_line [p4,p5];
 
  p6 = [p1.x - offset_HRORM + ground_plan_base,p1.y,0];  
  ents.add_line [p6,p5];
  
  
  # draw face  for hips in plan view
      pts = []
      pts[0] = p1;
      pts[1] = p3
      pts[2] = p2;
      pts[3] = p6;
      pts[4] = p5;  
      pts[5] = p4;
      pts[6] = pts[0]
      new_rafter = ents.add_face pts
      status = new_rafter.back_material=[101, 156, 239]       
      new_rafter.material= Sketchup::Color.new(101, 156, 239)
      
 
 #center point of hip rafter on roof surface
  p7 = [p2_mid.x ,p1.y+profile_rafter_length,0];  
  ents.add_line [p7,p1];
  ents.add_line [p7,p2];
  ents.add_line [p7,p2_mid];
  #center point of hip rafter on roof surface minus the vertical height
  p8 = [p2_mid.x ,p1.y+profile_rafter_length-vertical_HROR,0];  
  ents.add_line [p8,p4];
  ents.add_line [p8,p6];
  
   # draw face  for hips on roof surface
      pts = []
      pts[0] = p1;
      pts[1] = p7
      pts[2] = p2;
      pts[3] = p6;
      pts[4] = p8;  
      pts[5] = p4;
      pts[6] = pts[0]
      new_rafter = ents.add_face pts
      status = new_rafter.back_material=[153, 107, 61]       
      new_rafter.material= Sketchup::Color.new(153, 107, 61)
  
  
  p9 = [p1.x+ground_plan_base+(ground_plan_base/2),p1.y,0];   
  p10 =[p9.x,p1.y-development_line_depth,0];  
  ents.add_line [p2,p9];
  ents.add_line [p10,p9];
  
  
  p11 = [p6.x, p1.y-development_line_depth,0];  
  ents.add_line [p11,p6];
  p12 = [p4.x, p1.y-development_line_depth,0];  
  ents.add_line [p12,p4];
  ents.add_line [p12,p10];
  
  p13 = [p8.x, p8.y-backing_line_height,0];  
  ents.add_line [p12,p13];
  ents.add_line [p11,p13];
  
  
  # draw face  for hip rafter internal backing on roof surface
      pts = []
      pts[0] = p12;
      pts[1] = p4
      pts[2] = p8;
      pts[3] = p6;
      pts[4] = p11;  
      pts[5] = p13;
      pts[6] = pts[0]
      new_rafter = ents.add_face pts
     status = new_rafter.back_material=[30, 150, 30]       
    new_rafter.material= Sketchup::Color.new(30, 150, 30)
  
  
 
  p14 =  [p9.x, p3.y,0]; 
  ents.add_line [p9,p14];
  
  p15 =  [p14.x+profile_rise, p14.y,0]; 
  ents.add_line [p15,p14];
  ents.add_line [p15,p9];
  
  p16 =  [p15.x-plumb_cut, p15.y,0]; 
  p17 =  [p9.x-plumb_cut, p9.y,0]; 
   # profile rafter depth line
   ents.add_line [p16,p17];
   
      
   p18 =  [p9.x, p9.y+square_line_base,0]; 
   p19 =  [p1.x, p9.y+square_line_base,0]; 
   #seat line on Roof Surface
   ents.add_line [p18,p19];
   
  line_base = square_line_base_sine / Math.tan(profile_rafter_slope_angle );
  tail_sine_y = profile_rafter_depth * Math.sin(profile_rafter_slope_angle );
  tail_sine_x =  tail_sine_y * Math.tan(profile_rafter_slope_angle );
  tail_radius = tail_sine_y / Math.cos(profile_rafter_slope_angle );
  p20 =  [p9.x-tail_sine_x, p9.y-tail_sine_y,0]; 
  p20a =  [p9.x, p9.y-tail_radius,0]; 
   
  ents.add_line [p17,p20];
  ents.add_line [p9,p20];
     
  p18a =  [p9.x+square_line_base_sine, p9.y+line_base,0]; 
  p18b =  [p9.x, p9.y+level_cut,0]; 
       
   ents.add_line [p18a,p18b];
   
   
   # draw face  for profile rafter
      pts = []
      pts[0] = p9;
      pts[1] = p15
      pts[2] = p16;
      pts[3] = p17;
      pts[4] = p9;  
      new_rafter = ents.add_face pts
      status = new_rafter.back_material=[153, 107, 61]       
      new_rafter.material= Sketchup::Color.new(153, 107, 61)
      
       # draw face for profile rafter tail
      pts = []
      pts[0] = p9;
      pts[1] = p17
      pts[2] = p20;
      pts[3] = p9;
      new_rafter = ents.add_face pts
     status = new_rafter.back_material=[30, 150, 30]       
    new_rafter.material= Sketchup::Color.new(30, 150, 30)
    
   vec1 = p9.vector_to p18;
   vec2 = p9.vector_to p18a;
   vec1.length = p9.distance p18;
   vec2.length = p9.distance p18a;   
   angle = vec1.angle_between vec2;
   normal = (vec1 * vec2).normalize;    
   arc = ents.add_arc p9, vec1, normal, square_line_base, 0, angle, 30
     
     
   vec1 = p9.vector_to p20;
   vec2 = p9.vector_to p20a;
   vec1.length = p9.distance p20;
   vec2.length = p9.distance p20a;   
   angle = vec1.angle_between vec2;
   normal = (vec1 * vec2).normalize;    
   arc = ents.add_arc p9, vec1, normal, tail_radius, 0, angle, 30
  
  p21 =  [p9.x, p9.y+level_cut,0];     
  p21a =  [p1.x, p9.y+level_cut,0];  
  # seat line plan view
  ents.add_line [p21,p21a];
  
  
  # hip lines
  hip_base_angle = Math::PI/2 - plan_angle;
  hip_base_x = ((ground_plan_base/2) *Math.cos( hip_base_angle));
  hip_base_y = ((ground_plan_base/2) *Math.sin( hip_base_angle));
  
   pt_hip =  [p2.x + hip_base_x,p2.y + hip_base_y,0];  
   pt_hip_parallel=   [p3.x + hip_base_x,p3.y + hip_base_y,0];    
    
  ents.add_line [p2,pt_hip];
  ents.add_line [p3,pt_hip_parallel];
  ents.add_line [pt_hip,pt_hip_parallel];
   
   
  hip_peak_x =  profile_rise*Math.cos(hip_base_angle);
  hip_peak_y = profile_rise*Math.sin(hip_base_angle);
  pt_hip_peak=   [pt_hip_parallel.x + hip_peak_x,pt_hip_parallel.y + hip_peak_y,0];    

  ents.add_line [pt_hip,pt_hip_peak];
  ents.add_line [pt_hip_peak,pt_hip_parallel];

  hip_shift = offset_HRORM   * Math.sin(hip_base_angle);
  hip_shift_x = hip_shift *Math.cos(Math::PI - plan_angle);
  hip_shift_y = hip_shift *Math.sin(Math::PI -  plan_angle);
  
   hip_shift_rise = hip_shift  * Math.tan(hip_angle);   
  
   hip_shift_point =  [pt_hip.x + hip_shift_x,pt_hip.y + hip_shift_y,0];  

   ents.add_line [hip_shift_point,p6];
   
   rotate_angle = Math::PI/2 + (Math::PI - plan_angle);
   
  hip_shift_peak_x =  hip_shift_rise *Math.cos(rotate_angle);
  hip_shift_peak_y = hip_shift_rise *Math.sin(rotate_angle);
  hip_shift_peak_point =  [pt_hip_peak.x + hip_shift_peak_x,pt_hip_peak.y + hip_shift_peak_y,0];  
  
  ents.add_line [hip_shift_point,hip_shift_peak_point];
  
   
  rotate_angle = Math::PI - plan_angle;
  hip_level_cut_x =  hip_level_cut *Math.cos(rotate_angle);
  hip_level_cut_y = hip_level_cut *Math.sin(rotate_angle);
  hip_level_cut_point =  [pt_hip.x + hip_level_cut_x,pt_hip.y + hip_level_cut_y,0];  
  y = hip_level_cut_point.y - p2.y;
  x = y / Math.tan(hip_base_angle);
  hip_claw_point_plan =  [hip_level_cut_point.x - x,p1.y,0]; 
   
  ents.add_line [hip_claw_point_plan ,hip_level_cut_point];
    
  hip_level_cut_rise = hip_level_cut  * Math.tan(hip_angle);  
  rotate_angle = Math::PI/2 + (Math::PI - plan_angle);
  hip_depth_peak_x =  hip_level_cut_rise *Math.cos(rotate_angle);
  hip_depth_peak_y = hip_level_cut_rise *Math.sin(rotate_angle);
  hip_depth_peak_point =  [pt_hip_peak.x + hip_depth_peak_x,pt_hip_peak.y + hip_depth_peak_y,0];  
  
  ents.add_line [hip_depth_peak_point,hip_level_cut_point];
  
   
    # draw face  for hip rafter profile
      pts = []
      pts[0] = pt_hip;
      pts[1] = pt_hip_peak;
      pts[2] = hip_depth_peak_point;
      pts[3] = hip_level_cut_point;
      pts[4] = pt_hip; 
      new_rafter = ents.add_face pts
      status = new_rafter.back_material=[153, 107, 61]       
      new_rafter.material= Sketchup::Color.new(153, 107, 61)
      
  
  
  
 y = hip_level_cut_point .y - p21.y;
 x = y / Math.tan(hip_base_angle);
 height = p21.y - p18.y;
 intersecting_pt = [hip_level_cut_point .x - x, p21.y,0];
 
 claw_pt_1 = [intersecting_pt.x,intersecting_pt.y - height,0];
 # vertical intersecting line
 ents.add_line [intersecting_pt,claw_pt_1];
 rise =claw_pt_1.y;
 base = rise / Math.tan(roof_sheathing_angle);

 claw_pt_2 = [intersecting_pt.x + base,p2.y,0];
 claw_base_run = claw_pt_2.x - p2_mid.x; 
 base_rise = claw_base_run  *  Math.tan(roof_sheathing_angle);
 claw_pt_3 = [p2_mid.x,p2_mid.y + base_rise,0];
 claw_pt_4 = [p2_mid.x-claw_base_run,p2_mid.y,0];
 
 # hip claw lines on roof surface
 ents.add_line [claw_pt_2,claw_pt_3];
 ents.add_line [claw_pt_4,claw_pt_3];
 
 claw_base_run = hip_claw_point_plan.x - p2_mid.x; 
 base_rise = claw_base_run  *  Math.tan(roof_sheathing_angle);
 claw_pt_5 = [p2_mid.x,p2_mid.y + base_rise,0];
 claw_pt_6 = [p2_mid.x-claw_base_run,p2_mid.y,0];
 
 # hip claw lines on roof surface
 ents.add_line [hip_claw_point_plan,claw_pt_5];
 ents.add_line [claw_pt_5,claw_pt_6];

    Sketchup.active_model.rendering_options["ModelTransparency"]=false;
    view = Sketchup.active_model.active_view ;
    #view.field_of_view = 30;
    #result = Sketchup.send_action 10501; # set view to Top
    view.zoom_extents; 

    group.to_component;
    model.commit_operation;
    
   
end # end function
   

   
   
end # class DrawDownTool 

end #module ROOFFRAMING

file_loaded("RoofFraming/drawdown.rb")
