# Copyright 2012 Sim Ayers
# version 1.0
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------


module ROOFFRAMING

class PrismTool < TetrahedronTool

$major_pitch = 8.0;
$minor_pitch = 8.0;
$deck_angle = 90.0;
$eave_length = 12.0;
$show_angles = "no";
$folded = "yes";
$dihedral = "yes";


def initialize
       
    $major_pitch = 6.0 if not $major_pitch;
    $minor_pitch = 8.0 if not $minor_pitch;
    $deck_angle = 90.0 if not $deck_angle;
    $eave_length = 12.0 if not $eave_length;
    $show_angles = "no" if not $show_angles;
    $folded = "yes" if not $folded;
    $dihedral = "yes" if not $dihedral;
   

    # First prompt for the dimensions.  This is done using the inputbox method
        
    # The first step is to create some arrays which contain the prompts
    # and default values.
    prompts = ["Major Pitch","Minor Pitch","Eave Angle","Eave Length","Dim Angles","Folded","Show Dihedral Angle"]
    #default values
    values = [$major_pitch,$minor_pitch,$deck_angle,$eave_length,$show_angles,$folded,$dihedral]
    
    popups = [nil, nil, nil, nil,"yes|no","yes|no|Both","yes|no","no|yes"]
    
    # Now display the inputbox
    results = inputbox prompts, values,popups, "Prism Roof Framing Kernel";

    return if not results # This means that the user cancelled the operation
    $major_pitch,$minor_pitch,$deck_angle,$eave_length,$show_angles,$folded,$dihedral = results
    
    if ($major_pitch <= 0.01)
     	UI.messagebox("Major Pitch can not be 0.")
     	return
    end
     if ($minor_pitch <= 0.01)
     	UI.messagebox("Minor Pitch can not be 0.")
     	return
    end
    
    if ($eave_length <= 0.0 )
     	UI.messagebox("Eave Length can not be 0.")
     	return
    end
    if ($deck_angle <= 0.0 )
     	UI.messagebox("Eave Angle can not be 0.")
     	return
    end
    if ($deck_angle >= 179.0 )
     	UI.messagebox("Eave Angle can not be greater 180.")
     	return
    end
    
    show = true
    show = false if($show_angles == "no")
    both = false
    both =  true if( $folded == "Both")
    show_dihedral = false
    show_dihedral = true if($dihedral =="yes");
     p1 = [0,0,0]
 
 if($folded == "no" || $folded == "Backing Angle" || $folded == "Both")
    	draw_prism_unfolded(p1,$major_pitch,$minor_pitch,$deck_angle,$eave_length,show,both,show_dihedral);
      show_roof_angles = false;
    	draw_prism(p1,$major_pitch,$minor_pitch,$deck_angle,$eave_length,show,both,show_dihedral)   if $folded == "Both";
    else
    	draw_prism(p1,$major_pitch,$minor_pitch,$deck_angle,$eave_length,show,both,show_dihedral);  
    end
    
end


 
# draw the plan angle dimensions unfolded
def draw_prism_unfolded(p1,major_pitch,minor_pitch,deck_angle,eave_length,show_angles,both,show_dihedral)  

    model = Sketchup.active_model
    # this enables undo of the whole operation as a unit
    model.start_operation "Prism Plane"

    # create a new group in which we will draw our dimension entities
    model_ents = model.active_entities
    group = model_ents.add_group
    ents = group.entities
    group.name="Prism"
    wplayer=model.layers.add("Prism Plane")
    wplayer.visible=true
    group.layer=wplayer
    group.casts_shadows=false
    group.receives_shadows=false
	
       
    eave_angle = deck_angle.degrees
    major_side = eave_length

    major_pitch_angle = Math.atan(major_pitch / 12)      
    minor_pitch_angle = Math.atan(minor_pitch / 12)  
    
    plan_angles = Array.new(2);
    plan_angles = find_plan_angle(major_pitch_angle,minor_pitch_angle,eave_angle)   
    major_plan_angle = plan_angles[0]; 
    minor_plan_angle = plan_angles[1]; 
    
	
	hip_angle = Math.atan(Math.tan(major_pitch_angle)* Math.sin(major_plan_angle))
	hip_run_length = major_side / Math.cos(major_plan_angle)	
	major_run = major_side * Math.tan(major_plan_angle)
	major_rise = major_run * Math.tan(major_pitch_angle)
	minor_run = hip_run_length * Math.sin(minor_plan_angle)
	minor_eave = hip_run_length * Math.cos(minor_plan_angle)
	minor_side = minor_eave

	puts "new dimensions " if $DEBUG_VERBOSE;
	
	
	eave_angle_d = angle_sprintf(eave_angle);
	puts "eave angle = " + eave_angle_d if $DEBUG_VERBOSE;
	major_d = angle_sprintf(major_plan_angle);
	minor_d = angle_sprintf(minor_plan_angle);
	puts "major d = " + major_d if $DEBUG_VERBOSE;
	puts "minor d = " + minor_d if $DEBUG_VERBOSE;

	major_pitch_angle_d = angle_sprintf(major_pitch_angle);
	minor_pitch_angle_d = angle_sprintf(minor_pitch_angle);
	puts "major pitch angle = " + major_pitch_angle_d if $DEBUG_VERBOSE;
	puts "minor pitch angle = " + minor_pitch_angle_d if $DEBUG_VERBOSE;
	hip_angle_d = angle_sprintf(hip_angle);
	puts "hip rafter angle = " + hip_angle_d if $DEBUG_VERBOSE;
	puts "hip run = " + length_sprintf(hip_run_length) if $DEBUG_VERBOSE;
	
  m = length_sprintf(major_side)
  puts "major_side = " + m if $DEBUG_VERBOSE;
  m = length_sprintf(minor_side)
  puts "minor_side = " + m if $DEBUG_VERBOSE;
    
		
	#p1 = [0,0,0]
	radius = eave_length/3; #radius used for angluar text position
  @pts = []
  @pts[0] = [p1.x,p1.y,0]
  @pts[1] = [ p1.x + eave_length,p1.y ,0 ]
  vec11 = @pts[1].vector_to @pts[0]
	vector = Geom::Vector3d.new 0,0,1
  vector2 = vector.normalize!

        
    dd = major_plan_angle; # plan angle
    ss = major_pitch_angle; # roof pitch angle
    r1 = Math.atan(Math.tan(ss) * Math.sin(dd)); # hip rafter angle
    ee = Math.atan(Math.cos(ss) / Math.tan(dd)); # jack rafter angle
    bb = Math.atan(Math.sin(r1) / Math.tan(dd)); # hip rafter backing angle
    run = (eave_length * Math.tan(dd)); # common rafter run
    rise = run * Math.tan(ss); # major common rafter rise
    hip_run = (eave_length / Math.cos(dd))
    hip_rafter = hip_run / Math.cos(r1)
    common_rafter = run / Math.cos(ss)
    alitude = hip_run * Math.sin(r1)
    r2 = r1 + dd
    d2 = 90.degrees - dd
    backing_leg = alitude + hip_run
    backing_run = run / Math.tan(d2);
    
    r5m = Math.atan(Math.tan(r1) * Math.cos(major_plan_angle )); 
    r5m_rise = eave_length * Math.tan(r5m);
    r5a = Math.atan(Math.tan(r1) * Math.cos(minor_plan_angle )); 
    r5a_rise = minor_eave * Math.tan(r5a); 
    
      # draw major plan angle
    pts = []
    pts[0] = [p1.x,p1.y,0]
    pts[1] = [ p1.x + eave_length,p1.y ,0 ]
    pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles && !both ;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles && !both ;
    
    # draw prism angle #1
    pts = []
    pts[0] = [p1.x,p1.y,0]
    pts[1] = [ p1.x + eave_length,p1.y ,0 ]
    pts[2] = [ p1.x + eave_length ,p1.y - r5m_rise, 0]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles && !both ;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles && !both ;
    
    # draw prism angle #2
    pts = []
    pts[0] = [ p1.x + eave_length,p1.y ,0 ]
    pts[1] = [ p1.x + eave_length + r5m_rise,p1.y ,0 ]
    pts[2] = [ p1.x + eave_length+rise ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles && !both ;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles && !both ;
    
    # draw common rafter pitch
    pts = []
    pts[0] = [ p1.x + eave_length,p1.y ,0 ]
    pts[1] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[2] = [ p1.x + eave_length+rise ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;

   # hip rafter
    pts = []
    pts[0] = [p1.x,p1.y,0]
    pts[1] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[2] = [ p1.x + (hip_rafter * Math.cos(r2)) ,p1.y + (hip_rafter * Math.sin(r2)) , 0]
    pts[3] =  pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;

   

	
    minor_rotate_angle = eave_angle;

    #draw minor plan angle
    m_pts = []
    m_pts[0] = [p1.x,p1.y,0]
    m_pts[1] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0]
    m_pts[2] = [p1.x + major_side ,p1.y+ major_run,0]
    m_pts[3] = m_pts[0]
    dimension_angle(m_pts[1],m_pts[0],m_pts[2],radius,ents) if show_angles;
    dimension_angle(m_pts[1],m_pts[2],m_pts[0],radius,ents) if show_angles;
    ents.add_face m_pts   
    
    # show eave angle
     m_pts = []
     m_pts[0] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0];
     m_pts[1] = [p1.x,p1.y,0];
     m_pts[2] = [ p1.x + eave_length,p1.y ,0 ];
     dimension_angle(m_pts[0],m_pts[1],m_pts[2],radius*2,ents) if show_angles; # show eave angle
      
    #draw minor run & rafter
    m_pts = []
    m_pts[0] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0];
    m_pts[1] = [p1.x + major_side ,p1.y+ major_run,0];
    m_pts[2] = [p1.x + major_side + rise,p1.y+ major_run,0]; # fix rise rotated
    m_pts[3] = m_pts[0];
    rotated_pt = rotate_point(rise,p1.x + eave_length,p1.y+ major_run,minor_rotate_angle)
    m_pts[2] = rotated_pt;
    fix_point = rotated_pt;
    test_angle = dimension_angle_test(m_pts[1],m_pts[0],m_pts[2]);
    dimension_angle(m_pts[1],m_pts[0],m_pts[2],radius,ents) if show_angles;
    dimension_angle(m_pts[1],m_pts[2],m_pts[0],radius,ents) if show_angles;
    ents.add_face m_pts        
    
    minor_rafter = m_pts[0].distance m_pts[2];
         
    # draw prism angle #1
    m_pts = []
    m_pts[0] = [p1.x,p1.y,0]
    m_pts[1] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0];
    m_pts[2] = [p1.x + major_side ,p1.y + major_run,r5a_rise];
    m_pts[3] = m_pts[0];
    rotate_angle = (eave_angle - 90.degrees) + 360.degrees;
    rotated_pt = rotate_point(-r5a_rise ,m_pts[1].x ,m_pts[1].y ,rotate_angle);
    m_pts[2] = rotated_pt

    dimension_angle(m_pts[1],m_pts[0],m_pts[2],radius,ents) if show_angles;
    dimension_angle(m_pts[1],m_pts[2],m_pts[0],radius,ents) if show_angles;
    ents.add_face m_pts;   

     # draw prism angle #2
      m_pts = []
    m_pts[0] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0];
    m_pts[1] = fix_point; 
    m_pts[2] = [p1.x + major_side + rise,p1.y+ major_run,0]; # fix rise rotated
    m_pts[3] = m_pts[0];
    rotated_pt = rotate_point(r5a_rise,m_pts[0].x,m_pts[0].y,minor_rotate_angle);
    m_pts[2] = rotated_pt;
    dimension_angle(m_pts[1],m_pts[0],m_pts[2],radius,ents) if show_angles;
    dimension_angle(m_pts[1],m_pts[2],m_pts[0],radius,ents) if show_angles;
    ents.add_face m_pts;        

  
    minor_a = angle_degrees(minor_pitch_angle);
    puts "test_angle = " + test_angle.to_s if $DEBUG_VERBOSE;
    puts "minor_a = " + minor_a.to_s if $DEBUG_VERBOSE;
       
    UI.messagebox("Minor Rafter Pitch Angle: " + minor_a.to_s + "\nMinor Dim Angle: " + test_angle.to_s + "\nWarning\nThis Roof is Unconstructable") if(test_angle != minor_a);
        
    Sketchup.active_model.rendering_options["ModelTransparency"]=true 
    view = Sketchup.active_model.active_view 
    view.field_of_view = 30
    #result = Sketchup.send_action 10501; # set view to Top
    view.zoom_extents 

    group.to_component 
    model.commit_operation
    
   
 end # end function
 
 # draw the plan angle dimensions folded
def draw_prism(p1,major_pitch,minor_pitch,deck_angle,eave_length,show_angles,both,show_dihedral)  

    model = Sketchup.active_model
    # this enables undo of the whole operation as a unit
    model.start_operation "Prism Plane"

    # create a new group in which we will draw our dimension entities
    model_ents = model.active_entities
    group = model_ents.add_group
    ents = group.entities
    group.name="Prism"
    wplayer=model.layers.add("Prism Plane")
    wplayer.visible=true
    group.layer=wplayer
    group.casts_shadows=false
    group.receives_shadows=false
	
       
    eave_angle = deck_angle.degrees
    major_side = eave_length

    major_pitch_angle = Math.atan(major_pitch / 12)      
    minor_pitch_angle = Math.atan(minor_pitch / 12)  
    
    plan_angles = Array.new(2);
    plan_angles = find_plan_angle(major_pitch_angle,minor_pitch_angle,eave_angle)   
    major_plan_angle = plan_angles[0]; 
    minor_plan_angle = plan_angles[1]; 

	
	hip_angle = Math.atan(Math.tan(major_pitch_angle)* Math.sin(major_plan_angle))
	hip_run_length = major_side / Math.cos(major_plan_angle)	
	major_run = major_side * Math.tan(major_plan_angle)
	major_rise = major_run * Math.tan(major_pitch_angle)
	minor_run = hip_run_length * Math.sin(minor_plan_angle)
	minor_eave = hip_run_length * Math.cos(minor_plan_angle)
	minor_side = minor_eave

	puts "new dimensions " if $DEBUG_VERBOSE;
	
	
	eave_angle_d = angle_sprintf(eave_angle);
	puts "eave angle = " + eave_angle_d if $DEBUG_VERBOSE;
	major_d = angle_sprintf(major_plan_angle);
	minor_d = angle_sprintf(minor_plan_angle);
	puts "major d = " + major_d if $DEBUG_VERBOSE;
	puts "minor d = " + minor_d if $DEBUG_VERBOSE;

	major_pitch_angle_d = angle_sprintf(major_pitch_angle);
	minor_pitch_angle_d = angle_sprintf(minor_pitch_angle);
	puts "major pitch angle = " + major_pitch_angle_d if $DEBUG_VERBOSE;
	puts "minor pitch angle = " + minor_pitch_angle_d if $DEBUG_VERBOSE;
	hip_angle_d = angle_sprintf(hip_angle);
	puts "hip rafter angle = " + hip_angle_d if $DEBUG_VERBOSE;
	puts "hip run = " + length_sprintf(hip_run_length) if $DEBUG_VERBOSE;
	
  m = length_sprintf(major_side)
  puts "major_side = " + m if $DEBUG_VERBOSE;
  m = length_sprintf(minor_side)
  puts "minor_side = " + m if $DEBUG_VERBOSE;
    
	
	#p1 = [0,0,0]
	radius = eave_length/3; #radius used for angluar text position
  @pts = []
  @pts[0] = [p1.x,p1.y,0]
  @pts[1] = [ p1.x + eave_length,p1.y ,0 ]
  vec11 = @pts[1].vector_to @pts[0]
	vector = Geom::Vector3d.new 0,0,1
  vector2 = vector.normalize!
	
        
    dd = major_plan_angle; # plan angle
    ss = major_pitch_angle; # roof pitch angle
    r1 = Math.atan(Math.tan(ss) * Math.sin(dd)); # hip rafter angle
    ee = Math.atan(Math.cos(ss) / Math.tan(dd)); # jack rafter angle
    bb = Math.atan(Math.sin(r1) / Math.tan(dd)); # hip rafter backing angle
    run = (eave_length * Math.tan(dd)); # common rafter run
    rise = run * Math.tan(ss); # major common rafter rise
    
    r5m = Math.atan(Math.tan(r1) * Math.cos(major_plan_angle )); 
    r5m_rise = eave_length * Math.tan(r5m);
     r5a = Math.atan(Math.tan(r1) * Math.cos(minor_plan_angle )); 
    r5a_rise = minor_eave * Math.tan(r5a);
   
      # draw major plan angle
    pts = []
    pts[0] = [p1.x,p1.y,0]
    pts[1] = [ p1.x + eave_length,p1.y ,0 ]
    pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles && !both ;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles && !both ;

    # draw common rafter pitch
    pts = []
    pts[0] = [ p1.x + eave_length,p1.y ,0 ]
    pts[1] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , rise]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
    
    
    # draw r5m prism 1
    pts = []
    pts[0] = [ p1.x,p1.y ,0 ]
    pts[1] = [ p1.x + eave_length ,p1.y , 0]
    pts[2] = [ p1.x + eave_length ,p1.y , r5m_rise]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;

 # draw r5pm prism 2
    pts = []
    pts[0] = [ p1.x + eave_length ,p1.y , 0]
    pts[1] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , rise]
    pts[2] = [ p1.x + eave_length ,p1.y , r5m_rise]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
    
    # draw hip rafter
    pts = []
    pts[0] = [p1.x,p1.y,0]
    pts[1] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , 0]
    pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , rise]
    pts[3] =  pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
    
    # draw major prism sheathing
    pts = []
    pts[0] = [p1.x,p1.y,0]
    pts[1] = [ p1.x + eave_length ,p1.y , r5m_rise]
    pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , rise]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;
       
    # draw roof sheathing
    pts = []
    pts[0] = [p1.x,p1.y,0]
    pts[1] = [ p1.x + eave_length ,p1.y , 0]
    pts[2] = [ p1.x + eave_length ,p1.y + (eave_length * Math.tan(dd)) , rise]
    pts[3] = pts[0]
    new_rafter = ents.add_face pts
    dimension_angle(pts[1],pts[0],pts[2],radius, ents ) if show_angles;
    dimension_angle(pts[1],pts[2],pts[3],radius, ents ) if show_angles;

    minor_rotate_angle = eave_angle;

    #draw minor plan angle
    m_pts = []
    m_pts[0] = [p1.x,p1.y,0]
    m_pts[1] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0]
    m_pts[2] = [p1.x + major_side ,p1.y+ major_run,0]
    m_pts[3] = m_pts[0]
    dimension_angle(m_pts[1],m_pts[0],m_pts[2],radius,ents) if show_angles;
    dimension_angle(m_pts[1],m_pts[2],m_pts[0],radius,ents) if show_angles;
    ents.add_face m_pts        
     
     m_pts = []
     m_pts[0] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0]
     m_pts[1] = [p1.x,p1.y,0]
     m_pts[2] = [ p1.x + eave_length,p1.y ,0 ]
     dimension_angle(m_pts[0],m_pts[1],m_pts[2],radius*2,ents) if show_angles; # show eave angle
      
    #draw minor run & rafter
    m_pts = []
    m_pts[0] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0]
    m_pts[1] = [p1.x + major_side ,p1.y+ major_run,0]
    m_pts[2] = [p1.x + major_side ,p1.y + major_run,major_rise]
    m_pts[3] = m_pts[0]
    test_angle = dimension_angle_test(m_pts[1],m_pts[0],m_pts[2]);
    dimension_angle(m_pts[1],m_pts[0],m_pts[2],radius,ents) if show_angles;
    dimension_angle(m_pts[1],m_pts[2],m_pts[0],radius,ents) if show_angles;
    ents.add_face m_pts        
 
    
    
    #draw r5a prism
    m_pts = []
    m_pts[0] = [p1.x ,p1.y,0]
    m_pts[1] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0]
    m_pts[2] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),r5a_rise]
    m_pts[3] = m_pts[0]
    ents.add_face m_pts    
    dimension_angle(m_pts[1],m_pts[0],m_pts[2],radius,ents) if show_angles;
    dimension_angle(m_pts[1],m_pts[2],m_pts[0],radius,ents) if show_angles;
        
  #draw r5pa prism
    m_pts = []
    m_pts[0] =  [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0]
    m_pts[1] =[p1.x + major_side ,p1.y + major_run,major_rise]
    m_pts[2] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),r5a_rise]
    m_pts[3] = m_pts[0]
    ents.add_face m_pts    
    dimension_angle(m_pts[1],m_pts[0],m_pts[2],radius,ents) if show_angles;
    dimension_angle(m_pts[1],m_pts[2],m_pts[0],radius,ents) if show_angles;
     
     #draw minor prism sheathing
    m_pts = []
    m_pts[0] = [p1.x,p1.y,0]
    m_pts[1] =  [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),r5a_rise]
    m_pts[2] = [p1.x + major_side ,p1.y + major_run,major_rise]
    m_pts[3] = m_pts[0]
    dimension_angle(m_pts[1],m_pts[0],m_pts[2],radius,ents) if show_angles;
    dimension_angle(m_pts[1],m_pts[2],m_pts[0],radius,ents) if show_angles;
    ents.add_face m_pts      
    
     #draw minor roof sheathing
    m_pts = []
    m_pts[0] = [p1.x,p1.y,0]
    m_pts[1] = [p1.x + (minor_side * Math.cos(minor_rotate_angle)),p1.y + ( minor_side * Math.sin(minor_rotate_angle)),0]
    m_pts[2] = [p1.x + major_side ,p1.y + major_run,major_rise]
    m_pts[3] = m_pts[0]
    dimension_angle(m_pts[1],m_pts[0],m_pts[2],radius,ents) if show_angles;
    dimension_angle(m_pts[1],m_pts[2],m_pts[0],radius,ents) if show_angles;
    ents.add_face m_pts        
  
  
    pts = draw_dihedral_angle_folded(dd,ss,r1,bb,hip_run_length,eave_length,p1.x,p1.y,ents,show_angles,radius,false) if show_dihedral;
    draw_dihedral_angle_folded_minor(major_plan_angle,eave_angle,eave_length,p1.x,p1.y,ents,show_angles,radius,pts) if show_dihedral;
    
    minor_a = angle_degrees(minor_pitch_angle);
    puts "test_angle = " + test_angle.to_s if $DEBUG_VERBOSE;
    puts "minor_a = " + minor_a.to_s if $DEBUG_VERBOSE;
       
    UI.messagebox("Minor Rafter Pitch Angle: " + minor_a.to_s + "\nMinor Dim Angle: " + test_angle.to_s + "\nWarning\nThis Roof is Unconstructable") if(test_angle != minor_a);
        
    
    Sketchup.active_model.rendering_options["ModelTransparency"]=true 
    view = Sketchup.active_model.active_view 
    view.field_of_view = 30
    #result = Sketchup.send_action 10501; # set view to Top
    view.zoom_extents 

    group.to_component 
    model.commit_operation
    
   
end # end function
   
   # draw a dihedral angle folded for the minor pitch angle
def draw_dihedral_angle_folded_minor(dd,eave_angle,eave_length,originx,originy,ents,show_angles,radius,pts)
    
    angle_a = eave_angle;
    angle_b = 90.degrees - dd;
    angle_c = 180.degrees - angle_a - angle_b;
    #law of sines
    side_c = eave_length;
    side_a = get_triangle_ASA(angle_b,side_c,angle_a);
      
    leg_rise = side_a * Math.sin(angle_b);
    leg_run = side_a * Math.cos(angle_b);
    puts "side_a " + side_a.to_s if $DEBUG_VERBOSE; 
    puts "leg_run " + leg_run.to_s if $DEBUG_VERBOSE; 
    puts "leg_rise " + leg_rise.to_s if $DEBUG_VERBOSE; 
      
    pts1 = pts[2]
    pts3 = [originx+eave_length-leg_run,originy+leg_rise,0]
    new_rafter = ents.add_line pts1, pts3
    
    pts4 = pts3
    pts2 = pts[0]
    new_rafter = ents.add_line pts4, pts2
 
    dimension_angle(pts1,pts3,pts2,radius, ents ) if show_angles;   
     
   
end 
   
   
end # class PrismTool

end #module ROOFFRAMING

file_loaded("RoofFraming/prism.rb")
